@@ -1,8 +1,8 @@
 /*
-** Astrolog (Version 7.60) File: astrolog.h
+** Astrolog (Version 7.80) File: astrolog.h
 **
 ** IMPORTANT NOTICE: Astrolog and all chart display routines and anything
-** not enumerated below used in this program are Copyright (C) 1991-2023 by
+** not enumerated below used in this program are Copyright (C) 1991-2025 by
 ** Walter D. Pullen (Astara@msn.com, http://www.astrolog.org/astrolog.htm).
 ** Permission is granted to freely use, modify, and distribute these
 ** routines provided these credits and notices remain unmodified with any
@@ -10,8 +10,8 @@
 **
 ** The main ephemeris databases and calculation routines are from the
 ** library SWISS EPHEMERIS and are programmed and copyright 1997-2008 by
-** Astrodienst AG. The use of that source code is subject to the license for
-** Swiss Ephemeris Free Edition, available at http://www.astro.com/swisseph.
+** Astrodienst AG. Use of that source code is subject to license for Swiss
+** Ephemeris Free Edition at https://www.astro.com/swisseph/swephinfo_e.htm.
 ** This copyright notice must not be changed or removed by any user of this
 ** program.
 **
@@ -48,7 +48,7 @@
 ** Initial programming 8/28-30/1991.
 ** X Window graphics initially programmed 10/23-29/1991.
 ** PostScript graphics initially programmed 11/29-30/1992.
-** Last code change made 4/8/2023.
+** Last code change made 6/19/2025.
 */
 
 /*
@@ -180,8 +180,8 @@
   // as accessed with the -b switch. This is normally the default dir above
   // but may be changed to be somewhere else.
 
-#define DEFAULT_LONG DM(122,20) 
-#define DEFAULT_LAT  DM(47, 36)
+#define DEFAULT_LONG DMS(122, 19, 55)
+#define DEFAULT_LAT  DMS(47,  36, 22)
   // Change numbers to longitude and latitude of your current location. Use
   // negative values for eastern or southern degrees.
 
@@ -302,10 +302,15 @@
 #ifdef GRAPH
 #define SWISSGRAPH
 #endif
-#endif
+#endif // SWISS
 #ifdef PLACALC
 #define EPHEM
 #endif
+#ifdef GRAPH
+#ifdef CONSTEL
+#define CONSTELGRAPH
+#endif
+#endif // GRAPH
 
 #define _CRT_SECURE_NO_DEPRECATE
 #define _CRT_NONSTDC_NO_DEPRECATE
@@ -370,6 +375,15 @@
 #ifdef SWISS
 #endif // SWISS
 
+#ifdef PC
+#define sprintf2 snprintf
+#define S(sz) (sz), (int)sizeof(sz)
+#define SO(pch, sz) (pch), (sizeof(sz) - ((pch) - (sz)))
+#else
+#define sprintf2 sprintf
+#define S(sz) (sz)
+#define SO(pch, sz) (pch)
+#endif
 
 /*
 ** Make sure only legal combinations of options are active.
@@ -467,9 +481,9 @@
 #define fTrue  TRUE
 
 #define szAppNameCore "Astrolog"
-#define szVersionCore "7.60"
-#define szVerCore     "760"
-#define szDateCore    "April 2023"
+#define szVersionCore "7.80"
+#define szVerCore     "780"
+#define szDateCore    "June 2025"
 #define szAddressCore \
   "Astara@msn.com - http://www.astrolog.org/astrolog.htm"
 #define szNowCore     "now"
@@ -478,25 +492,28 @@
 #define szNulCore     "nul"
 #define szObjUnknown  "???"
 
-#define cchSzDef   80
-#define cchSzMax   255
-#define cchSzLine  (cchSzMax*4)
-#define dwCanary   0x87654321
-#define nDegMax    360
-#define nDegHalf   180
-#define nMillion   1000000
-#define nLarge     9999999
-#define nNegative  -1000
-#define yeaJ2G     1582
-#define monJ2G     mOct
-#define dayJ2G1    4
-#define dayJ2G2    15
-#define zonLMT     24.0
-#define zonLAT     23.0
-#define dstAuto    24.0
-#define rStarLite  -1.46
-#define rStarSpan  7.0
-#define rStarNot   999.99
+#define cchSzDef  80
+#define cchSzMax  255
+#define cchSzLine (cchSzMax*4)
+#define dwCanary  0x87654321
+#define nDegMax   360
+#define nDegHalf  180
+#define nMillion  1000000
+#define nLarge    9999999
+#define nNegative -1000
+#define yeaJ2G    1582
+#define monJ2G    mOct
+#define dayJ2G1   4
+#define dayJ2G2   15
+#define zonLMT    24.0
+#define zonLAT    23.0
+#define dstAuto   24.0
+#define rStarLite -1.46
+#define rStarSpan 7.0
+#define rStarNot  999.99
+#define iAnimDay  4
+#define iAnimNow  10
+#define iFillMax  255
 
 // Math and conversion constants
 
@@ -540,7 +557,6 @@
 #define chDegL   '\260' // 176 (Latin-1)
 #define chDegI   '\370' // 248 (IBM)
 #define chRet    'R'
-#define chRet2   'r'
 #define chSep    ','
 #define chSep2   ';'
 
@@ -558,6 +574,7 @@
 #define cRay      7
 #define cRing     6
 #define cHasMoons 11
+#define cNakshat  27
 
 // Font size constants
 
@@ -755,8 +772,8 @@ enum _aspects {
   aUd4 = 23,
   aUd5 = 24,
   cAspect = 24,
-  cAspect2 = cAspect + aOpp,
-  cAspect3 = cAspect2 + aOpp,
+  cAspect2 = cAspect + aOpp,   // Parallel and Contraparallel
+  cAspect3 = cAspect2 + aOpp,  // Eclipse Conjunct and Opposite
 };
 
 // House systems
@@ -836,16 +853,16 @@ enum _relationshipchart {
   rcNone       = 0,
   rcSynastry   = 1,
   rcComposite  = 2,
-  rcMidpoint   = 3,
-  rcDifference = 4,
+  rcMidpoint   = 3,   // Time space midpoint
+  rcDifference = 4,   // Date difference
   rcBiorhythm  = 5,
-  rcDual       = -1,
+  rcDual       = -1,  // Bi-wheel
   rcTriWheel   = -2,
   rcQuadWheel  = -3,
   rcQuinWheel  = -4,
   rcHexaWheel  = -5,
-  rcTransit    = -6,
-  rcProgress   = -7,
+  rcTransit    = -6,  // Transit and natal
+  rcProgress   = -7,  // Progressed and natal
 };
 
 // Aspect configurations
@@ -866,8 +883,8 @@ enum _aspectconfigurations {
 
 enum _aspectsorting {
   asj = 0,  // By power
-  aso = 1,  // By orb (+/- merged)
-  asn = 2,  // By orb (+/- separated)
+  aso = 1,  // By orb magnitude (+/- merged)
+  asn = 2,  // By orb value (+/- separated)
   asO = 3,  // By 1st object name
   asP = 4,  // By 2nd object name
   asA = 5,  // By aspect
@@ -924,7 +941,8 @@ enum _decandivisiontype {
   dd12      = 7,  // 12th harmonic
   ddDwad    = 8,  // Dwad
   dd27      = 9,  // 27 Nakshatras
-  ddMax     = 10,
+  ddConstel = 10, // Constellations
+  ddMax     = 11,
 };
 
 // Graphics chart modes
@@ -947,60 +965,61 @@ enum _graphicschart {
   gTraTraGra  = 15,
   gTraNatGra  = 16,
   gMoons      = 17,
-  gSphere     = 18,
-  gWorldMap   = 19,
-  gGlobe      = 20,
-  gPolar      = 21,
-  gTelescope  = 22,
-  gBiorhythm  = 23,
+  gExo        = 18,
+  gSphere     = 19,
+  gWorldMap   = 20,
+  gGlobe      = 21,
+  gPolar      = 22,
+  gTelescope  = 23,
+  gBiorhythm  = 24,
 #ifdef WIN
-  gAspect     = 24,
-  gArabic     = 25,
-  gTraTraTim  = 26,
-  gTraTraInf  = 27,
-  gTraNatTim  = 28,
-  gTraNatInf  = 29,
-  gSign       = 30,
-  gObject     = 31,
-  gHelpAsp    = 32,
-  gConstel    = 33,
-  gPlanet     = 34,
-  gRay        = 35,
-  gMeaning    = 36,
-  gSwitch     = 37,
-  gObscure    = 38,
-  gKeystroke  = 39,
-  gCredit     = 40,
-  gMax        = 41,
+  gAspect     = 25,
+  gArabic     = 26,
+  gTraTraTim  = 27,
+  gTraTraInf  = 28,
+  gTraNatTim  = 29,
+  gTraNatInf  = 30,
+  gSign       = 31,
+  gObject     = 32,
+  gHelpAsp    = 33,
+  gConstel    = 34,
+  gPlanet     = 35,
+  gRay        = 36,
+  gMeaning    = 37,
+  gSwitch     = 38,
+  gObscure    = 39,
+  gKeystroke  = 40,
+  gCredit     = 41,
+  gMax        = 42,
 #endif
 };
 
 // Colors
 
 enum _colors {
-  kReverse = -2,
-  kDefault = -1,
-  kBlack   = 0,
-  kMaroon  = 1,
-  kDkGreen = 2,
-  kOrange  = 3,
-  kDkBlue  = 4,
-  kPurple  = 5,
-  kDkCyan  = 6,
-  kLtGray  = 7,
-  kDkGray  = 8,
-  kRed     = 9,
-  kGreen   = 10,
-  kYellow  = 11,
-  kBlue    = 12,
-  kMagenta = 13,
-  kCyan    = 14,
-  kWhite   = 15,
-  kElement = 16,
-  kRay     = 17,
-  kStar    = 18,
-  kPlanet  = 19,
-  kNull    = 16,
+  kReverse = -2,  // Reverse video
+  kDefault = -1,  // Default color
+  kBlack   = 0,   // Black
+  kMaroon  = 1,   // Dark Red
+  kDkGreen = 2,   // Dark Green
+  kOrange  = 3,   // Dark Yellow
+  kDkBlue  = 4,   // Dark Blue
+  kPurple  = 5,   // Purple
+  kDkCyan  = 6,   // Dark Cyan
+  kLtGray  = 7,   // Light Gray
+  kDkGray  = 8,   // Dark Gray
+  kRed     = 9,   // Red
+  kGreen   = 10,  // Green
+  kYellow  = 11,  // Yellow
+  kBlue    = 12,  // Blue
+  kMagenta = 13,  // Magenta
+  kCyan    = 14,  // Cyan
+  kWhite   = 15,  // White
+  kElement = 16,  // Color of element
+  kRay     = 17,  // Color of Ray
+  kStar    = 18,  // Color of star's brightness
+  kPlanet  = 19,  // Color of planet orbited
+  kNull    = 16,  // Invalid color
 };
 
 // Arabic parts
@@ -1167,6 +1186,7 @@ enum _terminationcode {
 #define NHex(ch) ((int)((ch) <= '9' ? (ch) - '0' : (ch) - 'a' + 10) & 15)
 #define NHex2(ch) ((int)((ch) <= '9' ? (ch) - '0' : (ch) - 'A' + 10) & 15)
 #define FSzSet(sz) ((sz) != NULL && *(sz) != chNull)
+#define SzSet(sz) ((sz) != NULL ? (sz) : "")
 #define NMultDiv(n1, n2, n3) ((int)((long)(n1) * (n2) / (n3)))
 #define Ratio(v1, v2, v3) ((v1) + ((v2) - (v1)) * (v3))
 #define ZFromS(s) ((real)(((s)-1)*30))
@@ -1179,6 +1199,7 @@ enum _terminationcode {
 #define RFloor(r) floor(r)
 #define NFloor(r) ((int)RFloor(r))
 #define RFract(r) ((r) - RFloor(r))
+#define RMod(r1, r2) fmod(r1, r2)
 #define Sq(n) ((n)*(n))
 #define RSqr(r) sqrt(r)
 #define RLog(r) log(r)
@@ -1228,8 +1249,9 @@ enum _terminationcode {
 #define FSector(s)    FBetween(s, 1, cSector)
 #define ChDashF(f)    (f ? '=' : '_')
 #define SzNumF(f)     (f ? "1 " : "0 ")
-#define ChDst(dst)    (dst == 0.0 ? 'S' : (dst == 1.0 ? 'D' : \
-  (dst != dstAuto ? 'A' : (is.fDst ? 'D' : 'S'))))
+#define ChDst(dst)    ((dst) == 0.0 ? 'S' : ((dst) == 1.0 ? 'D' : \
+  ((dst) != dstAuto ? 'A' : (is.fDst ? 'D' : 'S'))))
+#define DstReal(dst)  ((dst) != dstAuto ? (dst) : (real)is.fDst)
 #define DayInYear(yea) (365-28+DayInMonth(2, yea))
 #define FChSwitch(ch) ((ch) == '-' || (ch) == '/' || (ch) == '_' || \
   (ch) == '=' || (ch) == ':')
@@ -1254,16 +1276,17 @@ enum _terminationcode {
 #define FValidHarmonic(r) FBetween(r, -10000000.0, 10000000.0)
 #define FValidDecan(n) FBetween(n, 0, ddMax-1)
 #define FValidWheel(n) FBetween(n, 0, WHEELROWS)
+#define FValidAppSep(n) FBetween(n, 0, 2)
 #define FValidAstrograph(n) ((n) > 0 && (n) < 90)
 #define FValidAstrograph2(n) FBetween(n, 0, 40000)
 #define FValidPart(n) FBetween(n, 1, cPart)
 #define FValidDwad(n) FBetween(n, 0, 12)
 #define FValidBioday(n) FBetween(n, 1, 199)
 #define FValidScreen(n) FBetween(n, 20, 200)
-#define FValidMacro(n) FBetween(n, 1, 48)
+#define FValidMacro(n) ((n) >= 0)
 #define FValidList(n) FBetween(n, 0, is.cci-1)
 #define FValidGlyphs(n) FBetween(n, 0, 223222)
-#define FValidDecaType(n) FBetween(n, 0, 2)
+#define FValidDecaType(n) FBetween(n, 0, 5)
 #define FValidDecaSize(n) FBetween(n, 0, 100)
 #define FValidDecaLine(n) FBetween(n, 1, 1000)
 #define FValidGrid(n) FBetween(n, 0, cObj)
@@ -1300,14 +1323,16 @@ enum _terminationcode {
 #define kModeB(m) kElemB[(m) <= 1 ? (m) : eWat]
 #define FInterpretObj(obj) (szMindPart[obj][0] != chNull)
 #define FInterpretAsp(asp) ((asp) > 0 && szInteract[asp][0])
-#define szPerson  (ciMain.nam[0] ? ciMain.nam : "This person")
-#define szPerson0 (ciMain.nam[0] ? ciMain.nam : "the person")
-#define szPerson1 (ciMain.nam[0] ? ciMain.nam : "Person1")
-#define szPerson2 (ciTwin.nam[0] ? ciTwin.nam : "Person2")
+#define szPerson  (FSzSet(ciMain.nam) ? ciMain.nam : "This person")
+#define szPerson0 (FSzSet(ciMain.nam) ? ciMain.nam : "the person")
+#define szPerson1 (FSzSet(ciMain.nam) ? ciMain.nam : "Person1")
+#define szPerson2 (FSzSet(ciTwin.nam) ? ciTwin.nam : "Person2")
 #define FIgnore(i) ignore[i]
 #define FIgnore2(i) ignore2[i]
 #define FAllow(i) (us.objRequire < 0 || (i) == us.objRequire)
 #define FNoTimeOrSpace(ci) ((ci).mon < mJan)
+#define f1K (us.fSeconds && us.fSecond1K)
+#define VSeconds(i, j, k) (!us.fSeconds ? (i) : (!us.fSecond1K ? (j) : (k)))
 
 #define loop for (;;)
 #define inv(v) v = !(v)
@@ -1315,6 +1340,7 @@ enum _terminationcode {
 #define PrintL() PrintCh('\n')
 #define PrintL2() PrintSz("\n\n")
 #define PrintF(sz) fprintf(file, "%s", sz)
+#define SwapTemp(v1, v2, vT) vT = v1; v1 = v2; v2 = vT
 #define SwapN(n1, n2) (n1)^=(n2)^=(n1)^=(n2)
 #define FSwitchF(f) ((((f) || fOr) && !fAnd) != fNot)
 #define FSwitchF2(f) (((f) || (fOr || fNot)) && !fAnd)
@@ -1327,6 +1353,7 @@ enum _terminationcode {
 #define CONST const
 #define PAllocateCore(cb) malloc(cb)
 #define DeallocatePCore(p) free(p)
+#define DeallocatePIf(p) if ((p) != NULL) DeallocateP(p);
 #define ldTime 2440588L
 #ifndef PC
 #define chDirSep '/'
@@ -1353,7 +1380,7 @@ enum _terminationcode {
 #endif
 
 // Should an object in the outer wheel be restricted?
-#define FProper2(i) (!(us.nRel == rcTransit ? ignore2[i] : ignore[i]))
+#define FProper2(i) (!(us.nRel <= rcTransit ? ignore2[i] : ignore[i]))
 
 // Are particular coordinates on the chart?
 #define FInRect(x, y, x1, y1, x2, y2) \
@@ -1378,7 +1405,7 @@ enum _terminationcode {
 // Do settings indicate the current chart should have the info sidebar?
 #define fSidebar \
   ((gi.nMode == gWheel || gi.nMode == gHouse || gi.nMode == gMidpoint || \
-  gi.nMode == gSector || gi.nMode == gSphere) && gs.fText && !us.fVelocity)
+  gi.nMode == gSector || gi.nMode == gSphere) && gs.fText && gs.fDoSidebar)
 
 // Is the current chart most properly displayed as a square graphic?
 #define fSquare \
@@ -1405,7 +1432,7 @@ enum _terminationcode {
   !((gi.nMode == gWorldMap || gi.nMode == gGlobe || gi.nMode == gPolar) && \
     (gs.fAlt || gs.fConstel)) && \
   !((gi.nMode == gWheel || gi.nMode == gHouse || gi.nMode == gMidpoint || \
-    gi.nMode == gSector || gi.nMode == gSphere) && !us.fVelocity))
+    gi.nMode == gSector || gi.nMode == gSphere) && gs.fDoSidebar))
 #endif // GRAPH
 
 
@@ -1506,6 +1533,20 @@ typedef struct _TransInfo {
   real retT;     // Transiting planet's zodiac position velocity
 } TransInfo;
 
+typedef struct _ExoplanetData {
+  char *sz;      // The name of the exoplanet
+  real ra;       // RA of exoplanet's location
+  real dec;      // Declination of exoplanet's location
+  real mag;      // Magnitude of exoplanet's star
+  real epoch;    // Epoch of middle of exoplanet transit
+  real epochU;   // Uncertainty offset of exoplanet epoch
+  real period;   // Period of exoplanet transit in days
+  real periodU;  // Uncertainty of exoplanet period in days
+  real dur;      // Duration of exoplanet transit in hours
+  real JDLoop;   // Time of current exoplanet transit
+  KI kiLoop;     // Color used for current exoplanet transit
+} ExoData;
+
 typedef struct _AtlasEntry {
   real lon;              // Longitude of city
   real lat;              // Latitude of city
@@ -1583,6 +1624,19 @@ typedef struct _ObjDraw {
 
 typedef word * TransGraInfo[objMax][objMax][cAspect+1];
 
+typedef struct _ExtraStar {
+  real lon;           // Zodiac position.
+  real lat;           // Vertical latitude.
+  real dir;           // Velocity or speed.
+  real mag;           // Magnitude or brightness.
+  PT3R pt;            // Coordinates in 3D space.
+  char sz[cchSzDef];  // Buffer for name of star.
+  char *pchNam;       // Star's classic name.
+  char *pchDes;       // Star's designation name.
+  char *pchBest;      // Best available name.
+  KI ki;              // Color to use for star.
+} ES;
+
 typedef struct _UserSettings {
 
   // Chart types
@@ -1611,6 +1665,7 @@ typedef struct _UserSettings {
   flag fAtlasNear;      // -Nl
   flag fZoneChange;     // -Nz
   flag fMoonChart;      // -8
+  flag fExoTransit;     // -Ux
 
   // Chart suboptions
   flag fVelocity;       // -v0
@@ -1677,6 +1732,7 @@ typedef struct _UserSettings {
   flag fNoSwitches;
   flag fLoopInit;    // -Q0
   flag fSeconds;     // -b0
+  flag fSecond1K;    // -b1
   flag fPlacalcAst;  // -ba
   flag fPlacalcPla;  // -bp
   flag fMatrixPla;   // -bm
@@ -1697,6 +1753,8 @@ typedef struct _UserSettings {
   flag fSidereal2;     // -Ys
   flag fTrueNode;      // -Yn
   flag fNoNutation;    // -Yn0
+  flag fOffsetOnly;    // -Yn1
+  flag fNaturalNode;   // -Ynn
   flag fEuroDate;      // -Yd
   flag fEuroTime;      // -Yt
   flag fEuroDist;      // -Yv
@@ -1720,6 +1778,7 @@ typedef struct _UserSettings {
   flag fStarsList;     // -YRU0
   flag fStarMagDist;   // -YUb
   flag fStarMagAbs;    // -YUb0
+  flag fNoDisplay;     // -Y0
   flag fNoWrite;       // -0o
   flag fNoRead;        // -0i
   flag fNoQuit;        // -0q
@@ -1753,19 +1812,14 @@ typedef struct _UserSettings {
   int   nScreenWidth;  // -I
   int   nWriteFormat;  // -o
   int   nListAll;      // -5e
-  real  dstDef;        // -z0
-  real  zonDef;        // -z
-  real  lonDef;        // -zl
-  real  latDef;        // -zl
   real  elvDef;        // -zv
   real  tmpDef;        // -zf
-  char *namDef;        // -zj
-  char *locDef;        // -zj
   char *rgszPath[10];  // -Yi
   char *szADB;         // -Y5i
   char *szAstColor;    // -YkE
   char *szStarsColor;  // -YkU
   char *szStarsList;   // -YRU
+  char *szExoList;     // -YUx
 
   // Value subsettings
   int   nWheelRows;        // Number of rows per house to use for -w wheel.
@@ -1785,6 +1839,7 @@ typedef struct _UserSettings {
   int   cSequenceLine;     // -Yq
   long  lTimeAddition;     // -Yz
   real  rDeltaT;           // -Yz0
+  real  rStation;          // -Yw
   real  rObjAddition;      // -YzO
   real  rCuspAddition;     // -YzC
   int   objRot1;           // -Y1
@@ -1824,10 +1879,17 @@ typedef struct _UserSettings {
   char *szExpFontHou;  // -~FC
   char *szExpFontObj;  // -~FO
   char *szExpFontAsp;  // -~FA
+  char *szExpFontNak;  // -~FN
   char *szExpSort;     // -~v
   char *szExpDecan;    // -~v3
+  char *szExpDegree;   // -~sd
   char *szExpStar;     // -~U
   char *szExpAst;      // -~U0
+  char *szExpExo;      // -~Ux
+  char *szExpIntV;     // -~Iv
+  char *szExpIntV2;    // -~IV
+  char *szExpIntA;     // -~Ia
+  char *szExpIntA2;    // -~IA
   char *szExpCity;     // -~XL
   char *szExpSidebar;  // -~Xt
   char *szExpKey;      // -~XQ
@@ -1849,8 +1911,6 @@ typedef struct _InternalSettings {
   flag fProgress;      // Are we doing a chart involving progression?
   flag fReturn;        // Are we doing a transit chart for returns?
   flag fMult;          // Have we already printed at least one text chart?
-  flag fSeconds;       // Do we print locations to nearest second?
-  flag fSzPersist;     // Are parameter strings persistent when processing?
   flag fSzInteract;    // Are we in middle of chart so some setting fixed?
   flag fNoEphFile;     // Have we already had a ephem file not found error?
   flag fSwissPathSet;  // Has the Swiss Ephemeris path been set yet?
@@ -1878,6 +1938,9 @@ typedef struct _InternalSettings {
   int iciCur;          // Index of most recently used chart in chart list.
   int iciIndex1;       // Index into chart list used for chart #1 with -5e.
   int iciIndex2;       // Index into chart list used for chart #2 with -5e3.
+  int cexod;           // Number of exoplanet transit stars loaded from file.
+  int cszMacro;        // Number of command switch macro strings in list.
+  int cesSort;         // Number of extra star structures in sorted list.
   int cAlloc;          // Number of memory allocations currently allocated.
   int cAllocTotal;     // Total memory allocations allocated this session.
   int cbAllocSize;     // Total bytes in all memory allocations allocated.
@@ -1892,6 +1955,9 @@ typedef struct _InternalSettings {
   RuleEntry *rgrue;    // List of all Daylight Saving change rule entries.
   real *rgzonCol;      // Cache of time zone offsets for each zone area.
   CI *rgci;            // List of chart information records for chart list.
+  ExoData *rgexod;     // List of exoplanet transit stars loaded from file.
+  char **rgszMacro;    // List of command switch macro strings.
+  ES *rgesSort;        // List of sorted extra stars or extra asteroids.
   FILE *fileIn;        // The switch file currently being read from.
   FILE *S;             // File to write text to.
   real T;              // Julian time for chart.
@@ -1906,21 +1972,6 @@ typedef struct _InternalSettings {
   real rNut;           // Nutation offset.
 } IS;
 
-#ifdef SWISS
-typedef struct _ExtraStar {
-  real lon;           // Zodiac position.
-  real lat;           // Vertical latitude.
-  real dir;           // Velocity or speed.
-  real mag;           // Magnitude or brightness.
-  PT3R pt;            // Coordinates in 3D space.
-  char sz[cchSzDef];  // Buffer for name of star.
-  char *pchNam;       // Star's classic name.
-  char *pchDes;       // Star's designation name.
-  char *pchBest;      // Best available name.
-  KI ki;              // Color to use for star.
-} ES;
-#endif
-
 #ifdef GRAPH
 typedef struct _Bitmap {
   int x;      // Horizontal pixel size of bitmap
@@ -1936,6 +1987,7 @@ typedef struct _GraphicsSettings {
   flag fInverse;     // Are we drawing in reverse video (-Xr set).
   flag fRoot;        // Are we drawing on the X11 background (-XB set).
   flag fText;        // Are we printing chart info on chart (-Xt set).
+  flag fDoSidebar;   // Are we drawing wheel chart sidebar (-Xv0 set).
   int nFontAll;      // Which fonts to use for sign/house/obj/asp (-YXf).
   int nFontTxt;      // Which font to use for graphics text       (-YXft).
   int nFontSig;      // Which font to use for signs of the zodiac (-YXfs).
@@ -1955,11 +2007,13 @@ typedef struct _GraphicsSettings {
   flag fEquator;     // Are we showing equator on maps/globes (-Xe set).
   flag fEcliptic;    // Are we drawing oriented to ecliptic (-YXe set).
   flag fAllStar;     // Are we drawing all sefstars.txt stars (-XU set).
+  flag fAllExo;      // Are we drawing astexo.csv exoplanets (-XUx set).
   flag fHouseExtra;  // Are we showing additional house info (-XC set).
   flag fPrintMap;    // Are we printing globe names on draw (-XPv set).
   flag fKeepSquare;  // Are we preserving chart aspect ratio (-XQ set).
   flag fAnimMap;     // Are we animating map instead of time (-XN set).
   flag fThick;       // Are we drawing thicker lines in charts (-Xx set).
+  flag fAntialias;   // Are we drawing lines antialiased (-Xx0 set).
   flag fIndianWheel; // Are wheel charts North/South Indian (-XJ set).
   flag fMoonWheel;   // Are moons drawn around planets in wheels (-X8 set).
   int xWin;          // Current hor. size of graphic chart (-Xw).
@@ -2042,6 +2096,7 @@ typedef struct _GraphicsInternal {
   int ispace;         // Index of most recent coordinate (-S -X).
   int cspace;         // Coordinates within table so far (-S -X).
   int *rgzCalendar;   // Aspect coordinates in calendar (-K -X).
+  char *rgConstel;    // Constellation equatorial coordinate lookup array.
   flag fBmp;          // Are 24 bit bitmaps being used? (-Xbw set).
   Bitmap bmp;         // Bitmap storing chart contents, sized appropriately.
   Bitmap bmpBack;     // Bitmap storing background, as loaded from file.
@@ -2053,6 +2108,7 @@ typedef struct _GraphicsInternal {
   int cStarsLin;      // Count of extra star coordinates (-YXU).
 #endif
 #ifdef ISG
+  KI kiPen;           // The current pen scribble color (-Xk).
   flag fPause;        // Is animation paused? (-Xnp set).
   int nDir;           // Animation step factor and direction (-Xnf).
   int xWinResize;     // Manually resized hor. size of graphic chart.
@@ -2134,6 +2190,7 @@ typedef struct _OrbitalElements {
 } OE;
 
 #define szFileJPLCore  "astrolog.jpl"
+#define szFileExoCore  "astexo.csv"
 #ifdef WIN
 #define szFileTempCore "astrolog.tmp"
 #define szFileAutoCore "astrolog.bmp"
@@ -2154,6 +2211,7 @@ typedef struct _WindowInternal {
   HPEN hpen;         // Pen with the current line color.
   HBRUSH hbrush;     // Fill if any with the current color.
   HFONT hfont;       // Font of current text size being printed.
+  HBITMAP hbmp;      // Bitmap storing contents to be copied to window.
   HBITMAP hbmpBack;  // Bitmap for the background image.
   HBITMAP hbmpPrev;  // Bitmap to restore after using background.
   HANDLE hMutex;     // To ensure output file isn't already open.
@@ -2192,12 +2250,12 @@ typedef struct _WindowInternal {
   flag fWindowChart;   // Does window resize cause chart change?
   flag fNoUpdate;      // Do we not automatically update screen?
   flag fNoPopup;       // Do we not display warning messages?
+  flag fStarLine;      // Do we display all constellation lines?
   flag fSaverRun;      // Do we run Astrolog as a screen saver?
   flag fAutoSave;      // Are we saving chart after win draw?
   flag fAutoSaveNum;   // Autosave charts are incremental files?
   flag fAutoSaveWire;  // Autosave wireframe instead of bitmap?
   int nAutoSaveNum;    // Number of incremental bitmap save.
-  KI kiPen;            // The current pen scribble color.
   UINT nTimerDelay;    // Milliseconds between animation draws.
 } WI;
 #endif
@@ -2211,6 +2269,7 @@ typedef struct _WindowInternal {
   HDC hdcBack;       // The current DC for the background bitmap.
   HPEN hpen;         // Pen with the current line color.
   HBRUSH hbrush;     // Fill if any with the current color.
+  HBITMAP hbmp;      // Bitmap storing contents to be copied to window.
   HBITMAP hbmpBack;  // Bitmap for the background image.
   HBITMAP hbmpPrev;  // Bitmap to restore after using background.
   short xClient;     // Horizontal & vertical window size.
@@ -2224,7 +2283,6 @@ typedef struct _WindowInternal {
   int yMouse;
   LPARAM lParamRC;   // Coordinates where right click originated.
   Bitmap bmpWin;     // Bitmap storing contents to be copied to window.
-  KI kiPen;          // The current pen scribble color.
 } WI;
 #endif
 
