@@ -1,8 +1,8 @@
 /*
-** Astrolog (Version 7.60) File: extern.h
+** Astrolog (Version 7.80) File: extern.h
 **
 ** IMPORTANT NOTICE: Astrolog and all chart display routines and anything
-** not enumerated below used in this program are Copyright (C) 1991-2023 by
+** not enumerated below used in this program are Copyright (C) 1991-2025 by
 ** Walter D. Pullen (Astara@msn.com, http://www.astrolog.org/astrolog.htm).
 ** Permission is granted to freely use, modify, and distribute these
 ** routines provided these credits and notices remain unmodified with any
@@ -10,8 +10,8 @@
 **
 ** The main ephemeris databases and calculation routines are from the
 ** library SWISS EPHEMERIS and are programmed and copyright 1997-2008 by
-** Astrodienst AG. The use of that source code is subject to the license for
-** Swiss Ephemeris Free Edition, available at http://www.astro.com/swisseph.
+** Astrodienst AG. Use of that source code is subject to license for Swiss
+** Ephemeris Free Edition at https://www.astro.com/swisseph/swephinfo_e.htm.
 ** This copyright notice must not be changed or removed by any user of this
 ** program.
 **
@@ -48,7 +48,7 @@
 ** Initial programming 8/28-30/1991.
 ** X Window graphics initially programmed 10/23-29/1991.
 ** PostScript graphics initially programmed 11/29-30/1992.
-** Last code change made 4/8/2023.
+** Last code change made 6/19/2025.
 */
 
 /*
@@ -73,6 +73,7 @@ extern int NParseCommandLine P((char *, char **));
 extern int NPromptSwitches P((char *, char *[MAXSWITCHES]));
 extern int NProcessSwitchesRare P((int, char **, int, flag, flag, flag));
 extern flag FProcessSwitches P((int, char **));
+extern void InitRestrictions P((flag));
 extern void InitProgram P((void));
 extern void FinalizeProgram P((flag));
 
@@ -133,6 +134,8 @@ extern void FinalizeProgram P((flag));
 #define DEFAULT_LOC DMS(122, 19, 55), DMS(47, 36, 22)
 #define FProperGraph(o) !(!us.fGraphAll && (FCusp(o) || \
   (us.fInDayMonth && ((o) == oMoo || (us.fInDayYear && (o) <= oMar)))))
+#define FProperEphem2(o) (!(us.nRel <= rcTransit ? ignore2[o] : ignore[o]) && \
+  !(gs.fAlt && ((o) == oMoo || (o) == oFor)))
 
 #define FCmSwissAny() (us.fEphemFiles && !us.fPlacalcPla)
 #define FCmSwissEph() (us.fEphemFiles && !us.fPlacalcPla && us.nSwissEph <= 0)
@@ -146,10 +149,11 @@ extern void FinalizeProgram P((flag));
 extern US us;
 extern IS is;
 extern CI ciCore, ciMain, ciTwin, ciThre, ciFour, ciFive, ciHexa,
-  ciTran, ciSave, ciGreg;
+  ciDefa, ciTran, ciSave, ciGreg;
 extern CP cp0, cp1, cp2, cp3, cp4, cp5, cp6;
 extern CP * CONST rgpcp[cRing+1];
 extern CI * CONST rgpci[cRing+1];
+extern flag rgfProg[cRing+1];
 
 extern real force[objMax];
 extern GridInfo *grid;
@@ -197,19 +201,23 @@ extern CONST char *szAppName, *szSignName[cSign+1], *szSignAbbrev[cSign+1],
   *szAspectGlyph[cAspect2+1], *szAspectConfig[cAspConfig],
   *szElem[cElem], *szMode[3], *szMonth[cSign+1], *szDay[cWeek], *szZon[cZone],
   *rgszDir[4], *szSuffix[cSign+1], *szEphem[cmMax];
-extern CONST int rgAspConfig[cAspConfig];
+extern CONST int rgAspConfig[cAspConfig], dxOff[4], dyOff[4],
+  iCnstlZodiac[cSign+1];
 extern CONST real rZon[cZone];
 extern CONST char *szObjName[objMax+4], *szCnstlName[cCnstl+1],
   *szCnstlAbbrev[cCnstl+1], *szCnstlMeaning[cCnstl+1],
   *szCnstlGenitive[cCnstl+1];
-extern CONST real rStarBrightMatrix[cStar+1], rStarData[cStar*6];
-extern char *szMindPart[objMax], *szDesc[cSign+1], *szDesire[cSign+1],
-  *szLifeArea[cSign+1], *szInteract[cAspect+1], *szTherefore[cAspect+1],
-  *szModify[3][cAspect];
+extern CONST real rStarBrightMatrix[cStar+1], rStarData[cStar*6],
+  lonCnstlZodiac[cSign+2];
+extern CONST char *szMindPartDef[objMax], *szDescDef[cSign+1],
+  *szDesireDef[cSign+1], *szLifeAreaDef[cSign+1], *szInteractDef[cAspect+1],
+  *szThereforeDef[cAspect+1], *szModify[3][cAspect];
+extern CONST char *szMindPart[objMax], *szDesc[cSign+1], *szDesire[cSign+1],
+  *szLifeArea[cSign+1], *szInteract[cAspect+1], *szTherefore[cAspect+1];
 extern CONST StrLook rgObjName[], rgSystem[], rgAspectName[];
 extern CONST StrLookR rgZodiacOffset[];
-extern CONST char *szNakshatra[27+1], *szEclipse[etMax], rgchEclipse[etMax+1],
-  *szAppSep[6], rgchAppSep[6+1];
+extern CONST char *szNakshatra[cNakshat+1], *rgszDecan[ddMax],
+  *szEclipse[etMax], rgchEclipse[etMax+1], *szAppSep[6], rgchAppSep[6+1];
 
 extern real rStarBrightDef[cStar+1], rStarBright[cStar+1],
   rStarBrightDistDef[cStar+1];
@@ -224,7 +232,7 @@ extern CONST int cSatellite[oPlu+1], nMooMap[6][8], rgobjHasMoons[cHasMoons];
 
 extern CONST AI ai[cPart];
 
-extern char *szMacro[48], *szWheel[cRing+1];
+extern char *szWheel[cRing+1];
 extern CONST char *szColor[cColor+4], *szColorHTML[cColor];
 
 extern int rgObjRay[oNorm+1], rgSignRay[cSign+1], rgSignRay2[cSign+1][cRay+1],
@@ -239,6 +247,8 @@ extern CONST char *szRayName[cRay+1], *szRayWill[cRay+1];
 #define FEqSz(sz1, sz2) (NCompareSz(sz1, sz2) == 0)
 #define FEqSzI(sz1, sz2) (NCompareSzI(sz1, sz2) == 0)
 #define RgAllocate(n, t, sz) ((t *)PAllocate((n) * sizeof(t), sz))
+#define FCloneSz(szSrc, pszDst) FCloneSzCore(szSrc, pszDst, fFalse)
+#define SzTime(h, m, s) SzTimeR(h, m, s, -1.0)
 #define PrintAltitude(deg) PrintSz(SzAltitude(deg))
 #define FEqCI(ci1, ci2) (\
   ci1.mon == ci2.mon && ci1.day == ci2.day && ci1.yea == ci2.yea && \
@@ -298,6 +308,7 @@ extern int DaysInMonth P((int, int));
 extern int DayOfWeek P((int, int, int));
 extern int AddDay P((int, int, int, int));
 extern void AddTime P((CI *, int, int));
+extern real GetOffsetCI P((CONST CI *));
 extern real GetOrb P((int, int, int));
 extern CONST char *SzAspect P((int));
 extern CONST char *SzAspectAbbrev P((int));
@@ -326,15 +337,17 @@ extern flag FErrorSubswitch P((CONST char *, char, flag));
 extern void ErrorEphem P((CONST char *, long));
 extern void AnsiColor P((int));
 extern void PrintZodiac P((real));
+extern char ChRet P((real));
 extern char *SzZodiac P((real));
 extern char *SzAltitude P((real));
 extern char *SzDegree P((real));
 extern char *SzDegree2 P((real));
 extern char *SzHMS P((int));
 extern char *SzDate P((int, int, int, int));
-extern char *SzTime P((int, int, int));
+extern char *SzTimeR P((int, int, int, real));
 extern char *SzTim P((real));
 extern char *SzZone P((real));
+extern char *SzOffset P((real, real, real));
 extern char *SzLocation P((real, real));
 extern char *SzElevation P((real));
 extern char *SzTemperature P((real));
@@ -356,11 +369,12 @@ extern void UTF8ToLatinSz P((char *));
 extern void UTF8ToIBMSz P((char *));
 extern void ConvertSzFromUTF8 P((char *));
 extern CONST char *ConvertSzToLatin P((CONST char *, char *, int));
-extern char *SzPersist P((char *));
-extern char *SzCopy P((char *));
+extern flag FCloneSzCore P((CONST char *, char **, flag));
+extern char *SzClone P((char *));
 extern pbyte PAllocate P((long, CONST char *));
 extern void DeallocateP P((void *));
 extern pbyte RgReallocate P((void *, int, int, int, CONST char *));
+extern flag FEnsureMacro P((int));
 #ifdef DEBUG
 extern void Assert P((flag));
 #else
@@ -372,6 +386,8 @@ extern void Terminate P((int));
 // From io.cpp
 
 #define getbyte() BRead(file)
+#define AdvancePast(ch) while (*pch && *pch != (ch)) pch++; \
+  if (*pch == (ch)) pch++;
 
 extern FILE *FileOpen P((CONST char *, int, char *));
 extern byte BRead P((FILE *));
@@ -448,9 +464,11 @@ extern int GetParallel P((CONST real *, CONST real *, CONST real *,
   CONST real *, CONST real *, CONST real *, int, int, real *));
 extern flag FCreateGrid P((flag));
 extern flag FCreateGridRelation P((flag));
+extern int NCheckEclipseSolar P((int, int, int, real *));
 extern int NCheckEclipse P((int, int, real *));
 extern int NCheckEclipseLunar P((int, int, int, real *));
 extern int NCheckEclipseAny P((int, int, int, real *));
+extern int NCheckEclipseSolarLoc P((real, real, real *));
 extern void CreateElemTable P((ET *));
 
 #ifdef SWISS
@@ -464,8 +482,10 @@ extern void SwissHouse P((real, real, real, int,
   real *, real *, real *, real *, real *, real *, real *, real *));
 extern void SwissComputeStars P((real, flag));
 extern flag SwissComputeStar P((real, ES *));
+extern flag SwissComputeStarSort P((real, ES *));
 extern flag SwissTestStar P((char *));
 extern flag SwissComputeAsteroid P((real, ES *, flag));
+extern flag SwissComputeAsteroidSort P((real, ES *));
 extern void SwissGetObjName P((char *, int));
 extern flag FSwissPlanetData P((real, int, real *, real *, real *));
 extern real SwissRefract P((real));
@@ -568,6 +588,7 @@ extern void PrintMidpointSummary P((int *, int, real));
 extern void ChartMidpoint P((void));
 extern void ChartHorizon P((void));
 extern void ChartOrbit P((void));
+extern int ChartEsoteric P((flag));
 extern void ChartSector P((void));
 extern flag ChartAstroGraph P((void));
 extern void PrintChart P((flag));
@@ -601,14 +622,16 @@ extern void DisplayRelation P((void));
 
 extern void ChartInDaySearch P((flag));
 extern void ChartTransitSearch P((flag));
-extern void ChartInDayHorizon P((void));
+extern void ChartHorizonRising P((void));
 extern void ChartEphemeris P((void));
+extern flag ChartExoplanet P((flag));
 
 
 // From intrpret.cpp
 
 #define RObjInf(i) rObjInf[Min(i, oNorm1)]
 #define RTransitInf(i) rTransitInf[Min(i, oNorm1)]
+extern CONST char *rgEsoRayArea[5];
 
 #ifdef INTERPRET
 extern void FieldWord P((CONST char *));
@@ -626,7 +649,7 @@ extern void InterpretAspectRelation P((int, int));
 extern void InterpretGridRelation P((void));
 extern void InterpretMidpointRelation P((int, int));
 extern void InterpretAstroGraph P((int, int, int, int));
-extern void InterpretEsoteric P((void));
+extern int InterpretEsoteric P((flag));
 extern void PrintEsoteric P((void));
 #endif
 extern void SortRank P((real *, int *, int, flag));
@@ -698,15 +721,16 @@ extern KV rgbbmp[cColor];
 extern KI kMainB[9], kRainbowB[cRainbow+1], kElemB[cElem], kAspB[cAspect+1],
   kObjB[objMax], kRayB[cRay+2];
 extern CONST char *rgszFontName[cFont], rgszFontAllow[6][cFont+1];
-extern CONST real rgrObjRing[oNep-oJup+2][2];
-extern CONST PT3R rgvObjRing[oNep-oJup+2];
+extern CONST real rgrObjRing[oNep-oJup+3][2];
+extern CONST PT3R rgvObjRing[oNep-oJup+3];
 extern CONST char
-  *szDrawSign[cSign+2], *szDrawSign2[cSign+2], *szDrawSign3[cSign+2],
+  *szDrawSign[cSign+3], *szDrawSign2[cSign+3], *szDrawSign3[cSign+3],
   *szDrawObjectDef[objMaxG], *szDrawObjectDef2[objMaxG],
   *szDrawHouse[cSign+1], *szDrawHouse2[cSign+1], *szDrawHouse3[cSign+1],
   *szDrawAspectDef[cAspect3+1], *szDrawAspectDef2[cAspect3+1],
   *szDrawCh[256-32], *szDrawCh2[256-32],
-  *szWorldData[62*3], *szDrawConstel[cCnstl+1];
+  *szWorldData[62*3],
+  *szDrawConstel[cCnstl+1], *szDrawConstelLine[(cCnstl+1)*2+1];
 extern CONST char *szDrawObject[objMaxG], *szDrawObject2[objMaxG],
   *szDrawAspect[cAspect3+1], *szDrawAspect2[cAspect3+1];
 
@@ -730,7 +754,8 @@ extern CONST char *szDrawObject[objMaxG], *szDrawObject2[objMaxG],
 
 #define KStarB(mag) \
   ((mag) < 2.0 ? gi.kiOn : ((mag) < 4.0 ? gi.kiLite : gi.kiGray))
-#define IObjRing(i) ((i) < oHau ? (i)-oJup : (i)-oHau+oNep-oJup+1)
+#define IObjRing(i) ((i) < oHau ? (i)-oJup : \
+  ((i) == oHau ? (i)-oHau+oNep-oJup+1 : (i)-oQua+oNep-oJup+2))
 #define nGlyphAll (gs.nGlyphCap*100000 + gs.nGlyphUra*10000 + \
   gs.nGlyphPlu*1000 + gs.nGlyphLil*100 + gs.nGlyphVer*10 + gs.nGlyphEri)
 
@@ -777,9 +802,7 @@ extern void DrawSz P((CONST char *, int, int, int));
 extern void DrawSign P((int, int, int));
 extern void DrawHouse P((int, int, int));
 extern void DrawObject P((int, int, int));
-#ifdef SWISS
 extern void DrawStar P((int, int, CONST ES *));
-#endif
 extern void DrawAspect P((int, int, int));
 extern void DrawNakshatra P((int, int, int));
 extern int NFromPch P((CONST char **));
@@ -853,6 +876,7 @@ extern void BmpCopyWin P((CONST Bitmap *, HDC, int, int));
 extern flag FBmpDrawBack P((Bitmap *));
 extern flag FBmpDrawMap P((void));
 extern flag FBmpDrawMap2 P((int, int, int, int, real, real, real, real));
+extern flag FBmpAntialias P((void));
 extern void WriteXBitmap P((FILE *, CONST char *, char));
 extern void WriteAscii P((FILE *));
 extern void WriteBmp P((FILE *));
@@ -888,19 +912,21 @@ extern flag DrawFillWheel P((int, int, int, int));
 extern void DrawWheel P((real *, real *, int, int, real, real,
   real, real, real));
 extern void DrawRing P((int, int, real *, real *, int, int, real,
-  real, real, real, real, real, real, real));
+  real, real, real, real, real, real, real, real));
 extern void DrawObjects P((ObjDraw *, int, int));
 extern void DrawAspectLine
-  P((int, int, int, int, real, real, real, real, real));
+  P((int, int, int, int, real, real, real, real, real, flag));
 extern flag EnumWorldLines P((int *, int *, int *, int *, int *));
 #ifdef CONSTEL
 extern flag EnumConstelLines P((int *, int *, int *, int *, int *));
+extern int LookupConstel P((real, real));
 #endif
 extern flag FReadWorldData P((CONST char **, CONST char **, CONST char **));
 extern flag FGlobeCalc P((real, real, int *, int *, CONST CIRC *, real));
 #ifdef SWISS
 extern flag EnumStarsLines P((flag, ES **, ES **));
 #endif
+extern flag EnumExoplanets P((ES *));
 
 extern void DrawLeyLine P((real, real, real, real));
 extern void DrawLeyLines P((real));
@@ -974,6 +1000,7 @@ extern void SquareX P((int *, int *, flag));
 extern void InteractX P((void));
 extern void EndX P((void));
 #endif
+extern flag FProcessYXU P((CONST char *, CONST char *, flag));
 extern int NProcessSwitchesX P((int, char **, int, flag, flag, flag));
 extern int NProcessSwitchesRareX P((int, char **, int, flag, flag, flag));
 extern int DetectGraphicsChartMode P((void));
