@@ -1,8 +1,8 @@
 /*
-** Astrolog (Version 7.60) File: io.cpp
+** Astrolog (Version 7.80) File: io.cpp
 **
 ** IMPORTANT NOTICE: Astrolog and all chart display routines and anything
-** not enumerated below used in this program are Copyright (C) 1991-2023 by
+** not enumerated below used in this program are Copyright (C) 1991-2025 by
 ** Walter D. Pullen (Astara@msn.com, http://www.astrolog.org/astrolog.htm).
 ** Permission is granted to freely use, modify, and distribute these
 ** routines provided these credits and notices remain unmodified with any
@@ -10,8 +10,8 @@
 **
 ** The main ephemeris databases and calculation routines are from the
 ** library SWISS EPHEMERIS and are programmed and copyright 1997-2008 by
-** Astrodienst AG. The use of that source code is subject to the license for
-** Swiss Ephemeris Free Edition, available at http://www.astro.com/swisseph.
+** Astrodienst AG. Use of that source code is subject to license for Swiss
+** Ephemeris Free Edition at https://www.astro.com/swisseph/swephinfo_e.htm.
 ** This copyright notice must not be changed or removed by any user of this
 ** program.
 **
@@ -48,7 +48,7 @@
 ** Initial programming 8/28-30/1991.
 ** X Window graphics initially programmed 10/23-29/1991.
 ** PostScript graphics initially programmed 11/29-30/1992.
-** Last code change made 4/8/2023.
+** Last code change made 6/19/2025.
 */
 
 #include "astrolog.h"
@@ -81,6 +81,7 @@ FILE *FileOpen(CONST char *szFile, int nFileMode, char *szPath)
 
   // Some file types we want to open as binary instead of Ascii.
   sprintf(szMode, "r%s", nFileMode >= 2 ? "b" : "");
+  // Get directory containing Astrolog executable.
 #ifdef WIN
   GetModuleFileName(wi.hinst, szExe, cchSzMax);
 #else
@@ -142,8 +143,8 @@ FILE *FileOpen(CONST char *szFile, int nFileMode, char *szPath)
     // specific system environment variable.
     sprintf(sz, "%s%s", ENVIRONVER, szVerCore);
     env = getenv(sz);
-    if (env && *env) {
-      sprintf(sz, "%s%c%s", env, chDirSep, szFileT);
+    if (FSzSet(env)) {
+      sprintf2(S(sz), "%s%c%s", env, chDirSep, szFileT);
       file = fopen(sz, szMode);
       if (file != NULL)
         goto LDone;
@@ -151,8 +152,8 @@ FILE *FileOpen(CONST char *szFile, int nFileMode, char *szPath)
 
     // Next look in the directory in the general environment variable.
     env = getenv(ENVIRONALL);
-    if (env && *env) {
-      sprintf(sz, "%s%c%s", env, chDirSep, szFileT);
+    if (FSzSet(env)) {
+      sprintf2(S(sz), "%s%c%s", env, chDirSep, szFileT);
       file = fopen(sz, szMode);
       if (file != NULL)
         goto LDone;
@@ -160,8 +161,8 @@ FILE *FileOpen(CONST char *szFile, int nFileMode, char *szPath)
 
     // Next look in directory in the version prefix environment variable.
     env = getenv(ENVIRONVER);
-    if (env && *env) {
-      sprintf(sz, "%s%c%s", env, chDirSep, szFileT);
+    if (FSzSet(env)) {
+      sprintf2(S(sz), "%s%c%s", env, chDirSep, szFileT);
       file = fopen(sz, szMode);
       if (file != NULL)
         goto LDone;
@@ -228,10 +229,11 @@ dword LRead(FILE *file)
 
 flag FProcessSwitchFile(CONST char *szFile, FILE *file)
 {
-  char szLine[cchSzLine], *argv[MAXSWITCHES], ch;
-  int argc, i;
+  char rgchLine[cchSzLine], *argv[MAXSWITCHES], *szLine = rgchLine, *szNew, ch;
+  int argc, cchLine = cchSzLine, i;
   flag fHaveFile, fRet = fFalse;
 
+  // Open a file if don't already have one.
   fHaveFile = (file != NULL);
   if (!fHaveFile) {
     file = FileOpen(szFile, 0, NULL);
@@ -249,12 +251,25 @@ flag FProcessSwitchFile(CONST char *szFile, FILE *file)
     PrintWarning(szLine);
     goto LDone;
   }
+
+  // Read the file one line at a time.
   loop {
     while (!feof(file) && (ch = getbyte()) < ' ')
       ;
     if (feof(file))
       break;
-    for (szLine[0] = ch, i = 1; i < cchSzLine-1 && !feof(file); i++) {
+    for (szLine[0] = ch, i = 1; !feof(file); i++) {
+      if (i >= cchLine-1) {
+        // Double the size of the buffer if the line overfills it.
+        szNew = (char *)RgReallocate(szLine, cchLine, sizeof(char), cchLine*2,
+          "line");
+        if (szNew == NULL)
+          goto LDone;
+        if (szLine != rgchLine)
+          DeallocateP(szLine);
+        szLine = szNew;
+        cchLine *= 2;
+      }
       ch = getbyte();
       if (!((uchar)ch >= ' ' || ch == chTab))
         break;
@@ -273,6 +288,8 @@ flag FProcessSwitchFile(CONST char *szFile, FILE *file)
   fRet = fTrue;
 
 LDone:
+  if (szLine != rgchLine)
+    DeallocateP(szLine);
   is.fileIn = NULL;
   if (!fHaveFile && file != NULL)
     fclose(file);
@@ -289,7 +306,7 @@ flag FOutputData(void)
   char sz[cchSzMax], *pch;
   FILE *file;
   int i, j, iMax;
-  real dst, rT;
+  real rT;
 
   if (us.fNoWrite)
     return fFalse;
@@ -324,11 +341,9 @@ flag FOutputData(void)
       return fFalse;
     }
     if (us.fWriteOld) {
-      dst = Dst;
-      if (dst == dstAuto)
-        dst = (real)is.fDst;
       fprintf(file, "%d\n%d\n%d\n%.2f\n%.2f\n%.2f\n%.2f\n", Mon, Day, Yea,
-        DegToDec(Tim), DegToDec(Zon-dst), DegToDec(Lon), DegToDec(Lat));
+        DegToDec(Tim), DegToDec(GetOffsetCI(&ciMain)),
+        DegToDec(Lon), DegToDec(Lat));
     } else {
       fprintf(file, "@AI%s  ; %s chart info.\n", szVerCore, szAppName);
       i = us.fAnsiChar;
@@ -339,11 +354,13 @@ flag FOutputData(void)
       fprintf(file, "%s %s\n", SzZone(Zon), SzLocation(Lon, Lat));
       // Don't want double quotes within double quoted string parameters.
       fprintf(file, "%czi \"", chSwitch);
-      for (pch = ciMain.nam; *pch; pch++)
-        putc(*pch != '"' ? *pch : '\'', file);
+      if (FSzSet(ciMain.nam))
+        for (pch = ciMain.nam; *pch; pch++)
+          putc(*pch != '"' ? *pch : '\'', file);
       fprintf(file, "\" \"");
-      for (pch = ciMain.loc; *pch; pch++)
-        putc(*pch != '"' ? *pch : '\'', file);
+      if (FSzSet(ciMain.loc))
+        for (pch = ciMain.loc; *pch; pch++)
+          putc(*pch != '"' ? *pch : '\'', file);
       fprintf(file, "\"\n");
       us.fAnsiChar = i;
     }
@@ -380,7 +397,16 @@ flag FOutputData(void)
 
     } else {
       fprintf(file, "@AP%s  ; %s chart positions.\n", szVerCore, szAppName);
-      fprintf(file, "%czi \"%s\" \"%s\"\n", chSwitch, ciMain.nam, ciMain.loc);
+      // Don't want double quotes within double quoted string parameters.
+      fprintf(file, "%czi \"", chSwitch);
+      if (FSzSet(ciMain.nam))
+        for (pch = ciMain.nam; *pch; pch++)
+          putc(*pch != '"' ? *pch : '\'', file);
+      fprintf(file, "\" \"");
+      if (FSzSet(ciMain.loc))
+        for (pch = ciMain.loc; *pch; pch++)
+          putc(*pch != '"' ? *pch : '\'', file);
+      fprintf(file, "\"\n");
       iMax = Max(is.nObj, cuspHi);
       for (i = 0; i <= iMax; i++) if (!ignore[i] || FCusp(i)) {
         fprintf(file, "%cYF ", chSwitch);
@@ -404,18 +430,13 @@ flag FOutputData(void)
   // Now write any extra strings that were on the command line after the -o
   // specification but before the next switch, to the file as comments.
 
-  for (i = 1; i < is.cszComment; i++) {
-    is.rgszComment++;
-    fprintf(file, "%s%s\n", us.fWriteOld ? "" : "; ", is.rgszComment[1]);
-  }
+  for (i = 0; i < is.cszComment; i++)
+    fprintf(file, "%s%s\n", us.fWriteOld ? "" : "; ", is.rgszComment[i]);
   fclose(file);
   return fTrue;
 }
 
 
-#define AdvancePast(ch) while (*pch && *pch != (ch)) pch++; \
-  if (*pch == (ch)) pch++;
-
 // Load an Astrological Exchange Format (AAF) file into the default set of
 // chart information, given a file name or a file handle.
 
@@ -468,7 +489,7 @@ flag FProcessAAFFile(CONST char *szFile, FILE *file)
         sprintf(sz, "%s %s", sz2, sz1);
       else
         sprintf(sz, "%s", sz2);
-      ciCore.nam = SzCopy(sz);
+      ciCore.nam = SzClone(sz);
       AdvancePast(',');
       DD = NFromSz(pch);
       AdvancePast('.');
@@ -496,7 +517,7 @@ flag FProcessAAFFile(CONST char *szFile, FILE *file)
         sprintf(sz, "%s, %s", sz1, sz2);
       else
         sprintf(sz, "%s", sz2);
-      ciCore.loc = SzCopy(sz);
+      ciCore.loc = SzClone(sz);
       grf |= 1;
 
     // Input row #2.
@@ -564,13 +585,15 @@ flag FOutputAAFFile(void)
 
   // Output row #1.
   fprintf(file, "#A93:*,");
-  for (pch = ciMain.nam; *pch; pch++)
-    putc(*pch == ',' ? ';' : *pch, file);
+  if (FSzSet(ciMain.nam))
+    for (pch = ciMain.nam; *pch; pch++)
+      putc(*pch == ',' ? ';' : *pch, file);
   fSav = us.fEuroTime; us.fEuroTime = fTrue;
   fprintf(file, ",*,%d.%d.%d,%s,", Day, Mon, Yea, SzTim(Tim));
   us.fEuroTime = fSav;
-  for (pch = ciMain.loc; *pch && *pch != ','; pch++)
-    putc(*pch, file);
+  if (FSzSet(ciMain.loc))
+    for (pch = ciMain.loc; *pch && *pch != ','; pch++)
+      putc(*pch, file);
   putc(',', file);
   if (!*pch)
     putc('*', file);
@@ -635,7 +658,7 @@ flag FProcessQuickFile(CONST char *szFile, FILE *file)
     for (i = 23-1; i >= 0 && sz[i] <= ' '; i--)
       ;
     sz[i+1] = chNull;
-    ciCore.nam = SzCopy(sz);
+    ciCore.nam = SzClone(sz);
     CopyRgchToSz(szLine+23, 3, sz, cchSzDef);
     MM = NParseSz(sz, pmMon);
     DD = NParseSz(szLine+23+3, pmDay);
@@ -657,7 +680,7 @@ flag FProcessQuickFile(CONST char *szFile, FILE *file)
     for (i = 25-1; i >= 0 && sz[i] <= ' '; i--)
       ;
     sz[i+1] = chNull;
-    ciCore.loc = SzCopy(sz);
+    ciCore.loc = SzClone(sz);
     SS = 0.0;
     if (FZonDst(&szLine[23+24])) {
       ZZ += 1.0; SS += 1.0;
@@ -687,8 +710,8 @@ flag FOutputQuickFile(void)
 {
   char sz[cchSzMax];
   FILE *file = NULL;
-  real rOff, rDst;
-  int i, iMax, j, fSav1;
+  real rOff, rDst, rLon, rLat;
+  int i, iMax, j, k, j2, k2, fSav1;
   flag fRet = fFalse, fSav2;
   CI *pci;
 
@@ -715,12 +738,12 @@ flag FOutputQuickFile(void)
     fprintf(file, "%c%c%c", szMonth[pci->mon][0],
       ChCap(szMonth[pci->mon][1]), ChCap(szMonth[pci->mon][2]));
     fSav1 = us.fAnsiChar; us.fAnsiChar = 4;
-    fSav2 = is.fSeconds; is.fSeconds = fTrue;
+    fSav2 = us.fSeconds; us.fSeconds = fTrue;
     fprintf(file, "%3d,%5d%s", pci->day, pci->yea, SzTim(pci->tim));
-    us.fAnsiChar = fSav1; is.fSeconds = fSav2;
-    rDst = (pci->dst == dstAuto ? (real)is.fDst : pci->dst);
-    rOff = (pci->zon == zonLMT ? pci->lon / 15.0 : pci->zon) - rDst;
-    if (pci->zon == zonLMT)
+    us.fAnsiChar = fSav1; us.fSeconds = fSav2;
+    rDst = DstReal(pci->dst);
+    rOff = GetOffsetCI(pci);
+    if (pci->zon == zonLMT || pci->zon == zonLAT)
       sprintf(sz, "LMT");
     else {
       sprintf(sz, "   ");
@@ -732,12 +755,18 @@ flag FOutputQuickFile(void)
           break;
         }
     }
+    rLon = RSgn(pci->lon) * (RAbs(pci->lon) + rSmall);
+    rLat = RSgn(pci->lat) * (RAbs(pci->lat) + rSmall);
+    j = (int)(RFract(RAbs(rLon))*60.0);
+    k = (int)(RFract(RAbs(rLat))*60.0);
+    j2 = (int)(RFract(RAbs(rLon))*3600.0) % 60;
+    k2 = (int)(RFract(RAbs(rLat))*3600.0) % 60;
     fprintf(file, " %s%c%02d:%02d", sz, rOff < 0.0 ? '-' : '+',
       (int)RAbs(rOff), (int)(RFract(RAbs(rOff))*60.0+rRound/60.0));
-    fprintf(file, "%03d%c%02d'%02d", (int)RAbs(pci->lon), pci->lon < 0.0 ?
-      'E' : 'W', (int)(RFract(RAbs(pci->lon))*60.0+rRound/60.0), 0);
-    fprintf(file, " %02d%c%02d'%02d", (int)RAbs(pci->lat), pci->lat < 0.0 ?
-      'S' : 'N', (int)(RFract(RAbs(pci->lat))*60.0+rRound/60.0), 0);
+    fprintf(file, "%03d%c%02d'%02d", (int)RAbs(rLon), rLon < 0.0 ?
+      'E' : 'W', j, j2);
+    fprintf(file, " %02d%c%02d'%02d", (int)RAbs(rLat), rLat < 0.0 ?
+      'S' : 'N', k, k2);
     fprintf(file, " %-25.25s \n", pci->loc);
   }
 
@@ -756,7 +785,7 @@ flag FProcessADBFile(CONST char *szFile, FILE *file)
 {
   char szLine[cchSzLine], sz[cchSzDef], szLoc1[cchSzDef], szLoc2[cchSzDef],
     ch, *pch, *pch2;
-  int i, grf, cchSz = CchSz(us.szADB);
+  int i, grf, cchSz = (us.szADB == NULL ? 0 : CchSz(us.szADB));
   flag fHaveFile, fDidOne = fFalse, fDidStart, fDidEnd, fDidLon, fRet = fFalse;
 
   fHaveFile = (file != NULL);
@@ -851,7 +880,7 @@ flag FProcessADBFile(CONST char *szFile, FILE *file)
           ;
         CopyRgchToSz(pch, pch2 - pch, sz, cchSzDef);
         ConvertSzFromUTF8(sz);
-        ciCore.nam = SzCopy(sz);
+        ciCore.nam = SzClone(sz);
         grf |= 256;
       }
       if ((grf & 512) == 0 && fDidLon && FEqRgch(pch, "\">", 2, fFalse)) {
@@ -870,7 +899,7 @@ flag FProcessADBFile(CONST char *szFile, FILE *file)
         CopyRgchToSz(pch, pch2 - pch, szLoc2, cchSzDef);
         sprintf(sz, "%s, %s", szLoc1, szLoc2);
         ConvertSzFromUTF8(sz);
-        ciCore.loc = SzCopy(sz);
+        ciCore.loc = SzClone(sz);
         grf |= 1024;
       }
       if (cchSz > 0 && (grf & 2048) == 0 &&
@@ -981,7 +1010,7 @@ flag FProcessSFTextFile(CONST char *szFile, FILE *file)
       pch[-13] = chNull;
     else if (cch > 15 && FEqRgch(pch - 15, " - Female Chart", 15, fFalse))
       pch[-15] = chNull;
-    ciCore.nam = SzCopy(szLine);
+    ciCore.nam = SzClone(szLine);
     nState = 2;
   } else if (nState == 2) {
     for (pch = szLine; *pch && *pch <= ' '; pch++)
@@ -1013,7 +1042,7 @@ flag FProcessSFTextFile(CONST char *szFile, FILE *file)
     if (FBetween(*pch, 'A', 'Z')) {
       while (*pch > ' ')
         pch++;
-      SS = pch[-2] == 'D' ? 1.0 : 0.0;
+      SS = (pch - szLine >= 3 && FZonDst(&pch[-3]) ? 1.0 : 0.0);
     } else
       SS = 0.0;
     ZZ = RParseSz(pch, pmZon) + SS;
@@ -1032,7 +1061,7 @@ flag FProcessSFTextFile(CONST char *szFile, FILE *file)
       if (pch > szLine && pch[-1] == ',')
         pch[-1] = chNull;
     }
-    ciCore.loc = SzCopy(pch2);
+    ciCore.loc = SzClone(pch2);
     AA = RParseSz(pch, pmLat);
     while (*pch && *pch != ',')
       pch++;
@@ -1067,15 +1096,16 @@ LDone:
 }
 
 
-// Output the chart list in memory to an Astrolog chart list file. (If the
-// chart list is empty, output a length 1 list consisting of the main chart.)
+// Output the chart list in memory to an Astrolog chart list file. If the
+// chart list is empty, output a length zero list.
 
 flag FOutputChartList()
 {
   char sz[cchSzDef], *pch;
   FILE *file;
   CI *pci;
-  int cci, i, nSav;
+  int i, nSav;
+  real dst;
 
   if (us.fNoWrite)
     return fFalse;
@@ -1087,15 +1117,14 @@ flag FOutputChartList()
   }
 
   sprintf(sz, "@AL%s  ; %s chart list.\n", szVerCore, szAppName); PrintF(sz);
-  cci = is.cci <= 0 ? 1 : is.cci;
-  for (i = 0; i < cci; i++) {
-    pci = is.cci <= 0 ? &ciMain : &is.rgci[i];
+  for (i = 0; i < is.cci; i++) {
+    pci = &is.rgci[i];
+    dst = DstReal(pci->dst);
     nSav = us.fAnsiChar;
     us.fAnsiChar = fFalse;
     fprintf(file, "%cqcl %.3s %2d %d %s %s ",
       chSwitch, szMonth[pci->mon], pci->day, pci->yea, SzTim(pci->tim),
-      pci->dst == 0.0 ? "ST" : (pci->dst == 1.0 ? "DT" :
-      SzZone(pci->dst == dstAuto ? (real)is.fDst : pci->dst)));
+      dst == 0.0 ? "ST" : (dst == 1.0 ? "DT" : SzZone(DstReal(dst))));
     fprintf(file, "%s %s ", SzZone(pci->zon), SzLocation(pci->lon, pci->lat));
     // Don't put double quotes within double quoted string parameters.
     putc('"', file);
@@ -1120,7 +1149,7 @@ flag FOutputDaedalusStar()
 {
   char sz[cchSzDef];
   FILE *file = NULL;
-  int i, j = 0, k;
+  int j = 0, k;
   ES es;
 
   if (us.fNoWrite)
@@ -1138,7 +1167,7 @@ flag FOutputDaedalusStar()
   // Output extra stars.
   if (gs.fAllStar) {
     SwissComputeStar(0.0, NULL);
-    for (i = 1; SwissComputeStar(is.T, &es); i++) {
+    while (SwissComputeStar(is.T, &es)) {
       k = 255 - (int)((es.mag - rStarLite) / rStarSpan * 224.0);
       k = Min(k, 255); k = Max(k, 32);
       fprintf(file, "SetStar 1 %d %d %d GrayN %d\n", j,
@@ -1151,7 +1180,7 @@ flag FOutputDaedalusStar()
   // Output extra asteroids.
   if (gs.nAstLo > 0) {
     SwissComputeAsteroid(0.0, NULL, fFalse);
-    for (i = gs.nAstLo; SwissComputeAsteroid(is.T, &es, fFalse); i++) {
+    while (SwissComputeAsteroid(is.T, &es, fFalse)) {
       k = 255 - (int)((es.mag - rStarLite) / rStarSpan * 224.0);
       k = Min(k, 255); k = Max(k, 32);
       fprintf(file, "SetStar 1 %d %d %d GrayN %d\n", j,
@@ -1161,6 +1190,19 @@ flag FOutputDaedalusStar()
     }
   }
 
+  // Output exoplanets.
+  if (gs.fAllExo) {
+    EnumExoplanets(NULL);
+    while (EnumExoplanets(&es)) {
+      EquToEcl(&es.lon, &es.lat);
+      fprintf(file, "SetStar 1 %d %d %d %s\n", j,
+        (int)(Mod(270.0 - es.lon) * 32768.0 / rDegMax),
+        (int)((es.lat + rDegQuad) * 32768.0 / rDegHalf),
+        es.ki == kGreen ? "Green" : "Maize");
+      j++;
+    }
+  }
+
   fprintf(file, "nStarCount %d\nnStarSize 384\nfStars True\n", j);
   fclose(file);
   return fTrue;
@@ -1175,9 +1217,9 @@ CONST char *szTypSwiss[] = {"", "b", "O", "m", "j"};
 CONST char *szPntSwiss[] = {"", "n", "s", "p", "a"};
 
 // Take many of the user visible settings, and write them out to a new command
-// switch file, which may be read in to restore those settings. Most often
-// this would be used to create a new astrolog.as default settings file. This
-// is called from the -od switch and the File / Save Settings menu command.
+// switch file, which may be read in to restore those settings. Usually this
+// is used to create a new astrolog.as default settings file. This is called
+// from the -od switch and the File / Save Program Settings menu command.
 
 flag FOutputSettings()
 {
@@ -1203,30 +1245,30 @@ flag FOutputSettings()
     szVerCore, szAppName, szVersionCore, DEFAULT_INFOFILE); PrintFSz();
   PrintF(
     "; The contents of this file can be automatically generated with the\n"
-    "; \"File / Save Settings\" menu command, or with the -od command switch."
-    "\n\n");
+    "; \"File / Save Program Settings\" menu command, or with the -od "
+    "command switch.\n\n");
 
-  f1 = is.fSeconds; is.fSeconds = fTrue;
-  sprintf(sz, "-z %s                ", SzZone(us.zonDef)); PrintFSz();
-  is.fSeconds = f1;
+  f1 = us.fSeconds; us.fSeconds = fTrue;
+  sprintf(sz, "-z %s                ", SzZone(ciDefa.zon)); PrintFSz();
+  us.fSeconds = f1;
   PrintF("; Default time zone     [hours W or E of UTC   ]\n");
-  if (us.dstDef != dstAuto)
-    sprintf(sz, "-z0 %d                   ", (int)us.dstDef);
+  if (ciDefa.dst != dstAuto)
+    sprintf(sz, "-z0 %d                   ", (int)ciDefa.dst);
   else
     sprintf(sz, "-z0 Autodetect          ");
   PrintFSz();
   PrintF("; Default Daylight time [0 standard, 1 daylight]\n");
   f1 = us.fAnsiChar; us.fAnsiChar = 3;
-  f2 = is.fSeconds; is.fSeconds = fTrue;
-  sprintf(sz, "-zl %s  ", SzLocation(us.lonDef, us.latDef)); PrintFSz();
-  us.fAnsiChar = f1; is.fSeconds = f2;
+  f2 = us.fSeconds; us.fSeconds = fTrue;
+  sprintf(sz, "-zl %s  ", SzLocation(ciDefa.lon, ciDefa.lat)); PrintFSz();
+  us.fAnsiChar = f1; us.fSeconds = f2;
   PrintF("; Default location      [longitude and latitude]\n");
   sprintf(sz, "-zv %s               ", SzElevation(us.elvDef)); PrintFSz();
   PrintF("; Default elevation     [in feet or meters     ]\n");
   sprintf(sz, "-zf %s                 ", SzTemperature(us.tmpDef)); PrintFSz();
   PrintF("; Default temperature   [in Fahren. or Celsius ]\n");
   sprintf(sz, "-zj \"%s\" \"%s\" ; Default name and location\n\n",
-    us.namDef, us.locDef); PrintFSz();
+    SzSet(ciDefa.nam), SzSet(ciDefa.loc)); PrintFSz();
 
   // The -Yz line needs to be before the -n line in order to take effect.
   sprintf(sz, "-Yz %ld   ", us.lTimeAddition); PrintFSz();
@@ -1272,12 +1314,21 @@ flag FOutputSettings()
   sprintf(sz, "%cb0     ", ChDashF(us.fSeconds)); PrintFSz();
   PrintF(
     "; Print zodiac seconds      [\"_b0\" to minute, \"=b0\" to second  ]\n");
+  sprintf(sz, "%cb1     ", ChDashF(us.fSecond1K)); PrintFSz();
+  PrintF(
+    "; Print zodiac milliseconds [\"_b1\" to second, \"=b1\" to millisec]\n");
   sprintf(sz, "%cb      ", ChDashF(us.fEphemFiles)); PrintFSz();
   PrintF(
     "; Use ephemeris files       [\"=b\" uses them, \"_b\" doesn't      ]\n");
   sprintf(sz, "%c0b     ", ChDashF(us.fNoPlacalc)); PrintFSz();
   PrintF(
     "; Disable old calculations  [\"=0b\" disables them, \"_0b\" allows ]\n");
+  sprintf(sz, "%cv0     ", ChDashF(us.fVelocity)); PrintFSz();
+  PrintF(
+    "; Show average velocities   [\"=v0\" average, \"_v0\" does absolute]\n");
+  sprintf(sz, "=v3 %d   ", us.fListDecan ? us.nDecanType : 0); PrintFSz();
+  PrintF(
+    "; Wheel subdivision type    [Change \"0\" to desired subdivision ]\n");
   sprintf(sz, ":w %d    ", us.nWheelRows); PrintFSz();
   PrintF(
     "; Wheel chart text rows     [Change \"0\" to desired wheel rows  ]\n");
@@ -1290,6 +1341,9 @@ flag FOutputSettings()
   sprintf(sz, "%csr0    ", ChDashF(us.fEquator2)); PrintFSz();
   PrintF(
     "; Latitudes or declinations [\"_sr0\" shows lat., \"=sr0\" declin. ]\n");
+  sprintf(sz, ":gs %d   ", us.nAppSep); PrintFSz();
+  PrintF("; Aspect orb type           "
+    "[\"0\" +/-, \"1\" app/sep, \"2\" wax/wan ]\n");
   sprintf(sz, "%cYs     ", ChDashF(us.fSidereal2)); PrintFSz();
   PrintF(
     "; Use plane of solar system [\"_Ys\" is ecliptic, \"=Ys\" is solar ]\n");
@@ -1300,9 +1354,6 @@ flag FOutputSettings()
   PrintFSz();
   PrintF(
     "; Show eclipse information  [\"=Yu0\" shows, \"_Yu0\" doesn't show ]\n");
-  sprintf(sz, "%cYr     ", ChDashF(us.fRound)); PrintFSz();
-  PrintF(
-    "; Show rounded positions    [\"=Yr\" rounds, \"_Yr\" doesn't       ]\n");
   sprintf(sz, "%cYd     ", ChDashF(us.fEuroDate)); PrintFSz();
   PrintF(
     "; European date format      [\"_Yd\" is M/D/Y, \"=Yd\" is D-M-Y    ]\n");
@@ -1312,6 +1363,9 @@ flag FOutputSettings()
   sprintf(sz, "%cYv     ", ChDashF(us.fEuroDist)); PrintFSz();
   PrintF(
     "; European length units     [\"_Yv\" is imperial, \"=Yv\" is metric]\n");
+  sprintf(sz, "%cYr     ", ChDashF(us.fRound)); PrintFSz();
+  PrintF(
+    "; Show rounded positions    [\"=Yr\" rounds, \"_Yr\" doesn't       ]\n");
   sprintf(sz, "%cYC     ", ChDashF(us.fSmartCusp)); PrintFSz();
   PrintF(
     "; Smart cusp displays       [\"=YC\" is smart, \"_YC\" is normal   ]\n");
@@ -1324,6 +1378,9 @@ flag FOutputSettings()
   sprintf(sz, "-Ya%d    ", us.nCharset); PrintFSz();
   PrintF(
     "; Input character encoding  [0-3 is Default, IBM, Latin-1, UTF8]\n");
+  sprintf(sz, "%cYz1    ", ChDashF(us.fOffsetOnly)); PrintFSz();
+  PrintF(
+    "; Combine DST and time zone [\"=Yz1\" combines, \"_Yz1\" doesn't   ]\n");
   sprintf(sz, "-YP %d   ", us.nArabicNight); PrintFSz();
   PrintF(
     "; Arabic part formula       [\"1\" is fixed, \"0\" checks if night ]\n");
@@ -1331,8 +1388,10 @@ flag FOutputSettings()
   PrintF(
     "; Internet Web queries      [\"=0n\" disables them, \"_0n\" allows ]\n");
 
+  PrintF("\n-Yw "); FormatR(sz, us.rStation, 5); PrintFSz();
+  PrintF("       ; Stationary movement threshold  [0.0 is never \"S\"]\n");
   sprintf(sz,
-    "\n:pd %9.5f ; Progression degrees per day    [365 is secondary]\n",
+    ":pd %9.5f ; Progression degrees per day    [365 is secondary]\n",
     us.rProgDay); PrintFSz();
   PrintF(":pC "); FormatR(sz, us.rProgCusp, 5); PrintFSz();
   PrintF("       ; Progressed cusp movement ratio [1.0 is quotidian]\n");
@@ -1340,7 +1399,7 @@ flag FOutputSettings()
   PrintF("\n\n; FILE PATHS (-Yi1 through -Yi9):\n; For example, "
     "point -Yi1 to ephemeris dir, -Yi2 to chart files dir, etc.\n\n");
   for (i = 0; i < 10; i++)
-    if (us.rgszPath[i] && *us.rgszPath[i]) {
+    if (FSzSet(us.rgszPath[i])) {
       sprintf(sz, "-Yi%d \"%s\"\n", i, us.rgszPath[i]); PrintFSz();
     }
 
@@ -1350,7 +1409,8 @@ flag FOutputSettings()
     "; 22-33: Asc 2nd 3rd Nad 5th 6th Des 8th 9th MC 11th 12th\n"
     "; 34-42: Vul Cup Had Zeu Kro Apo Adm Vulk Pos\n"
     "; 43-51: Hyg Pho Eri Hau Mak Gon Qua Sed Orc\n"
-    "; 52-83: Planetary moons\n"
+    "; 52-78: Planetary moons\n"
+    "; 79-83: Planetary centers of body\n"
     "; 84-133: Fixed stars\n"
     "\n-YR 0 10     ");
   for (i = 0; i <= 10; i++) PrintF(SzNumF(ignore[i]));
@@ -1366,7 +1426,7 @@ flag FOutputSettings()
   for (i = 52; i <= 78; i++) PrintF(SzNumF(ignore[i]));
   PrintF(" ; Moons\n-YR 79 83    ");
   for (i = 79; i <= 83; i++) PrintF(SzNumF(ignore[i]));
-  PrintF("               ; Centers of Body\n-YR 84 108   ");
+  PrintF("               ; Centers of body\n-YR 84 108   ");
   for (i = 84; i <= 108; i++) PrintF(SzNumF(ignore[i]));
   PrintF(" ; Fixed stars\n-YR 109 133  ");
   for (i = 109; i <= 133; i++) PrintF(SzNumF(ignore[i]));
@@ -1402,7 +1462,7 @@ flag FOutputSettings()
     SzNumF(us.fIgnoreAlt0), SzNumF(us.fIgnoreDisequ)); PrintFSz();
   PrintF("-YR7 ");
   for (i = 0; i < rrMax; i++) PrintF(SzNumF(ignore7[i]));
-  PrintF(" ; Restrict rulerships: std, esoteric, hierarch, exalt, ray\n-YRZ ");
+  PrintF(" ; Restrict rulerships: std, esoteric, hierarch, exalt, Ray\n-YRZ ");
   for (i = 0; i < arMax; i++) PrintF(SzNumF(ignorez[i]));
   PrintF("   ; Restrict angle events: rising, zenith, setting, nadir\n\n\n");
 
@@ -1504,6 +1564,16 @@ flag FOutputSettings()
     rHouseInf[cSign + 4], rHouseInf[cSign + 5]); PrintFSz();
   PrintF(" ; In Esoteric, Hierarchical, Ray ruling (signs, houses)\n\n\n");
 
+  PrintF("; DEFAULT RULERSHIPS & EXALTATIONS:\n\n");
+  for (i = 1; i <= 10; i++)
+    { sprintf(sz, "-YJ %.3s %.3s %.3s\n", szObjName[i], szSignName[ruler1[i]],
+    ruler2[i] <= 0 ? "0" : szSignName[ruler2[i]]); PrintFSz(); }
+  PrintF("\n");
+  for (i = 1; i <= 10; i++)
+    { sprintf(sz, "-YJ0 %.3s %.3s\n", szObjName[i],
+    exalt[i] <= 0 ? "0" : szSignName[exalt[i]]); PrintFSz(); }
+  PrintF("\n\n");
+
   PrintF("; DEFAULT RAYS:\n\n-Y7C 1 12  ");
   for (i = 1; i <= cSign; i++)
     { sprintf(sz, " %d", rgSignRay[i]); PrintFSz(); }
@@ -1642,12 +1712,24 @@ flag FOutputSettings()
   sprintf(sz, "%cXu              ", ChDashF(gs.fBorder)); PrintFSz();
   PrintF(
     "; Chart border  [\"=Xu\" shows border, \"_Xu\" doesn't show     ]\n");
-  sprintf(sz, ":Xv %d            ", gs.nDecaFill); PrintFSz();
-  PrintF(
-    "; Wheel fill    [\"0\" for none, \"1\" for standard, \"2\" rainbow]\n");
   sprintf(sz, "%cXx              ", ChDashF(gs.fThick)); PrintFSz();
   PrintF(
     "; Thicker lines [\"=Xx\" is thicker, \"_Xx\" is thinner         ]\n");
+  sprintf(sz, "%cXx0             ", ChDashF(gs.fAntialias)); PrintFSz();
+  PrintF(
+    "; Antialiasing  [\"=Xx0\" is antialiased lines, \"_Xx0\" is not ]\n");
+  sprintf(sz, "%cXA              ", ChDashF(gs.fLabelAsp)); PrintFSz();
+  PrintF(
+    "; Glyphed lines [\"=XA\" glyphs on aspect lines, \"_XA\" doesn't]\n");
+  sprintf(sz, "%cXL              ", ChDashF(gs.fLabelCity)); PrintFSz();
+  PrintF(
+    "; Show cities   [\"=XL\" shows them in charts, \"_XL\" doesn't  ]\n");
+  sprintf(sz, "%cXv0             ", ChDashF(gs.fDoSidebar)); PrintFSz();
+  PrintF(
+    "; Show sidebar  [\"=Xv0\" shows on right edge, \"_Xv0\" doesn't ]\n");
+  sprintf(sz, ":Xv %d            ", gs.nDecaFill); PrintFSz();
+  PrintF(
+    "; Wheel fill    [\"0\" for none, \"1\" for standard, \"2\" rainbow]\n");
   sprintf(sz, ":Xb%c             ", gi.fBmp ? 'w' : ChUncap(gs.chBmpMode));
   PrintFSz();
   PrintF(
@@ -1661,7 +1743,7 @@ flag FOutputSettings()
   sprintf(sz, ":YXj %d           ", gs.cspace); PrintFSz();
   PrintF("; Orbit trail count\n");
   sprintf(sz, ":YX7 %d         ", gs.nRayWidth); PrintFSz();
-  PrintF("; Esoteric ray column influence width\n");
+  PrintF("; Esoteric Ray column influence width\n");
   sprintf(sz, ":YXf %06d      ", gs.nFontAll); PrintFSz();
   PrintF("; Fonts to use [text, signs, houses, planets, aspects, naks.]\n");
   sprintf(sz, ":YXp %d           ", gs.nOrient); PrintFSz();
@@ -1691,7 +1773,7 @@ void OpenDir(CONST char *szDir)
   long hFile;
   flag fAll = us.fWriteOld, fHaveFile;
 
-  // Only look at *.as (or *.dat) extension files.
+  // Only look at *.as extension files.
   sprintf(szPath, "%s\\*.%s", szDir, !fAll ? "as" : "*");
   hFile = _findfirst(szPath, &fi);
   fHaveFile = (hFile != -1);
@@ -1836,6 +1918,9 @@ int NParseSz(CONST char *szEntry, int pm)
       for (i = 1; i <= cAspect2; i++)
         if (FMatchSz(sz, szAspectAbbrev[i]))
           return i;
+      for (i = 1; i <= cAspect2; i++)
+        if (FMatchSz(sz, szAspectName[i]))
+          return i;
       i = SzLookup(rgAspectName, szEntry);
       if (i >= 0)
         return i;
@@ -2009,13 +2094,15 @@ real RParseSz(CONST char *szEntry, int pm)
       }
     }
   } else if (pm == pmOffset) {
+    // For sidereal offsets, check for ayanamsa names.
     for (i = 0; *rgZodiacOffset[i].sz; i++)
       if (FMatchSz(sz, rgZodiacOffset[i].sz))
         return rgZodiacOffset[i].r;
   }
 
   // Anything still at this point should be in a numeric format.
-  if (!FNumCh(ch) && ch != '+' && ch != '-' && ch != '.')
+  if (!(FNumCh(ch) || ch == '.' || ch == '+' ||
+    (ch == '-' && (FNumCh(sz[1]) || sz[1] == '.'))))
     return rLarge;
   r = atof(sz);
   if (r < 0.0) {
@@ -2168,7 +2255,7 @@ flag FInputData(CONST char *szFile)
 {
   FILE *file;
   char sz[cchSzMax], ch, *pch;
-  int i, fT;
+  int i;
   real k, l, m;
 
   // If we are to read from the virtual file "nul" that means to leave the
@@ -2197,6 +2284,11 @@ flag FInputData(CONST char *szFile)
     ciCore = ciGreg;
     return fTrue;
   }
+  if (FEqSz(szFile, "__d")) {
+    is.fHaveInfo = fTrue;
+    ciCore = ciDefa;
+    return fTrue;
+  }
 
   // If we are to read from the virtual file "__1" through "__6" then that
   // means copy the chart information from the specified chart slot.
@@ -2216,9 +2308,8 @@ flag FInputData(CONST char *szFile)
 
   if (FEqSz(szFile, szNowCore)) {
     is.fHaveInfo = fTrue;
-    SS = us.dstDef; ZZ = us.zonDef; OO = us.lonDef; AA = us.latDef;
+    ciCore = ciDefa;
     GetTimeNow(&MM, &DD, &YY, &TT, SS, ZZ);
-    ciCore.nam = us.namDef; ciCore.loc = us.locDef;
     return fTrue;
   }
 #endif
@@ -2246,7 +2337,7 @@ flag FInputData(CONST char *szFile)
       1, 12, pmMon);
     DD = NInputRange("Enter day   for chart (e.g. '1' '31') ",
       1, DayInMonth(MM, 0), pmDay);
-    YY = NInputRange("Enter year  for chart (e.g. '2023')   ",
+    YY = NInputRange("Enter year  for chart (e.g. '2025')   ",
       -32000, 32000, pmYea);
     if (FBetween(YY, 0, 99)) {
       sprintf(sz,
@@ -2258,9 +2349,9 @@ flag FInputData(CONST char *szFile)
       -2.0, 24.0, pmTim);
 #ifdef ATLAS
     InputString("Enter name of city or location", sz);
-    ciCore.loc = SzCopy(sz);
+    ciCore.loc = SzClone(sz);
     if (DisplayAtlasLookup(sz, 0, &i)) {
-      ciCore.loc = SzCopy(sz);      // DisplayAtlasLookup changes ciCore.loc.
+      ciCore.loc = SzClone(sz);      // DisplayAtlasLookup changes ciCore.loc.
       if (DisplayTimezoneChanges(is.rgae[i].izn, 0, &ciCore)) {
         sprintf(sz, "Atlas data for %s: (%cT Zone %s) %s\n", SzCity(i),
           ChDst(SS), SzZone(ZZ),
@@ -2280,10 +2371,10 @@ flag FInputData(CONST char *szFile)
     AA = RInputRange("Enter Latitude  of place (e.g. '47N36') ",
       -rDegQuad, rDegQuad, pmLat);
     InputString("Enter name or title for chart ", sz);
-    ciCore.nam = SzCopy(sz);
+    ciCore.nam = SzClone(sz);
 #ifndef ATLAS
     InputString("Enter name of city or location", sz);
-    ciCore.loc = SzCopy(sz);
+    ciCore.loc = SzClone(sz);
 #endif
     PrintL();
     is.cchRow = 0;
@@ -2298,7 +2389,6 @@ flag FInputData(CONST char *szFile)
   file = FileOpen(szFile, 1, NULL);
   if (file == NULL)
     return fFalse;
-  is.fHaveInfo = fTrue;
   if (!fgets(sz, cchSzMax, file))
     return fFalse;
   ch = sz[0];
@@ -2311,38 +2401,41 @@ flag FInputData(CONST char *szFile)
   // Read the chart parameters from a standard command switch file.
 
   if (ch == '@') {
-    fT = is.fSzPersist; is.fSzPersist = fFalse;
     if (!FProcessSwitchFile(szFile, file))
       return fFalse;
-    is.fSzPersist = fT;
 
   // Read the chart parameters from an Astrological Exchange (AAF) file.
 
   } else if (ch == '#') {
+    is.fHaveInfo = fTrue;
     if (!FProcessAAFFile(szFile, file))
       return fFalse;
 
   // Read the chart parameters from a Quick*Chart file.
 
   } else if (i == 100 || i == 101) {
+    is.fHaveInfo = fTrue;
     if (!FProcessQuickFile(szFile, file))
       return fFalse;
 
   // Read the chart parameters from a Astrodatabank file.
 
   } else if (ch == '<') {
+    is.fHaveInfo = fTrue;
     if (!FProcessADBFile(szFile, file))
       return fFalse;
 
   // Read the chart parameters from a Solar Fire export text file.
 
   } else if (ch == '\n') {
+    is.fHaveInfo = fTrue;
     if (!FProcessSFTextFile(szFile, file))
       return fFalse;
 
   // Read the chart info from an older style -o list of seven numbers.
 
   } else if (FNumCh(ch)) {
+    is.fHaveInfo = fTrue;
     SS = 0.0;
     fscanf(file, "%d%d%d", &MM, &DD, &YY);
     fscanf(file, "%lf%lf%lf%lf", &TT, &ZZ, &OO, &AA);
@@ -2368,12 +2461,13 @@ flag FInputData(CONST char *szFile)
       }
       if (*pch == '/' || *pch == '\\')
         pch++;
-      ciCore.nam = SzCopy(pch);
+      ciCore.nam = SzClone(pch);
     }
 
   // Read the actual chart positions from a file produced with -o0 switch.
 
   } else if (ch == 'S') {
+    is.fHaveInfo = fTrue;
     MM = -1;
 
     // Hack: A negative month value means the chart parameters are invalid,
@@ -2473,12 +2567,12 @@ flag GetJPLHorizons(int id, real *obj, real *objalt, real *dir, real *dist,
   real *diralt, real *dirlen, char *szOut)
 {
   char szUrl[cchSzLine], szLine[cchSzLine], szName[cchSzMax],
-    szMon[3][4], hr[3], min[3], *pch, *pch2, ch;
+    szMon[3][4], *pch, *pch2, ch;
   CI ci[3];
   PT3R pt[3];
   FILE *file;
   real sec[3], len[3], rT;
-  int phase = -1, i;
+  int hr[3], min[3], phase = -1, i;
   flag fSemicolon;
 
   if (us.fNoNetwork) {    // Don't allow if -0n set.
@@ -2502,7 +2596,7 @@ flag GetJPLHorizons(int id, real *obj, real *objalt, real *dir, real *dist,
       *pch = ChCap(*pch);
     hr[i] = NFloor(ci[i].tim);
     min[i] = (int)(RFract(RAbs(ci[i].tim))*60.0);
-    sec[i] = (int)(RFract(RAbs(ci[i].tim))*3600.0) % 60;
+    sec[i] =  RMod(RFract(RAbs(ci[i].tim))*3600.0 + rSmall, 60.0);
   }
 
   // Compose URL to download from internet.
