@@ -1,8 +1,8 @@
 /*
-** Astrolog (Version 7.60) File: placalc.h
+** Astrolog (Version 7.80) File: placalc.h
 **
 ** IMPORTANT NOTICE: Astrolog and all chart display routines and anything
-** not enumerated below used in this program are Copyright (C) 1991-2023 by
+** not enumerated below used in this program are Copyright (C) 1991-2025 by
 ** Walter D. Pullen (Astara@msn.com, http://www.astrolog.org/astrolog.htm).
 ** Permission is granted to freely use, modify, and distribute these
 ** routines provided these credits and notices remain unmodified with any
@@ -10,8 +10,8 @@
 **
 ** The main ephemeris databases and calculation routines are from the
 ** library SWISS EPHEMERIS and are programmed and copyright 1997-2008 by
-** Astrodienst AG. The use of that source code is subject to the license for
-** Swiss Ephemeris Free Edition, available at http://www.astro.com/swisseph.
+** Astrodienst AG. Use of that source code is subject to license for Swiss
+** Ephemeris Free Edition at https://www.astro.com/swisseph/swephinfo_e.htm.
 ** This copyright notice must not be changed or removed by any user of this
 ** program.
 **
@@ -48,7 +48,7 @@
 ** Initial programming 8/28-30/1991.
 ** X Window graphics initially programmed 10/23-29/1991.
 ** PostScript graphics initially programmed 11/29-30/1992.
-** Last code change made 4/8/2023.
+** Last code change made 6/19/2025.
 */
 
 #include "astrolog.h"
@@ -256,7 +256,7 @@ extern REAL8 test_near_zero(REAL8);
 extern REAL8 deltat(double);
 extern void to_mean_ekl();
 extern int afl2planet();
-extern void disturb(register struct kor *, REAL8 *, REAL8 *, REAL8, REAL8,
+extern void disturb(struct kor *, REAL8 *, REAL8 *, REAL8, REAL8,
   REAL8);
 extern REAL8 fnu(REAL8, REAL8, REAL8);
 
