@@ -1,8 +1,8 @@
 /*
-** Astrolog (Version 7.60) File: wdialog.cpp
+** Astrolog (Version 7.80) File: wdialog.cpp
 **
 ** IMPORTANT NOTICE: Astrolog and all chart display routines and anything
-** not enumerated below used in this program are Copyright (C) 1991-2023 by
+** not enumerated below used in this program are Copyright (C) 1991-2025 by
 ** Walter D. Pullen (Astara@msn.com, http://www.astrolog.org/astrolog.htm).
 ** Permission is granted to freely use, modify, and distribute these
 ** routines provided these credits and notices remain unmodified with any
@@ -10,8 +10,8 @@
 **
 ** The main ephemeris databases and calculation routines are from the
 ** library SWISS EPHEMERIS and are programmed and copyright 1997-2008 by
-** Astrodienst AG. The use of that source code is subject to the license for
-** Swiss Ephemeris Free Edition, available at http://www.astro.com/swisseph.
+** Astrodienst AG. Use of that source code is subject to license for Swiss
+** Ephemeris Free Edition at https://www.astro.com/swisseph/swephinfo_e.htm.
 ** This copyright notice must not be changed or removed by any user of this
 ** program.
 **
@@ -48,7 +48,7 @@
 ** Initial programming 8/28-30/1991.
 ** X Window graphics initially programmed 10/23-29/1991.
 ** PostScript graphics initially programmed 11/29-30/1992.
-** Last code change made 4/8/2023.
+** Last code change made 6/19/2025.
 */
 
 #include "astrolog.h"
@@ -68,6 +68,8 @@ void SetEditSz(HWND hdlg, int id, CONST char *sz)
   char sz2[cchSzDef];
   CONST char *pch;
 
+  if (sz == NULL)
+    sz = "";
   while (*sz == ' ')    // Strip off any extra leading spaces.
     sz++;
   pch = ConvertSzToLatin(sz, sz2, cchSzDef);
@@ -113,7 +115,7 @@ void SetEditMDYT(HWND hdlg, int idMon, int idDay, int idYea, int idTim,
     sprintf(sz, "%d", Max(i, 1)); SetCombo(idDay, sz);
   }
   SetEditN(idYea, yea);
-  for (i = 2015; i <= 2025; i++) {
+  for (i = 2020; i <= 2030; i++) {
     sprintf(sz, "%d", i); SetCombo(idYea, sz);
   }
   sprintf(sz, "%s", SzTim(tim));
@@ -132,8 +134,7 @@ void SetEditSZOA(HWND hdlg, int idDst, int idZon, int idLon, int idLat,
   real dst, real zon, real lon, real lat)
 {
   char sz[cchSzDef];
-  int i;
-  flag fT;
+  int i, nSav;
 
   ClearCombo(idDst);
   ClearCombo(idZon);
@@ -161,10 +162,10 @@ void SetEditSZOA(HWND hdlg, int idDst, int idZon, int idLon, int idLat,
       SetCombo(idZon, sz);
     }
   }
-  fT = us.fAnsiChar; us.fAnsiChar = fFalse;
+  nSav = us.fAnsiChar; us.fAnsiChar = fFalse;
   sprintf(sz, "%s", SzLocation(lon, lat));
-  us.fAnsiChar = fT;
-  i = 7 + is.fSeconds*3;
+  us.fAnsiChar = nSav;
+  i = 7 + VSeconds(0, 3, 7);
   sz[i] = chNull;
   SetEditSz(hdlg, idLon, &sz[0]);
   SetCombo(idLon, "122W20"); SetCombo(idLon, "0E00");
@@ -510,7 +511,8 @@ flag API DlgSaveChart()
 
   // Saving chart info, position, or setting command files can be done here.
   // Saving actual chart output isn't done until the next redraw.
-  is.szFileOut = gi.szFileOut = ofn.lpstrFile;
+  FCloneSz(ofn.lpstrFile, &is.szFileOut);
+  FCloneSz(ofn.lpstrFile, &gi.szFileOut);
   switch (wi.wCmd) {
   case cmdSaveChart:
     us.nWriteFormat = 0;
@@ -522,7 +524,7 @@ flag API DlgSaveChart()
     us.nWriteFormat = 'l';
     return FOutputData();
   case cmdSaveText:
-    is.szFileScreen = ofn.lpstrFile;
+    FCloneSz(ofn.lpstrFile, &is.szFileScreen);
     us.fGraphics = fFalse;
     wi.fRedraw = fTrue;
     break;
@@ -757,8 +759,8 @@ flag API DlgList(HWND hdlg, uint message, WORD wParam, LONG lParam)
 {
   DLGPROC dlgproc;
   char sz[cchSzLine], sz1[cchSzDef], sz2[cchSzDef];
-  int i, i2, j;
-  flag fT;
+  int i, i2, j, nSav;
+  flag fSav;
   CI ciT, *pci;
   HCURSOR hcurPrev = NULL;
   static flag fFilter = fFalse;
@@ -809,13 +811,14 @@ flag API DlgList(HWND hdlg, uint message, WORD wParam, LONG lParam)
 #endif
       }
       j = DayOfWeek(pci->mon, pci->day, pci->yea);
-      fT = us.fAnsiChar; us.fAnsiChar = fFalse;
+      nSav = us.fAnsiChar; us.fAnsiChar = 2;
+      fSav = us.fGraphics; us.fGraphics = fTrue;
       sprintf(sz, "%.3s %s %s (%cT Zone %s) %s %s%s%s", szDay[j],
         SzDate(pci->mon, pci->day, pci->yea, 3), SzTim(pci->tim),
         ChDst(pci->dst), SzZone(pci->zon), SzLocation(pci->lon, pci->lat),
         pci->nam, FSzSet(pci->nam) && FSzSet(pci->loc) ? "; " : "",
         pci->loc);
-      us.fAnsiChar = fT;
+      us.fAnsiChar = nSav; us.fGraphics = fSav;
       SetListN(dlLi, sz, i, j);
       i2++;
     }
@@ -1028,7 +1031,7 @@ LInit:
     SetEditMDYT(hdlg, dcInMon, dcInDay, dcInYea, dcInTim,
       ci.mon, ci.day, ci.yea, ci.tim);
     SetEditSZOA(hdlg, dcInDst, dcInZon, dcInLon, dcInLat,
-      ci.dst != dstAuto ? ci.dst : (real)is.fDst, ci.zon, ci.lon, ci.lat);
+      DstReal(ci.dst), ci.zon, ci.lon, ci.lat);
     SetFocus(GetDlgItem(hdlg, dcInMon));
 #ifndef ATLAS
     ShowWindow(GetDlgItem(hdlg, dbInCity), SW_HIDE);
@@ -1043,9 +1046,9 @@ LInit:
     if (wParam == dbInNow || wParam == dbInSet) {
 #ifdef TIME
       if (wParam == dbInNow) {
-        GetTimeNow(&ci.mon, &ci.day, &ci.yea, &ci.tim, us.dstDef, us.zonDef);
-        ci.dst = us.dstDef; ci.zon = us.zonDef;
-        ci.lon = us.lonDef; ci.lat = us.latDef;
+        GetTimeNow(&ci.mon, &ci.day, &ci.yea, &ci.tim, ciDefa.dst, ciDefa.zon);
+        ci.dst = ciDefa.dst; ci.zon = ciDefa.zon;
+        ci.lon = ciDefa.lon; ci.lat = ciDefa.lat;
       } else
 #endif
         ci = ciSave;
@@ -1068,12 +1071,12 @@ LInit:
       EnsureN(ci.yea, FValidYea(ci.yea), "year");
       EnsureN(ci.day, FValidDay(ci.day, ci.mon, ci.yea), "day");
       EnsureR(ci.tim, FValidTim(ci.tim), "time");
-      EnsureR(ci.dst, FValidZon(ci.dst), "daylight saving");
+      EnsureR(ci.dst, FValidDst(ci.dst), "daylight saving");
       EnsureR(ci.zon, FValidZon(ci.zon), "time zone");
       EnsureR(ci.lon, FValidLon(ci.lon), "longitude");
       EnsureR(ci.lat, FValidLat(ci.lat), "latitude");
-      GetEdit(deInNam, sz); ci.nam = SzCopy(sz);
-      GetEdit(deInLoc, sz); ci.loc = SzCopy(sz);
+      GetEdit(deInNam, sz); ci.nam = SzClone(sz);
+      GetEdit(deInLoc, sz); ci.loc = SzClone(sz);
       if (wi.nDlgChart >= 1) {
         *rgpci[wi.nDlgChart] = ci;
         if (wi.nDlgChart == 1)
@@ -1105,7 +1108,7 @@ flag API DlgDefault(HWND hdlg, uint message, WORD wParam, LONG lParam)
   switch (message) {
   case WM_INITDIALOG:
     SetEditSZOA(hdlg, dcDeDst, dcDeZon, dcDeLon, dcDeLat,
-      us.dstDef, us.zonDef, us.lonDef, us.latDef);
+      ciDefa.dst, ciDefa.zon, ciDefa.lon, ciDefa.lat);
     SetCombo(dcDeDst, "Autodetect");
     SetEditN(dcDeCor, (int)us.lTimeAddition);
     SetCombo(dcDeCor, "60"); SetCombo(dcDeCor, "0"); SetCombo(dcDeCor, "-60");
@@ -1113,8 +1116,8 @@ flag API DlgDefault(HWND hdlg, uint message, WORD wParam, LONG lParam)
     SetCombo(dcDeElv, "0m"); SetCombo(dcDeElv, "1000ft");
     SetEditSz(hdlg, dcDeTmp, SzTemperature(us.tmpDef));
     SetCombo(dcDeTmp, "0C"); SetCombo(dcDeTmp, "32F");
-    SetEditSz(hdlg, deDeNam, us.namDef);
-    SetEditSz(hdlg, deDeLoc, us.locDef);
+    SetEditSz(hdlg, deDeNam, ciDefa.nam);
+    SetEditSz(hdlg, deDeLoc, ciDefa.loc);
     SetFocus(GetDlgItem(hdlg, dcDeDst));
 #ifndef ATLAS
     ShowWindow(GetDlgItem(hdlg, dbInCity), SW_HIDE);
@@ -1127,6 +1130,7 @@ flag API DlgDefault(HWND hdlg, uint message, WORD wParam, LONG lParam)
 
   case WM_COMMAND:
     if (wParam == IDOK) {
+      ci = ciDefa;
       GetEdit(dcDeDst, sz); ci.dst = RParseSz(sz, pmDst);
       GetEdit(dcDeZon, sz); ci.zon = RParseSz(sz, pmZon);
       GetEdit(dcDeLon, sz); ci.lon = RParseSz(sz, pmLon);
@@ -1134,14 +1138,13 @@ flag API DlgDefault(HWND hdlg, uint message, WORD wParam, LONG lParam)
       GetEdit(dcDeCor, sz); us.lTimeAddition = NFromSz(sz);
       GetEdit(dcDeElv, sz); us.elvDef = RParseSz(sz, pmElv);
       GetEdit(dcDeTmp, sz); us.tmpDef = RParseSz(sz, pmTmp);
-      EnsureR(ci.dst, FValidZon(ci.dst), "daylight saving");
+      EnsureR(ci.dst, FValidDst(ci.dst), "daylight saving");
       EnsureR(ci.zon, FValidZon(ci.zon), "time zone");
       EnsureR(ci.lon, FValidLon(ci.lon), "longitude");
       EnsureR(ci.lat, FValidLat(ci.lat), "latitude");
-      us.dstDef = ci.dst; us.zonDef = ci.zon;
-      us.lonDef = ci.lon; us.latDef = ci.lat;
-      GetEdit(deDeNam, sz); us.namDef = SzCopy(sz);
-      GetEdit(deDeLoc, sz); us.locDef = SzCopy(sz);
+      ciDefa = ci;
+      GetEdit(deDeNam, sz); ciDefa.nam = SzClone(sz);
+      GetEdit(deDeLoc, sz); ciDefa.loc = SzClone(sz);
       wi.fCast = fTrue;
     }
 #ifdef ATLAS
@@ -1165,13 +1168,14 @@ flag API DlgInfoAll(HWND hdlg, uint message, WORD wParam, LONG lParam)
 {
   DLGPROC dlgproc;
   char sz[cchSzMax];
-  int i, n;
-  flag fT;
+  int i, n, nSav;
+  flag fSav;
   CI *pci;
 
   switch (message) {
   case WM_INITDIALOG:
-    fT = us.fAnsiChar; us.fAnsiChar = fFalse;
+    nSav = us.fAnsiChar; us.fAnsiChar = 2;
+    fSav = us.fGraphics; us.fGraphics = fTrue;
     for (i = 1; i <= cRing; i++) {
       pci = rgpci[i];
       n = DayOfWeek(pci->mon, pci->day, pci->yea);
@@ -1183,7 +1187,7 @@ flag API DlgInfoAll(HWND hdlg, uint message, WORD wParam, LONG lParam)
         "; " : "", pci->loc);
       SetDlgItemText(hdlg, ds02 + (i-1)*2, sz);
     }
-    us.fAnsiChar = fT;
+    us.fAnsiChar = nSav; us.fGraphics = fSav;
     if (wParam == (WORD)-1 && lParam == -1)
       return fFalse;
     i = us.nRel;
@@ -1192,6 +1196,8 @@ flag API DlgInfoAll(HWND hdlg, uint message, WORD wParam, LONG lParam)
     else if (i < rcHexaWheel)
       i = rcDual;
     SetRadio(dr01-i, dr01, dr06);
+    for (i = 2; i <= 5; i++)
+      SetCheck(dx02 - 2 + i, rgfProg[i]);
     return fTrue;
 
   case WM_COMMAND:
@@ -1207,6 +1213,8 @@ flag API DlgInfoAll(HWND hdlg, uint message, WORD wParam, LONG lParam)
     if (wParam == IDOK) {
       i = GetRadio(hdlg, dr01, dr06) + 1;
       SetRel(-(i-1));
+      for (i = 2; i <= 5; i++)
+        rgfProg[i] = GetCheck(dx02 - 2 + i);
     }
     if (wParam == IDOK || wParam == IDCANCEL) {
       EndDialog(hdlg, fTrue);
@@ -1589,7 +1597,7 @@ flag API DlgCustom(HWND hdlg, uint message, WORD wParam, LONG lParam)
           if (k == 0 || k >= 3)
             pch++;
           l = (k == 2 ? NParseSz(pch, pmObject) : NFromSz(pch));
-          EnsureN(l, l > 0 || k >= 4, "definition");
+          EnsureN(l, l >= (int)(k != 2) || k >= 4, "definition");
 #endif
           if (j) {
 #ifdef SWISS
@@ -1616,7 +1624,8 @@ flag API DlgCustom(HWND hdlg, uint message, WORD wParam, LONG lParam)
 #endif
             GetEdit(den01 - custLo + i, sz);
             if (!FEqSz(sz, szObjDisp[i]))
-              szObjDisp[i] = SzCopy(sz);
+              FCloneSzCore(sz, (char **)&szObjDisp[i],
+                szObjDisp[i] == szObjName[i]);
           }
         }
       }
@@ -1684,15 +1693,13 @@ flag API DlgCustomS(HWND hdlg, uint message, WORD wParam, LONG lParam)
           if (j) {
             GetEdit(den01 - starLo + i, sz);
             if (!FEqSz(sz, szObjDisp[i]))
-              szObjDisp[i] = SzCopy(sz);
+              FCloneSzCore(sz, (char **)&szObjDisp[i],
+                szObjDisp[i] == szObjName[i]);
 #ifdef SWISS
             GetEdit(ded01 - starLo + i, sz);
             k = i - starLo + 1;
-            if (FEqSz(sz, *szStarNameSwiss[k] ? szStarNameSwiss[k] :
-              szObjDisp[i]))
-              szStarCustom[k] = NULL;
-            else
-              szStarCustom[k] = SzCopy(sz);
+            FCloneSz(FEqSz(sz, *szStarNameSwiss[k] ? szStarNameSwiss[k] :
+              szObjName[i]) ? NULL : sz, &szStarCustom[k]);
 #endif
           }
         }
@@ -1714,20 +1721,20 @@ flag API DlgCustomS(HWND hdlg, uint message, WORD wParam, LONG lParam)
 
 flag API DlgRestrict(HWND hdlg, uint message, WORD wParam, LONG lParam)
 {
-  byte *lpb, *lpb2;
+  byte *pb;
   int i;
 
   switch (message) {
   case WM_INITDIALOG:
     if (wi.wCmd == cmdRes)
-      lpb = ignore;
+      pb = ignore;
     else {
       SetWindowText(hdlg, "Transit Object Restrictions");
       SetDlgItemText(hdlg, dbRT, "Copy &From Standard Restriction Set");
-      lpb = ignore2;
+      pb = ignore2;
     }
     for (i = 0; i <= dwarfHi; i++)
-      SetCheck(dx01 + i, lpb[i]);
+      SetCheck(dx01 + i, pb[i]);
     return fTrue;
 
   case WM_COMMAND:
@@ -1757,16 +1764,21 @@ flag API DlgRestrict(HWND hdlg, uint message, WORD wParam, LONG lParam)
         SetCheck(dx01 + i, !GetCheck(dx01 + i));
       break;
     case dbRT:
-      lpb2 = wi.wCmd == cmdRes ? ignore2 : ignore;
+      pb = (wi.wCmd == cmdRes ? ignore2 : ignore);
       for (i = 0; i <= dwarfHi; i++)
-        SetCheck(dx01 + i, lpb2[i]);
+        SetCheck(dx01 + i, pb[i]);
+      break;
+    case dbRe_YRi:
+      pb = (wi.wCmd == cmdRes ? ignoreMem : ignore2Mem);
+      for (i = 0; i <= dwarfHi; i++)
+        SetCheck(dx01 + i, pb[i]);
       break;
     }
 
     if (wParam == IDOK) {
-      lpb = wi.wCmd == cmdRes ? ignore : ignore2;
+      pb = (wi.wCmd == cmdRes ? ignore : ignore2);
       for (i = 0; i <= dwarfHi; i++)
-        lpb[i] = GetCheck(dx01 + i);
+        pb[i] = GetCheck(dx01 + i);
       if (!us.fCusp) {
         for (i = cuspLo; i <= cuspHi; i++)
           if (!ignore[i] || !ignore2[i]) {
@@ -2240,7 +2252,7 @@ flag API DlgTransit(HWND hdlg, uint message, WORD wParam, LONG lParam)
 {
   char sz[cchSzMax];
   int mon, day, yea, nty, nd, n1, n2;
-  real tim;
+  real tim, dst, zon;
 
   switch (message) {
   case WM_INITDIALOG:
@@ -2254,6 +2266,9 @@ flag API DlgTransit(HWND hdlg, uint message, WORD wParam, LONG lParam)
     SetRadio(dr01 + n1, dr01, dr07);
     SetEditMDYT(hdlg, dcTrMon, dcTrDay, dcTrYea, dcTrTim,
       MonT, DayT, YeaT, TimT);
+    SetEditSZOA(hdlg, dcTrDst, dcTrZon, dcDeLon, dcDeLat,
+      DstT, ZonT, LonT, LatT);
+    SetCombo(dcTrDst, "Autodetect");
     n2 = us.fInDayMonth + us.fInDayYear +
       us.fInDayYear*(NAbs(us.nEphemYears) > 1);
     SetRadio(dr08 + n2, dr08, dr11);
@@ -2275,9 +2290,11 @@ flag API DlgTransit(HWND hdlg, uint message, WORD wParam, LONG lParam)
   case WM_COMMAND:
 #ifdef TIME
     if (wParam == dbTr_tn) {
-      GetTimeNow(&mon, &day, &yea, &tim, us.dstDef, us.zonDef);
+      GetTimeNow(&mon, &day, &yea, &tim, ciDefa.dst, ciDefa.zon);
       SetEditMDYT(hdlg, dcTrMon, dcTrDay, dcTrYea, dcTrTim,
         mon, day, yea, tim);
+      SetEditSZOA(hdlg, dcTrDst, dcTrZon, dcInLon, dcInLat,
+        ciDefa.dst, ciDefa.zon, ciDefa.lon, ciDefa.lat);
     }
 #endif
     if (wParam == IDOK) {
@@ -2285,15 +2302,18 @@ flag API DlgTransit(HWND hdlg, uint message, WORD wParam, LONG lParam)
       GetEdit(dcTrDay, sz); day = NParseSz(sz, pmDay);
       GetEdit(dcTrYea, sz); yea = NParseSz(sz, pmYea);
       GetEdit(dcTrTim, sz); tim = RParseSz(sz, pmTim);
+      GetEdit(dcTrDst, sz); dst = RParseSz(sz, pmDst);
+      GetEdit(dcTrZon, sz); zon = RParseSz(sz, pmZon);
       nty = GetEditN(hdlg, deTr_tY);
       nd = GetEditN(hdlg, deTr_d);
       EnsureN(mon, FValidMon(mon), "month");
       EnsureN(yea, FValidYea(yea), "year");
       EnsureN(day, FValidDay(day, mon, yea), "day");
       EnsureR(tim, FValidTim(tim), "time");
+      EnsureR(dst, FValidDst(dst), "daylight saving");
+      EnsureR(zon, FValidZon(zon), "time zone");
       EnsureN(nd, FValidDivision(nd), "searching divisions");
-      SetCI(ciTran, mon, day, yea, tim,
-        us.dstDef, us.zonDef, us.lonDef, us.latDef);
+      SetCI(ciTran, mon, day, yea, tim, dst, zon, ciDefa.lon, ciDefa.lat);
       us.nEphemYears = nty;
       us.fIgnoreSign   = GetCheck(dxTr_YR0_s);
       us.fIgnoreDir    = GetCheck(dxTr_YR0_d);
@@ -2390,7 +2410,7 @@ flag API DlgProgress(HWND hdlg, uint message, WORD wParam, LONG lParam)
   case WM_COMMAND:
 #ifdef TIME
     if (wParam == dbPr_pn) {
-      GetTimeNow(&mon, &day, &yea, &tim, us.dstDef, us.zonDef);
+      GetTimeNow(&mon, &day, &yea, &tim, ciDefa.dst, ciDefa.zon);
       SetEditMDYT(hdlg, dcPrMon, dcPrDay, dcPrYea, dcPrTim,
         mon, day, yea, tim);
     }
@@ -2416,8 +2436,8 @@ flag API DlgProgress(HWND hdlg, uint message, WORD wParam, LONG lParam)
       us.rProgDay = rd;
       us.rProgCusp = rC;
       SetCI(ciTran, mon, day, yea, tim,
-        us.dstDef, us.zonDef, us.lonDef, us.latDef);
-      is.JDp = MdytszToJulian(MonT, DayT, YeaT, TimT, us.dstDef, us.zonDef);
+        ciDefa.dst, ciDefa.zon, ciDefa.lon, ciDefa.lat);
+      is.JDp = MdytszToJulian(MonT, DayT, YeaT, TimT, ciDefa.dst, ciDefa.zon);
       wi.fCast = fTrue;
     }
     if (wParam == IDOK || wParam == IDCANCEL) {
@@ -2433,9 +2453,6 @@ flag API DlgProgress(HWND hdlg, uint message, WORD wParam, LONG lParam)
 CONST char *rgszSort[asMax] = {"Power", "Orb Magnitude",
   "Orb Value", "1st Object Index", "2nd Object Index",
   "Aspect", "1st Object Position", "2nd Object Position", "Midpoint"};
-CONST char *rgszDecan[ddMax] = {"None", "Decan Ruler", "Decan Sign",
-  "Chaldean Decan", "Egyptian Term", "Ptolemaic Term", "Navamsa",
-  "12th Harmonic", "Dwad", "Nakshatra"};
 
 // Processing function for the chart subsettings dialog, as brought up with
 // the Chart / Chart Settings menu command.
@@ -2444,7 +2461,6 @@ flag API DlgChart(HWND hdlg, uint message, WORD wParam, LONG lParam)
 {
   char sz[cchSzMax];
   int nw, nl, nl2, np, nn, yb, nT, i;
-  flag f;
 
   switch (message) {
   case WM_INITDIALOG:
@@ -2508,11 +2524,7 @@ flag API DlgChart(HWND hdlg, uint message, WORD wParam, LONG lParam)
       EnsureN(np, FValidPart(np), "Arabic part");
       EnsureN(nn, nn >= 0, "nearest city count");
       EnsureN(yb, FValidBioday(yb), "Biorhythm days");
-      f = GetCheck(dxCh_v0);
-      if (us.fVelocity != f) {
-        us.fVelocity = f;
-        WiCheckMenu(cmdGraphicsSidebar, !f);
-      }
+      us.fVelocity = GetCheck(dxCh_v0);
       us.nWheelRows = nw;
       us.fWheelReverse = GetCheck(dxCh_w0);
       us.fGridConfig = GetCheck(dxCh_g0);
@@ -2545,6 +2557,8 @@ flag API DlgChart(HWND hdlg, uint message, WORD wParam, LONG lParam)
       for (i = 0; i < ddMax; i++)
         if (FMatchSz(sz, rgszDecan[i]))
           break;
+      if (i >= ddMax)
+        i = 0;
       us.fListDecan = (i > ddNone);
       if (i > ddNone)
         us.nDecanType = i;
@@ -2565,6 +2579,8 @@ CONST char *rgszFontDisp[cFont] = {szAppNameCore, "Wingdings", "Astro",
   "Hank's Nakshatra"};
 CONST char *rgszCityColor[6] = {"None", "Region", "Region+State",
   "Generic Zone", "Current Zone", "Rainbow"};
+CONST char *rgszWheelCorner[6] = {"None", "Spider Web", "Moire Pattern",
+  "Rays 1", "Rays 1.2", "Rays 12345"};
 
 // Processing function for the graphic settings dialog, as brought up with the
 // Graphics / Graphics Settings menu command.
@@ -2612,9 +2628,11 @@ flag API DlgGraphics(HWND hdlg, uint message, WORD wParam, LONG lParam)
     SetRadio(gs.objLeft > 0 ? dr02 :
       (gs.objLeft < 0 ? dr03 : dr01), dr01, dr03);
     SetEdit(deGr_X1, szObjName[gs.objLeft == 0 ? oSun : NAbs(gs.objLeft)-1]);
-    SetRadio(dr04 + gs.nDecaType, dr04, dr06);
+    for (i = 0; i < 6; i++)
+      SetCombo(dcGr_YXv, rgszWheelCorner[i]);
+    SetEdit(dcGr_YXv, rgszWheelCorner[gs.nDecaType]);
     SetEditN(deGr_YXv, gs.nDecaSize);
-    SetRadio(dr07 + Min(gs.nDecaFill, 2), dr07, dr09);
+    SetRadio(dr04 + gs.nDecaFill, dr04, dr07);
     for (i = dcGr_Xf0; i <= dcGr_Xf5; i++)
       for (j = 0; j < cFont; j++) {
         if (rgszFontAllow[i-dcGr_Xf0][j] < '0')
@@ -2676,7 +2694,15 @@ flag API DlgGraphics(HWND hdlg, uint message, WORD wParam, LONG lParam)
       gs.nScale = ns; gs.nScaleText = nS;
       gs.rBackPct = rI;
       gs.nGridCell = ng;
-      gs.cspace = GetEditN(hdlg, deGr_YXj);
+      i = GetEditN(hdlg, deGr_YXj);
+      if (gs.cspace != i) {
+        gs.cspace = i;
+        if (gi.rgspace != NULL) {
+          DeallocateP(gi.rgspace);
+          gi.rgspace = NULL;
+        }
+        gi.ispace = 0;
+      }
       gs.objTrack = nxz;
       gs.rspace = ryxs;
       gs.rRot = rxw; gs.rTilt = rxg;
@@ -2692,9 +2718,13 @@ flag API DlgGraphics(HWND hdlg, uint message, WORD wParam, LONG lParam)
       wi.fNoUpdate = GetCheck(dxGr_Wn);
       gs.nAllStar = (GetCheck(dxGr_XU2) << 1) | GetCheck(dxGr_XU1);
       gs.objLeft = GetCheck(dr01) ? 0 : (GetCheck(dr02) ? nx1+1 : -nx1-1);
-      gs.nDecaType = GetRadio(hdlg, dr04, dr06);
+      GetEdit(dcGr_YXv, sz);
+      for (i = 0; i < 6; i++)
+        if (NCompareSzI(sz, rgszWheelCorner[i]) == 0)
+          break;
+      gs.nDecaType = i < 6 ? i : 0;
       gs.nDecaSize = nyxv;
-      gs.nDecaFill = GetRadio(hdlg, dr07, dr09);
+      gs.nDecaFill = GetRadio(hdlg, dr04, dr07);
       for (i = dcGr_Xf0; i <= dcGr_Xf5; i++) {
         GetEdit(i, sz);
         // Astro font in slot #2 gets checked first, since it's a substring.
