@@ -1,8 +1,8 @@
 /*
-** Astrolog (Version 7.60) File: wdriver.cpp
+** Astrolog (Version 7.80) File: wdriver.cpp
 **
 ** IMPORTANT NOTICE: Astrolog and all chart display routines and anything
-** not enumerated below used in this program are Copyright (C) 1991-2023 by
+** not enumerated below used in this program are Copyright (C) 1991-2025 by
 ** Walter D. Pullen (Astara@msn.com, http://www.astrolog.org/astrolog.htm).
 ** Permission is granted to freely use, modify, and distribute these
 ** routines provided these credits and notices remain unmodified with any
@@ -10,8 +10,8 @@
 **
 ** The main ephemeris databases and calculation routines are from the
 ** library SWISS EPHEMERIS and are programmed and copyright 1997-2008 by
-** Astrodienst AG. The use of that source code is subject to the license for
-** Swiss Ephemeris Free Edition, available at http://www.astro.com/swisseph.
+** Astrodienst AG. Use of that source code is subject to license for Swiss
+** Ephemeris Free Edition at https://www.astro.com/swisseph/swephinfo_e.htm.
 ** This copyright notice must not be changed or removed by any user of this
 ** program.
 **
@@ -48,7 +48,7 @@
 ** Initial programming 8/28-30/1991.
 ** X Window graphics initially programmed 10/23-29/1991.
 ** PostScript graphics initially programmed 11/29-30/1992.
-** Last code change made 4/8/2023.
+** Last code change made 6/19/2025.
 */
 
 #include "astrolog.h"
@@ -277,7 +277,8 @@ void DoPopup(int imenu, HWND hwnd, LPARAM lParam)
     CheckPopup(cmdGraphicsAspect,  gs.fLabelAsp);
     CheckPopup(cmdGraphicsModify,  gs.fAlt);
     CheckPopup(cmdGraphicsHouse,   !gs.fHouseExtra);
-    CheckPopup(cmdGraphicsSidebar, !us.fVelocity);
+    CheckPopup(cmdGraphicsCity,    gs.fLabelCity);
+    CheckPopup(cmdGraphicsSidebar, gs.fDoSidebar);
     break;
   case menuV2:
     CheckPopup(cmdChartIndianS,    gi.nMode == gWheel && !gs.fHouseExtra);
@@ -287,7 +288,7 @@ void DoPopup(int imenu, HWND hwnd, LPARAM lParam)
     CheckPopup(cmdGraphicsEquator, !gs.fEquator);
     CheckPopup(cmdGraphicsAspect,  gs.fLabelAsp);
     CheckPopup(cmdGraphicsLabel,   !gs.fLabel);
-    CheckPopup(cmdGraphicsSidebar, !us.fVelocity);
+    CheckPopup(cmdGraphicsSidebar, gs.fDoSidebar);
     break;
   case menuG:
     CheckPopup(cmdParallel,       us.fParallel);
@@ -302,7 +303,7 @@ void DoPopup(int imenu, HWND hwnd, LPARAM lParam)
     CheckPopup(cmdGraphicsModify,  gs.fAlt);
     CheckPopup(cmdGraphicsCity,    !gs.fLabelCity);
     CheckPopup(cmdGraphicsHouse,   !gs.fHouseExtra);
-    CheckPopup(cmdGraphicsSidebar, !us.fVelocity);
+    CheckPopup(cmdGraphicsSidebar, gs.fDoSidebar);
     break;
   case menuZ:
     CheckPopup(cmdChartModify,     us.fPrimeVert);
@@ -310,6 +311,7 @@ void DoPopup(int imenu, HWND hwnd, LPARAM lParam)
     CheckPopup(cmdConstellation,   gs.fConstel);
     CheckPopup(cmdHouseSet3D,      !us.fHouse3D);
     CheckPopup(cmdGraphicsAllStar, gs.fAllStar);
+    CheckPopup(cmdGraphicsExo,     gs.fAllExo);
     CheckPopup(cmdHouseSetIndian,  us.fIndian);
     CheckPopup(cmdGraphicsHouse,   gs.fHouseExtra);
     CheckPopup(cmdGraphicsEquator, gs.fEquator);
@@ -329,11 +331,11 @@ void DoPopup(int imenu, HWND hwnd, LPARAM lParam)
     break;
   case menuH:
     CheckPopup(cmdGraphicsEquator, !gs.fEquator);
-    CheckPopup(cmdGraphicsModify,  gs.fAlt);
     CheckPopup(cmdGraphicsAspect,  gs.fLabelAsp);
+    CheckPopup(cmdGraphicsModify,  gs.fAlt);
     CheckPopup(cmdHouseSetIndian,  us.fIndian);
     CheckPopup(cmdGraphicsHouse,   !gs.fHouseExtra);
-    CheckPopup(cmdGraphicsSidebar, !us.fVelocity);
+    CheckPopup(cmdGraphicsSidebar, gs.fDoSidebar);
     break;
   case menuK:
     CheckPopup(cmdChartModify,    us.fCalendarYear);
@@ -357,6 +359,7 @@ void DoPopup(int imenu, HWND hwnd, LPARAM lParam)
     CheckPopup(cmdGraphicsEquator, !gs.fEquator);
     break;
   case menuL:
+    CheckPopup(cmdHouseSet3D,     us.fHouse3D);
     CheckPopup(cmdGraphicsBmp,    gi.fBmp);
     CheckPopup(cmdChartModify,    us.fLatitudeCross);
     CheckPopup(cmdGraphicsModify, gs.fAlt);
@@ -378,6 +381,7 @@ void DoPopup(int imenu, HWND hwnd, LPARAM lParam)
     break;
   case menuN:
     CheckPopup(cmdHouseSetIndian,  !us.fIndian);
+    CheckPopup(cmdChartModify,     gs.fSouth);
     CheckPopup(cmdGraphicsEquator, gs.fEquator);
     CheckPopup(cmdGraphicsHouse,   gs.fHouseExtra);
     CheckPopup(cmdGraphicsCity,    !gs.fLabelCity);
@@ -403,7 +407,7 @@ void DoPopup(int imenu, HWND hwnd, LPARAM lParam)
   case menuXX:
     CheckPopup(cmdChartModify,     gs.fSouth);
     CheckPopup(cmdGraphicsModify,  gs.fAlt);
-    CheckPopup(cmdGraphicsSidebar, !us.fVelocity);
+    CheckPopup(cmdGraphicsSidebar, gs.fDoSidebar);
     CheckPopup(cmdHouseSetIndian,  !us.fIndian);
     CheckPopup(cmdGraphicsHouse,   !gs.fHouseExtra);
     CheckPopup(cmdHouseSet3D,      !us.fHouse3D);
@@ -411,15 +415,18 @@ void DoPopup(int imenu, HWND hwnd, LPARAM lParam)
     CheckPopup(cmdGraphicsEquator, gs.fEquator);
     CheckPopup(cmdConstellation,   gs.fConstel);
     CheckPopup(cmdGraphicsAllStar, gs.fAllStar);
+    CheckPopup(cmdGraphicsExo,     gs.fAllExo);
     CheckPopup(cmdGraphicsAxis,    gs.fEcliptic);
     break;
   case menuXG:
     CheckPopup(cmdGraphicsModify,  !gs.fAlt);
     CheckPopup(cmdConstellation,   gs.fConstel);
+    CheckPopup(cmdHouseSet3D,      us.fHouse3D);
     CheckPopup(cmdGraphicsBmp,     gi.fBmp);
     CheckPopup(cmdGraphicsHouse,   gs.fHouseExtra);
     CheckPopup(cmdGraphicsEquator, gs.fEquator);
     CheckPopup(cmdGraphicsAllStar, gs.fAllStar);
+    CheckPopup(cmdGraphicsExo,     gs.fAllExo);
     CheckPopup(cmdGraphicsAxis,    gs.fEcliptic);
     CheckPopup(cmdGraphicsCity,    gs.fLabelCity);
     CheckPopup(cmdGraphicsAspect,  gs.fLabelAsp);
@@ -435,6 +442,7 @@ void DoPopup(int imenu, HWND hwnd, LPARAM lParam)
     CheckPopup(cmdGraphicsLabel,   gs.fLabel);
     CheckPopup(cmdGraphicsModify,  !gs.fAlt);
     CheckPopup(cmdGraphicsAllStar, gs.fAllStar);
+    CheckPopup(cmdGraphicsExo,     gs.fAllExo);
     CheckPopup(cmdGraphicsCity,    gs.fLabelCity);
     CheckPopup(cmdGraphicsAxis,    gs.fEcliptic);
     break;
@@ -464,14 +472,22 @@ void DoPopup(int imenu, HWND hwnd, LPARAM lParam)
     CheckPopup(cmdSecond,     us.fSeconds);
     CheckPopup(cmdHouseSet3D, us.fHouse3D);
     break;
-  case menu_S: case menu_H: case menu_J: case menu_L: case menu_P:
-  case menu_I: case menu_N:
+  case menu_L:
+    CheckPopup(cmdHouseSet3D, us.fHouse3D);
+    CheckPopup(cmdSecond,     us.fSeconds);
+    break;
+  case menu_S: case menu_H: case menu_J: case menu_P: case menu_I:
+  case menu_N:
     CheckPopup(cmdSecond, us.fSeconds);
     break;
   case menu_E: case menu_D: case menu_8:
     CheckPopup(cmdSecond,   us.fSeconds);
     CheckPopup(cmdParallel, us.fParallel);
     break;
+  case menu_Ux:
+    CheckPopup(cmdParallel,   !us.fParallel);
+    CheckPopup(cmdHouseSet3D, !us.fHouse3D);
+    break;
   case menu_B:
     CheckPopup(cmdParallel, us.fParallel);
     break;
@@ -545,13 +561,17 @@ int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
   DeleteMenu(wi.hmenu, cmdChartArabic, MF_BYCOMMAND);
 #endif
 #ifndef CONSTEL
-  DeleteMenu(wi.hmenu, cmdConstellation, MF_BYCOMMAND);
+  DeleteMenu(wi.hmenu, cmdConstellation,     MF_BYCOMMAND);
+  DeleteMenu(wi.hmenu, cmdGraphicsStarLine,  MF_BYCOMMAND);
+  DeleteMenu(wi.hmenu, cmdHelpConstellation, MF_BYCOMMAND);
 #endif
 #ifndef BIORHYTHM
   DeleteMenu(wi.hmenu, cmdRelBiorhythm, MF_BYCOMMAND);
 #endif
 #ifndef ATLAS
   DeleteMenu(wi.hmenu, cmdGraphicsCity, MF_BYCOMMAND);
+  DeleteMenu(wi.hmenu, cmdDocAtlas,     MF_BYCOMMAND);
+  DeleteMenu(wi.hmenu, cmdDocTimezone,  MF_BYCOMMAND);
 #endif
 #ifndef WSETUP
   DeleteMenu(wi.hmenu, cmdOpenDir,        MF_BYCOMMAND);
@@ -593,7 +613,6 @@ int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
   ofn.lpstrFileTitle = szFileTitle;
   prd.hwndOwner = wi.hwndMain;
   wi.haccel = LoadAccelerators(wi.hinst, MAKEINTRESOURCE(accelerator));
-  wi.kiPen = gi.kiLite;
 
   // Check for being used as a Windows screen saver.
 
@@ -617,8 +636,9 @@ int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
   // Process the astrolog.as file and the Windows command line.
 
   FProcessSwitchFile(DEFAULT_INFOFILE, NULL);
-  FProcessCommandLine(lpszCmdLine);
   ciTran = ciHexa = ciFive = ciFour = ciThre = ciTwin = ciMain = ciCore;
+  FProcessCommandLine(lpszCmdLine);
+  InitRestrictions(fTrue);
 
   // Actually bring up and display the window for the first time.
 
@@ -832,7 +852,7 @@ LRESULT API WndProc(HWND hwnd, UINT wMsg, WPARAM wParam, LPARAM lParam)
       }
       hdc = GetDC(hwnd);
       hpen = (HPEN)CreatePen(PS_SOLID, !gs.fThick ? 0 : 2,
-        (COLORREF)rgbbmp[wi.kiPen]);
+        (COLORREF)rgbbmp[gi.kiPen]);
       hpenOld = (HPEN)SelectObject(hdc, hpen);
 
       // Ctrl+click means draw a rectangle. Ctrl+Shift+click does ellipse.
@@ -855,7 +875,7 @@ LRESULT API WndProc(HWND hwnd, UINT wMsg, WPARAM wParam, LPARAM lParam)
 
       // A simple click means set a pixel and remember that location.
       } else {
-        SetPixel(hdc, x, y, (COLORREF)rgbbmp[wi.kiPen]);
+        SetPixel(hdc, x, y, (COLORREF)rgbbmp[gi.kiPen]);
         wi.xMouse = x; wi.yMouse = y;
       }
       SelectObject(hdc, hpenOld);
@@ -892,24 +912,25 @@ LRESULT API WndProc(HWND hwnd, UINT wMsg, WPARAM wParam, LPARAM lParam)
         case gTraTraGra: case gTraNatGra: DoPopup(menuB, hwnd, lParam); break;
         }
       } else {
-        if (us.nRel == rcBiorhythm) DoPopup(menu_Y, hwnd, lParam);
-        else if (us.fListing)       DoPopup(menu_V, hwnd, lParam);
-        else if (us.fWheel)         DoPopup(menu_W, hwnd, lParam);
-        else if (us.fGrid)          DoPopup(menu_G, hwnd, lParam);
-        else if (us.fAspList)       DoPopup(menu_A, hwnd, lParam);
-        else if (us.fMidpoint)      DoPopup(menu_M, hwnd, lParam);
-        else if (us.fHorizon)       DoPopup(menu_Z, hwnd, lParam);
-        else if (us.fOrbit)         DoPopup(menu_S, hwnd, lParam);
-        else if (us.fSector)        DoPopup(menu_H, hwnd, lParam);
-        else if (us.fCalendar)      DoPopup(menu_K, hwnd, lParam);
-        else if (us.fInfluence)     DoPopup(menu_J, hwnd, lParam);
-        else if (us.fEsoteric)      DoPopup(menu_7, hwnd, lParam);
-        else if (us.fAstroGraph)    DoPopup(menu_L, hwnd, lParam);
-        else if (us.fEphemeris)     DoPopup(menu_E, hwnd, lParam);
-        else if (us.fArabic)        DoPopup(menu_P, hwnd, lParam);
-        else if (us.fHorizonSearch) DoPopup(menu_I, hwnd, lParam);
-        else if (us.fAtlasNear)     DoPopup(menu_N, hwnd, lParam);
-        else if (us.fMoonChart)     DoPopup(menu_8, hwnd, lParam);
+        if (us.nRel == rcBiorhythm) DoPopup(menu_Y,  hwnd, lParam);
+        else if (us.fListing)       DoPopup(menu_V,  hwnd, lParam);
+        else if (us.fWheel)         DoPopup(menu_W,  hwnd, lParam);
+        else if (us.fGrid)          DoPopup(menu_G,  hwnd, lParam);
+        else if (us.fAspList)       DoPopup(menu_A,  hwnd, lParam);
+        else if (us.fMidpoint)      DoPopup(menu_M,  hwnd, lParam);
+        else if (us.fHorizon)       DoPopup(menu_Z,  hwnd, lParam);
+        else if (us.fOrbit)         DoPopup(menu_S,  hwnd, lParam);
+        else if (us.fSector)        DoPopup(menu_H,  hwnd, lParam);
+        else if (us.fCalendar)      DoPopup(menu_K,  hwnd, lParam);
+        else if (us.fInfluence)     DoPopup(menu_J,  hwnd, lParam);
+        else if (us.fEsoteric)      DoPopup(menu_7,  hwnd, lParam);
+        else if (us.fAstroGraph)    DoPopup(menu_L,  hwnd, lParam);
+        else if (us.fEphemeris)     DoPopup(menu_E,  hwnd, lParam);
+        else if (us.fArabic)        DoPopup(menu_P,  hwnd, lParam);
+        else if (us.fHorizonSearch) DoPopup(menu_I,  hwnd, lParam);
+        else if (us.fAtlasNear)     DoPopup(menu_N,  hwnd, lParam);
+        else if (us.fMoonChart)     DoPopup(menu_8,  hwnd, lParam);
+        else if (us.fExoTransit)    DoPopup(menu_Ux, hwnd, lParam);
         else if (us.fInDay    || us.fTransit)    DoPopup(menu_D, hwnd, lParam);
         else if (us.fInDayInf || us.fTransitInf) DoPopup(menu_T, hwnd, lParam);
         else if (us.fInDayGra || us.fTransitGra) DoPopup(menu_B, hwnd, lParam);
@@ -1099,6 +1120,7 @@ void ProcessState()
       case gMidpoint:   us.fMidpoint   = fTrue; break;
       case gArabic:     us.fArabic     = fTrue; break;
       case gMoons:      us.fMoonChart  = fTrue; break;
+      case gExo:        us.fExoTransit = fTrue; break;
       case gTraTraTim:  us.fInDay      = fTrue; break;
       case gTraTraInf:  us.fInDayInf   = fTrue; break;
       case gTraTraGra:  us.fInDayGra   = fTrue; break;
@@ -1153,10 +1175,14 @@ int NWmCommand(WORD wCmd)
   DLGPROC dlgproc;
   RECT rc;
   POINT pt;
+  CI ci;
   int i;
   long l;
   real r;
   flag fGraphicsSav, fRestoreSize = fFalse, fT;
+#ifdef CONSTEL
+  CONST char **ppch;
+#endif
 
 #ifdef EXPRESS
   // May want to adjust current command if AstroExpression says to do so.
@@ -1250,7 +1276,7 @@ int NWmCommand(WORD wCmd)
     break;
 
   case cmdCopyText:
-    is.szFileScreen = szFileTemp;
+    FCloneSz(szFileTemp, &is.szFileScreen);
     us.fGraphics = fFalse;
     wi.fRedraw = fTrue;
     break;
@@ -1267,7 +1293,7 @@ int NWmCommand(WORD wCmd)
 #endif
     if (us.fNoWrite)
       break;
-    gi.szFileOut = szFileTemp;
+    FCloneSz(szFileTemp, &gi.szFileOut);
     switch (wCmd) {
     case cmdCopyBitmap:  gs.ft = ftBmp; gs.chBmpMode = 'B'; break;
     case cmdCopyPicture: gs.ft = ftWmf;  break;
@@ -1405,7 +1431,6 @@ int NWmCommand(WORD wCmd)
 
   case cmdSecond:
     inv(us.fSeconds);
-    is.fSeconds = us.fSeconds;
     WiCheckMenu(cmdSecond, us.fSeconds);
     wi.fRedraw = fTrue;
     break;
@@ -1474,6 +1499,11 @@ int NWmCommand(WORD wCmd)
     }
     break;
 
+  case cmdSwap12:
+    SwapTemp(ciCore, ciTwin, ci);
+    wi.fCast = fTrue;
+    break;
+
   case cmdRelNo:
   case cmdRelComparison:
     SetRel(us.nRel ? rcNone : rcDual);
@@ -1627,6 +1657,11 @@ int NWmCommand(WORD wCmd)
     wi.nMode = gMoons;
     break;
 
+  case cmdChartExo:
+    wi.nMode = gExo;
+    us.fGraphics = fFalse;
+    break;
+
   case cmdMoons:
     WiDoDialog(DlgMoons, dlgMoons);
     break;
@@ -1887,6 +1922,20 @@ int NWmCommand(WORD wCmd)
     }
     break;
 
+  case cmdTextDecrease:
+    if (gs.nScaleText > 100) {
+      gs.nScaleText -= 50;
+      wi.fRedraw = fTrue;
+    }
+    break;
+
+  case cmdTextIncrease:
+    if (gs.nScaleText < 400) {
+      gs.nScaleText += 50;
+      wi.fRedraw = fTrue;
+    }
+    break;
+
   case cmdGraphicsBorder:
     inv(gs.fBorder);
     WiCheckMenu(cmdGraphicsBorder, gs.fBorder);
@@ -1900,12 +1949,12 @@ int NWmCommand(WORD wCmd)
     break;
 
   case cmdGraphicsSidebar:
-    inv(us.fVelocity);
-    if (!us.fVelocity) {
+    inv(gs.fDoSidebar);
+    if (gs.fDoSidebar) {
       gs.fText = fTrue;
       WiCheckMenu(cmdGraphicsText, fTrue);
     }
-    WiCheckMenu(cmdGraphicsSidebar, !us.fVelocity);
+    WiCheckMenu(cmdGraphicsSidebar, gs.fDoSidebar);
     if (wi.nMode != gWheel && gi.nMode != gHouse && gi.nMode != gMidpoint &&
       gi.nMode != gSector && gi.nMode != gSphere)
       wi.nMode = gWheel;
@@ -1915,6 +1964,14 @@ int NWmCommand(WORD wCmd)
   case cmdGraphicsThick:
     inv(gs.fThick);
     WiCheckMenu(cmdGraphicsThick, gs.fThick);
+    if (gs.nFontTxt <= 0)
+      us.fGraphics = fTrue;
+    wi.fRedraw = fTrue;
+    break;
+
+  case cmdGraphicsAntialias:
+    inv(gs.fAntialias);
+    WiCheckMenu(cmdGraphicsAntialias, gs.fAntialias);
     us.fGraphics = wi.fRedraw = fTrue;
     break;
 
@@ -1947,6 +2004,30 @@ int NWmCommand(WORD wCmd)
     us.fGraphics = wi.fRedraw = fTrue;
     break;
 
+  case cmdGraphicsExo:
+    inv(gs.fAllExo);
+    WiCheckMenu(cmdGraphicsExo, gs.fAllExo);
+    us.fGraphics = wi.fRedraw = fTrue;
+    break;
+
+#ifdef CONSTEL
+  case cmdGraphicsStarLine:
+    inv(wi.fStarLine);
+    if (wi.fStarLine) {
+      for (ppch = szDrawConstelLine; *ppch != NULL; ppch += 2)
+        if (!FProcessYXU(ppch[0], ppch[1], ppch != szDrawConstelLine))
+          break;
+      if (!gs.fAllStar) {
+        gs.fAllStar = fTrue;
+        WiCheckMenu(cmdGraphicsAllStar, fTrue);
+      }
+    } else
+      FProcessYXU("", "", fFalse);
+    WiCheckMenu(cmdGraphicsStarLine, wi.fStarLine);
+    us.fGraphics = wi.fRedraw = fTrue;
+    break;
+#endif
+
   case cmdGraphicsHouse:
     inv(gs.fHouseExtra);
     WiCheckMenu(cmdGraphicsHouse, gs.fHouseExtra);
@@ -2127,7 +2208,7 @@ int NWmCommand(WORD wCmd)
   case cmdPen13:
   case cmdPen14:
   case cmdPen15:
-    wi.kiPen = (int)(wCmd - cmdPen00);
+    gi.kiPen = (int)(wCmd - cmdPen00);
     WiRadioMenu(cmdPen00, cmdPen15, wCmd);
     break;
 
@@ -2156,8 +2237,11 @@ int NWmCommand(WORD wCmd)
   case cmdAnimateNo7:
   case cmdAnimateNo8:
   case cmdAnimateNo9:
+  case cmdAnimateNo_11:
+  case cmdAnimateNo_12:
+  case cmdAnimateNo_13:
     gs.nAnim = (gs.nAnim < 0 ? -1 : 1)*((int)(wCmd - cmdAnimateNo1) + 1);
-    WiRadioMenu(cmdAnimateNo1, cmdAnimateNow, wCmd);
+    WiRadioMenu(cmdAnimateNo1, cmdAnimateNo_13, wCmd);
     break;
 
   case cmdAnimateF1:
@@ -2196,12 +2280,12 @@ int NWmCommand(WORD wCmd)
     break;
 
   case cmdStepForward:
-    Animate(NAbs(gs.nAnim) >= 10 ? 4 : gs.nAnim, NAbs(gi.nDir));
+    Animate(NAbs(gs.nAnim) == iAnimNow ? iAnimDay : gs.nAnim, NAbs(gi.nDir));
     wi.fCast = fTrue;
     break;
 
   case cmdStepBackward:
-    Animate(NAbs(gs.nAnim) >= 10 ? 4 : gs.nAnim, -NAbs(gi.nDir));
+    Animate(NAbs(gs.nAnim) == iAnimNow ? iAnimDay : gs.nAnim, -NAbs(gi.nDir));
     wi.fCast = fTrue;
     break;
 
@@ -2256,6 +2340,10 @@ int NWmCommand(WORD wCmd)
     BootExternal("Notepad", "seorbel.txt");
     break;
 
+  case cmdDocExo:
+    BootExternal("Notepad", szFileExoCore);
+    break;
+
   case cmdHelpSign:
     wi.nMode = gSign;
     us.fGraphics = fFalse;
@@ -2344,18 +2432,18 @@ int NWmCommand(WORD wCmd)
 
   default:
     if (FBetween(wCmd, cmdMacro01, cmdMacro48)) {
-      i = (int)(wCmd - cmdMacro01);
-      if (szMacro[i]) {
-        FProcessCommandLine(szMacro[i]);
+      i = (int)wCmd - cmdMacro01 + 1;
+      if (is.rgszMacro != NULL && is.rgszMacro[i]) {
+        FProcessCommandLine(is.rgszMacro[i]);
         wi.fCast = wi.fMenuAll = fTrue;
-      } else if (i == 0/*F1*/ || i == 39/*Alt+F4*/) {
+      } else if (i == 1/*F1*/ || i == 40/*Alt+F4*/) {
         // F1 shows help if macro not already defined for it.
-        if (i == 0)
+        if (i == 1)
           BootExternal(NULL, "astrolog.htm");
-        else if (i == 39)
+        else if (i == 40)
           return -1;
       } else {
-        sprintf(sz, "Macro number %d is not defined.", i+1);
+        sprintf(sz, "Macro number %d is not defined.", i);
         PrintWarning(sz);
       }
       break;
@@ -2419,28 +2507,32 @@ void API RedoMenu()
   CheckMenu(cmdResCOB, us.fCOB);
   CheckMenu(cmdResStar, us.fStar);
   CheckMenu(cmdProgress, us.fProgress);
-  CheckMenu(cmdChartIndian, gs.fIndianWheel);
+  CheckMenu(cmdGraphicsReverse, gs.fInverse);
+  CheckMenu(cmdGraphicsMonochrome, !gs.fColor);
+  RadioMenu(cmdScale1, cmdScale4, cmdScale1 + gs.nScale/100 - 1);
+  CheckMenu(cmdGraphicsBorder, gs.fBorder);
+  CheckMenu(cmdGraphicsText, gs.fText);
+  CheckMenu(cmdGraphicsSidebar, gs.fDoSidebar);
+  CheckMenu(cmdGraphicsThick, gs.fThick);
+  CheckMenu(cmdGraphicsAntialias, gs.fAntialias);
+  CheckMenu(cmdGraphicsLabel, gs.fLabel);
+  CheckMenu(cmdGraphicsAspect, gs.fLabelAsp);
 #ifdef CONSTEL
   CheckMenu(cmdConstellation, gs.fConstel);
 #endif
   CheckMenu(cmdGraphicsAllStar, gs.fAllStar);
+  CheckMenu(cmdGraphicsExo, gs.fAllExo);
+  CheckMenu(cmdGraphicsStarLine, wi.fStarLine);
   CheckMenu(cmdGraphicsHouse, gs.fHouseExtra);
   CheckMenu(cmdGraphicsEquator, gs.fEquator);
   CheckMenu(cmdGraphicsCity, gs.fLabelCity);
   CheckMenu(cmdGraphicsBmp, gi.fBmp);
   CheckMenu(cmdGraphicsAxis, gs.fEcliptic);
-  CheckMenu(cmdGraphicsReverse, gs.fInverse);
-  CheckMenu(cmdGraphicsMonochrome, !gs.fColor);
-  CheckMenu(cmdGraphicsBorder, gs.fBorder);
-  CheckMenu(cmdGraphicsText, gs.fText);
-  CheckMenu(cmdGraphicsSidebar, !us.fVelocity);
-  CheckMenu(cmdGraphicsLabel, gs.fLabel);
-  CheckMenu(cmdGraphicsThick, gs.fThick);
-  RadioMenu(cmdScale1, cmdScale4, cmdScale1 + gs.nScale/100 - 1);
+  CheckMenu(cmdChartIndian, gs.fIndianWheel);
   CheckMenu(cmdGraphicsModify, gs.fAlt);
-  RadioMenu(cmdPen00, cmdPen15, cmdPen00 + wi.kiPen);
+  RadioMenu(cmdPen00, cmdPen15, cmdPen00 + gi.kiPen);
   CheckMenu(cmdAnimateNo, gs.nAnim > 0);
-  RadioMenu(cmdAnimateNo1, cmdAnimateNow, cmdAnimateNo + NAbs(gs.nAnim));
+  RadioMenu(cmdAnimateNo1, cmdAnimateNo_13, cmdAnimateNo + NAbs(gs.nAnim));
   RadioMenu(cmdAnimateF1, cmdAnimateF9, cmdAnimateF1 + NAbs(gi.nDir) - 1);
   CheckMenu(cmdAnimateReverse, gi.nDir < 0);
   CheckMenu(cmdAnimatePause, gi.fPause);
@@ -2582,7 +2674,7 @@ flag API FRedraw(void)
   PAINTSTRUCT ps;
   HDC hdcWin;
   HCURSOR hcurPrev = NULL;
-  HBITMAP hbmp, hbmpOld;
+  HBITMAP hbmpOld;
   HFONT hfontOld = NULL;
   char szFile[cchSzDef];
   int nScrollRow, i;
@@ -2613,8 +2705,8 @@ flag API FRedraw(void)
     hdcWin = BeginPaint(wi.hwnd, &ps);
     if (wi.fBuffer) {
       wi.hdc = CreateCompatibleDC(hdcWin);
-      hbmp = CreateCompatibleBitmap(hdcWin, wi.xClient, wi.yClient);
-      hbmpOld = (HBITMAP)SelectObject(wi.hdc, hbmp);
+      wi.hbmp = CreateCompatibleBitmap(hdcWin, wi.xClient, wi.yClient);
+      hbmpOld = (HBITMAP)SelectObject(wi.hdc, wi.hbmp);
       if (gs.fJetTrail)
         BitBlt(wi.hdc, 0, 0, wi.xClient, wi.yClient, hdcWin, 0, 0, SRCCOPY);
     } else
@@ -2639,13 +2731,22 @@ flag API FRedraw(void)
       TextClearScreen();
     FBmpDrawBack(NULL);
     i = gs.nScale/100;
-    wi.xChar = i < 2 ? 6 : (i < 3 ? 8 : (i < 4 ? 10 : 12));
-    wi.yChar = i < 2 ? 8 : (i < 3 ? 12 : (i < 4 ? 18 : 16));
-    wi.hfont = CreateFont(wi.yChar /*nHeight*/, wi.xChar /*nWidth*/,
-      0 /*nEscapement*/, 0 /*nOrientation*/, FW_DONTCARE,
-      0 /*fbItalic*/, 0 /*fbUnderline*/, 0 /*fbStrikeOut*/,
-      DEFAULT_CHARSET, OUT_RASTER_PRECIS, CLIP_DEFAULT_PRECIS,
-      DRAFT_QUALITY, FIXED_PITCH | FF_DONTCARE, "Terminal");
+    if (gs.nFontTxt > 0) {
+      wi.xChar = 3 + 3*i;
+      wi.yChar = (wi.xChar * 3 + 1) / 2;
+      wi.hfont = CreateFont(wi.xChar*2, 0, 0, 0, !gs.fThick ? 400 : 800,
+        fFalse, fFalse, fFalse, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS,
+        CLIP_DEFAULT_PRECIS, PROOF_QUALITY, FIXED_PITCH | FF_MODERN,
+        rgszFontName[gs.nFontTxt]);
+    } else {
+      wi.xChar = i < 2 ? 6 : (i < 3 ? 8 : (i < 4 ? 10 : 12));
+      wi.yChar = i < 2 ? 8 : (i < 3 ? 12 : (i < 4 ? 18 : 16));
+      wi.hfont = CreateFont(wi.yChar /*nHeight*/, wi.xChar /*nWidth*/,
+        0 /*nEscapement*/, 0 /*nOrientation*/, FW_DONTCARE,
+        0 /*fbItalic*/, 0 /*fbUnderline*/, 0 /*fbStrikeOut*/,
+        DEFAULT_CHARSET, OUT_RASTER_PRECIS, CLIP_DEFAULT_PRECIS,
+        DRAFT_QUALITY, FIXED_PITCH | FF_DONTCARE, "Terminal");
+    }
     hfontOld = (HFONT)SelectObject(wi.hdc, wi.hfont);
     // If printing, set the number of text rows per page.
     if (wi.hdcPrint != hdcNil) {
@@ -2674,18 +2775,19 @@ flag API FRedraw(void)
     (!wi.fAutoSaveWire ? gs.ft != ftBmp : gs.ft != ftWire)) {
     gs.ft = !wi.fAutoSaveWire ? ftBmp : ftWire;
     if (!wi.fAutoSaveNum)
-      gi.szFileOut = !wi.fAutoSaveWire ? szFileAutoCore : "astrolog.dw";
+      sprintf(szFile, "%s",
+        !wi.fAutoSaveWire ? szFileAutoCore : "astrolog.dw");
     else {
-      gi.szFileOut = szFile;
       sprintf(szFile, "ast%05d.%s",
         wi.nAutoSaveNum, !wi.fAutoSaveWire ? "bmp" : "dw");
       wi.nAutoSaveNum++;
     }
+    FCloneSz(szFile, &gi.szFileOut);
     Action();
+    FCloneSz(NULL, &gi.szFileOut);
     if (gs.ft == ftWire) {
       gs.xWin /= WIREMUL; gs.yWin /= WIREMUL; gs.nScale /= WIREMUL;
     }
-    gi.szFileOut = NULL;
     gs.ft = ftNone;
   }
 
@@ -2708,7 +2810,7 @@ flag API FRedraw(void)
       BitBlt(hdcWin, 0, 0, wi.xClient, wi.yClient,
         wi.hdc, -gi.xOffset, -gi.yOffset, SRCCOPY);
       SelectObject(wi.hdc, hbmpOld);
-      DeleteObject(hbmp);
+      DeleteObject(wi.hbmp);
       DeleteDC(wi.hdc);
     }
     EndPaint(wi.hwnd, &ps);
@@ -2736,7 +2838,7 @@ flag API FRedraw(void)
   // it on screen. If were in file mode, cleanup things here.
 
   if (is.szFileScreen != NULL || gs.ft != ftNone) {
-    is.szFileScreen = NULL;
+    FCloneSz(NULL, &is.szFileScreen);
     if (gs.ft == ftWmf) {
       gs.xWin /= METAMUL; gs.yWin /= METAMUL; gs.nScale /= METAMUL;
     } else if (gs.ft == ftPS) {
@@ -2960,6 +3062,8 @@ flag FCreateProgramGroup(flag fAll)
   DeleteShortcut(szDir, "Astrolog 7.30");
   DeleteShortcut(szDir, "Astrolog 7.40");
   DeleteShortcut(szDir, "Astrolog 7.50");
+  DeleteShortcut(szDir, "Astrolog 7.60");
+  DeleteShortcut(szDir, "Astrolog 7.70");
 
   // Add main shortcuts in folder.
   sprintf(szName, "%s %s", szAppName, szVersionCore);
