@@ -1,8 +1,8 @@
 /*
-** Astrolog (Version 7.60) File: xcharts2.cpp
+** Astrolog (Version 7.80) File: xcharts2.cpp
 **
 ** IMPORTANT NOTICE: Astrolog and all chart display routines and anything
-** not enumerated below used in this program are Copyright (C) 1991-2023 by
+** not enumerated below used in this program are Copyright (C) 1991-2025 by
 ** Walter D. Pullen (Astara@msn.com, http://www.astrolog.org/astrolog.htm).
 ** Permission is granted to freely use, modify, and distribute these
 ** routines provided these credits and notices remain unmodified with any
@@ -10,8 +10,8 @@
 **
 ** The main ephemeris databases and calculation routines are from the
 ** library SWISS EPHEMERIS and are programmed and copyright 1997-2008 by
-** Astrodienst AG. The use of that source code is subject to the license for
-** Swiss Ephemeris Free Edition, available at http://www.astro.com/swisseph.
+** Astrodienst AG. Use of that source code is subject to license for Swiss
+** Ephemeris Free Edition at https://www.astro.com/swisseph/swephinfo_e.htm.
 ** This copyright notice must not be changed or removed by any user of this
 ** program.
 **
@@ -48,7 +48,7 @@
 ** Initial programming 8/28-30/1991.
 ** X Window graphics initially programmed 10/23-29/1991.
 ** PostScript graphics initially programmed 11/29-30/1992.
-** Last code change made 4/8/2023.
+** Last code change made 6/19/2025.
 */
 
 #include "astrolog.h"
@@ -178,6 +178,9 @@ void FillSymbolRingM(int obj, real *symbol, real factor)
   int i, j, l;
   flag fMoved = fTrue;
 
+  // Keep adjusting as long as can still make changes, or until 'n' rounds are
+  // done. (With many moons, there just may not be enough room for all.)
+
   for (l = 0; fMoved && l < us.nDivision*2; l++) {
     fMoved = fFalse;
     for (i = custLo; i <= custHi; i++) {
@@ -195,10 +198,17 @@ void FillSymbolRingM(int obj, real *symbol, real factor)
         else if (temp > k2 && temp <= 0.0)
           k2 = temp;
       }
+
+      // If a moon's too close on one side, then move in other direction.
+
       if (k2 > -orb && k1 > orb) {
         fMoved = fTrue; symbol[i] = Mod(symbol[i]+orb*0.51+k2*0.49);
       } else if (k1 < orb && k2 < -orb) {
         fMoved = fTrue; symbol[i] = Mod(symbol[i]-orb*0.51+k1*0.49);
+
+      // If moon bracketed by close moons on both sides, then move it to the
+      // midpoint, so it's as far away as possible from either one.
+
       } else if (k2 > -orb && k1 < orb) {
         fMoved = fTrue; symbol[i] = Mod(symbol[i]+(k1+k2)*0.5);
       }
@@ -322,7 +332,7 @@ void DrawAspectRelation(int n1, int n2, real obj1[objMax], real obj2[objMax],
     for (i = is.nObj; i >= 0; i--)
       if (grid->n[i][j] && FProper2(i) && FProper(j) &&
         obj1[j] >= 0.0 && obj2[i] >= 0.0)
-        DrawAspectLine(i, j, cx, cy, obj1[j], obj2[i], rx, ry, rz);
+        DrawAspectLine(i, j, cx, cy, obj1[j], obj2[i], rx, ry, rz, fFalse);
 
 LExit:
   if (n1 != 1)
@@ -348,12 +358,13 @@ void XChartWheelRelation()
     symbol[objMax];
   byte ignoreT[objMax];
   int cx, cy, i;
-  real unitx, unity;
+  real ra, rp1, rl11, rl21, rz1, rg1, ri, rc, rp2, rl12, rl22, rz2, rg2,
+    rh1, rh2, rs, unitx, unity;
 
   // Set up variables and temporarily automatically decrease the horizontal
   // chart size to leave room for the sidebar if that mode is in effect.
 
-  if (gs.fText && !us.fVelocity)
+  if (gs.fText && gs.fDoSidebar)
     gs.xWin -= xSideT;
   cx = gs.xWin/2 - 1; cy = gs.yWin/2 - 1;
   unitx = (real)cx; unity = (real)cy;
@@ -361,6 +372,15 @@ void XChartWheelRelation()
     rDegQuad*(gs.objLeft < 0) : cp1.cusp[1];
   if (us.fIndian)
     gi.rAsc = gs.objLeft ? (gs.objLeft < 0 ? 120.0 : -60.0)-gi.rAsc : 0.0;
+  if (!gs.fLabelCity) {
+    ra = 0.40; rp1 = 0.41; rl11 = 0.43; rl21 = 0.46; rz1 = 0.0; rg1 = 0.50;
+    ri = 0.54; rc = 0.55; rp2 = 0.56; rl12 = 0.58; rl22 = 0.61; rz2 = 0.0;
+    rg2 = 0.65; rh1 = 0.70; rh2 = 0.78; rs = 0.82;
+  } else {
+    ra = 0.26; rp1 = 0.27; rl11 = 0.29; rl21 = 0.32; rz1 = 0.38; rg1 = 0.46;
+    ri = 0.50; rc = 0.51; rp2 = 0.52; rl12 = 0.54; rl22 = 0.57; rz2 = 0.63;
+    rg2 = 0.71; rh1 = 0.76; rh2 = 0.82; rs = 0.86;
+  }
 
   // Fill out arrays with the degree of each object, cusp, and sign glyph.
 
@@ -382,11 +402,9 @@ void XChartWheelRelation()
   // Go draw the outer sign and house rings. We are drawing only the houses
   // of one of the two charts in the relationship, however.
 
-  if (gs.fColor) {
-    DrawColor(kDkGreenB);
-    DrawCircle(cx, cy, (int)(unitx*0.55+rRound), (int)(unity*0.55+rRound));
-  }
-  DrawWheel(xsign, xhouse1, cx, cy, unitx, unity, 0.70, 0.78, 0.82);
+  DrawColor(kDkGreenB);
+  DrawCircle(cx, cy, (int)(unitx*rc+rRound), (int)(unity*rc+rRound));
+  DrawWheel(xsign, xhouse1, cx, cy, unitx, unity, rh1, rh2, rs);
 
   // Draw the outer ring of planets (based on the planets in the chart which
   // the houses do not reflect - the houses belong to the inner ring below).
@@ -400,7 +418,7 @@ void XChartWheelRelation()
       ignore[i] = ignore2[i];
     }
   DrawRing(2, 2 /* so lines are dotted */ + 1, xplanet2, symbol, cx, cy,
-    0.41, 0.43, 0.54, 0.56, 0.58, 0.61, 0.65, 1.0);
+    rp1, rl11, ri, rp2, rl12, rl22, rz2, rg2, 1.0);
   if (us.nRel <= rcTransit)
     for (i = 0; i <= is.nObj; i++)
       ignore[i] = ignoreT[i];
@@ -410,13 +428,13 @@ void XChartWheelRelation()
   // only one set of planets. Again, draw glyph, and a line to the true point.
 
   DrawRing(1, 2, xplanet1, symbol, cx, cy,
-    0.0, 0.0, 0.0, 0.41, 0.43, 0.46, 0.50, 1.1);
+    0.0, 0.0, 0.0, rp1, rl11, rl21, rz1, rg1, 1.1);
   FProcessCommandLine(szWheelX[0]);
 
   // Draw lines connecting planets between the two charts that have aspects.
 
   if (!gs.fEquator)
-    DrawAspectRelation(1, 2, xplanet1, xplanet2, cx, cy, 0.40);
+    DrawAspectRelation(1, 2, xplanet1, xplanet2, cx, cy, ra);
 
   // Draw sidebar with chart information and positions if need be.
 
@@ -424,22 +442,25 @@ void XChartWheelRelation()
 }
 
 
-// Draw a tri-wheel chart or quad-wheel chart, in which there are three or
-// four rings, among three or four sets of chart data being compared. This
-// chart is obtained when the -r3 or -r4 switch is combined with -X switch.
+// Draw a tri-wheel chart, quad-wheel chart, quin-wheel, or hexa-wheel, in
+// which there are three to six rings, among three to six sets of chart data
+// being compared. This chart is obtained when the -r3, -r4, -r5, or -r6
+// switch is combined with -X switch.
 
 void XChartWheelMulti()
 {
   real xsign[cSign+1], xhouse1[cSign+1], xplanet1[objMax], xplanet2[objMax],
     xplanet3[objMax], xplanet4[objMax], xplanet5[objMax], xplanet6[objMax],
     symbol[objMax], ri2, rp, rl1, rl2, rg, rT;
+  CP *pcp[cRing+1];
+  real *pxp[cRing+1];
   int cx, cy, i, fQuad, fQuin, fHexa, nRing;
   real unitx, unity, base, base2, off;
 
   // Set up variables and temporarily automatically decrease the horizontal
   // chart size to leave room for the sidebar if that mode is in effect.
 
-  if (gs.fText && !us.fVelocity)
+  if (gs.fText && gs.fDoSidebar)
     gs.xWin -= xSideT;
   cx = gs.xWin/2 - 1; cy = gs.yWin/2 - 1;
   unitx = (real)cx; unity = (real)cy;
@@ -454,6 +475,12 @@ void XChartWheelMulti()
   base = (fHexa ? 0.11 : (fQuin ? 0.22 : (fQuad ? 0.23 : 0.36)));
   base2 = base + (fQuin ? 0.01 : 0.02);
   off = fQuin ? 0.11 : 0.13;
+  pxp[1] = xplanet6; pxp[2] = xplanet5; pxp[3] = xplanet4;
+  pxp[4] = xplanet3; pxp[5] = xplanet2; pxp[6] = xplanet1;
+  for (i = 1; i <= nRing; i++) {
+    pcp[i] = rgpcp[nRing+1 - i];
+    pxp[i] = pxp[i + cRing - nRing];
+  }
 
   // Fill out arrays with the degrees of the cusps and sign glyphs, and the
   // positions of the planet rings.
@@ -469,15 +496,15 @@ void XChartWheelMulti()
   for (i = 1; i <= cSign; i++)
     xsign[i] = PZ(HousePlaceInX(ZFromS(i), 0.0));
   for (i = 0; i <= is.nObj; i++) {
-    xplanet1[i] = PZ(HousePlaceInX(cp1.obj[i], cp1.alt[i]));
-    xplanet2[i] = PZ(HousePlaceInX(cp2.obj[i], cp2.alt[i]));
-    xplanet3[i] = PZ(HousePlaceInX(cp3.obj[i], cp3.alt[i]));
+    xplanet1[i] = PZ(HousePlaceInX(pcp[1]->obj[i], pcp[1]->alt[i]));
+    xplanet2[i] = PZ(HousePlaceInX(pcp[2]->obj[i], pcp[2]->alt[i]));
+    xplanet3[i] = PZ(HousePlaceInX(pcp[3]->obj[i], pcp[3]->alt[i]));
     if (fQuad) {
-      xplanet4[i] = PZ(HousePlaceInX(cp4.obj[i], cp4.alt[i]));
+      xplanet4[i] = PZ(HousePlaceInX(pcp[4]->obj[i], pcp[4]->alt[i]));
       if (fQuin) {
-        xplanet5[i] = PZ(HousePlaceInX(cp5.obj[i], cp5.alt[i]));
+        xplanet5[i] = PZ(HousePlaceInX(pcp[5]->obj[i], pcp[5]->alt[i]));
         if (fHexa)
-          xplanet6[i] = PZ(HousePlaceInX(cp6.obj[i], cp6.alt[i]));
+          xplanet6[i] = PZ(HousePlaceInX(pcp[6]->obj[i], pcp[6]->alt[i]));
       }
     }
   }
@@ -485,22 +512,20 @@ void XChartWheelMulti()
   // Go draw the outer sign and house rings. We are drawing the houses of only
   // the outermost ring of the wheel, however.
 
-  if (gs.fColor) {
-    DrawColor(kDkGreenB);
-    rT = fQuin ? 0.64 : 0.61;
-    DrawCircle(cx, cy, (int)(unitx*rT+rRound), (int)(unity*rT+rRound));
+  DrawColor(kDkGreenB);
+  rT = fQuin ? 0.64 : 0.61;
+  DrawCircle(cx, cy, (int)(unitx*rT+rRound), (int)(unity*rT+rRound));
+  rT -= off;
+  DrawCircle(cx, cy, (int)(unitx*rT+rRound), (int)(unity*rT+rRound));
+  if (fQuad) {
     rT -= off;
     DrawCircle(cx, cy, (int)(unitx*rT+rRound), (int)(unity*rT+rRound));
-    if (fQuad) {
+    if (fQuin) {
       rT -= off;
       DrawCircle(cx, cy, (int)(unitx*rT+rRound), (int)(unity*rT+rRound));
-      if (fQuin) {
+      if (fHexa)
         rT -= off;
         DrawCircle(cx, cy, (int)(unitx*rT+rRound), (int)(unity*rT+rRound));
-        if (fHexa)
-          rT -= off;
-          DrawCircle(cx, cy, (int)(unitx*rT+rRound), (int)(unity*rT+rRound));
-      }
     }
   }
   if (fQuin)
@@ -519,14 +544,14 @@ void XChartWheelMulti()
     ri2 = 0.59; rp = 0.62; rl1 = 0.63; rl2 = 0.66; rg = 0.70;
   }
   DrawRing(1, nRing, xplanet1, symbol, cx, cy,
-    base, base2, ri2, rp, rl1, rl2, rg, 0.9);
+    base, base2, ri2, rp, rl1, rl2, 0.0, rg, 0.9);
 
   // Now draw the second to outermost ring of planets. Again, draw each glyph,
   // a line to its true point, and a line to the innermost ring.
 
   ri2 -= off; rp -= off; rl1 -= off; rl2 -= off; rg -= off;
   DrawRing(2, nRing, xplanet2, symbol, cx, cy,
-    base, base2, ri2, rp, rl1, rl2, rg, 1.1);
+    base, base2, ri2, rp, rl1, rl2, 0.0, rg, 1.1);
 
   // The third ring is next. Chart was cast earlier, and draw the glyphs and
   // lines to true point. If a fourth ring is being done, first finish the
@@ -534,7 +559,7 @@ void XChartWheelMulti()
 
   ri2 -= off; rp -= off; rl1 -= off; rl2 -= off; rg -= off;
   DrawRing(3, nRing, xplanet3, symbol, cx, cy,
-    base, base2, ri2, rp, rl1, rl2, rg, 1.4);
+    base, base2, ri2, rp, rl1, rl2, 0.0, rg, 1.4);
 
   if (fQuad) {
     // If the fourth ring is being done, take the chart that was cast earlier,
@@ -544,7 +569,7 @@ void XChartWheelMulti()
 
     ri2 -= off; rp -= off; rl1 -= off; rl2 -= off; rg -= off;
     DrawRing(4, nRing, xplanet4, symbol, cx, cy,
-      base, base2, ri2, rp, rl1, rl2, rg, 1.8);
+      base, base2, ri2, rp, rl1, rl2, 0.0, rg, 1.8);
 
     if (fQuin) {
       // If the fifth ring is being done, take the chart that was cast
@@ -554,7 +579,7 @@ void XChartWheelMulti()
 
       ri2 -= off; rp -= off; rl1 -= off; rl2 -= off; rg -= off;
       DrawRing(5, nRing, xplanet5, symbol, cx, cy,
-        base, base2, ri2, rp, rl1, rl2, rg, 2.3);
+        base, base2, ri2, rp, rl1, rl2, 0.0, rg, 2.3);
 
       if (fHexa) {
         // If the sixth (innermost) ring is being done, take the chart that was
@@ -562,7 +587,7 @@ void XChartWheelMulti()
 
         ri2 -= off; rp -= off; rl1 -= off; rl2 -= off; rg -= off;
         DrawRing(6, nRing, xplanet6, symbol, cx, cy,
-          base, base2, ri2, rp, rl1, rl2, rg, 3.8);
+          base, base2, ri2, rp, rl1, rl2, 0.0, rg, 3.8);
       }
     }
   }
@@ -572,24 +597,24 @@ void XChartWheelMulti()
 
   if (!gs.fEquator) {
     base -= 0.02;
-    DrawAspectRelation(1, 2, xplanet1, xplanet2, cx, cy, base);
-    DrawAspectRelation(1, 3, xplanet1, xplanet3, cx, cy, base);
-    DrawAspectRelation(2, 3, xplanet2, xplanet3, cx, cy, base);
+    DrawAspectRelation(1, 2, pxp[1], pxp[2], cx, cy, base);
+    DrawAspectRelation(1, 3, pxp[1], pxp[3], cx, cy, base);
+    DrawAspectRelation(2, 3, pxp[2], pxp[3], cx, cy, base);
     if (fQuad) {
-      DrawAspectRelation(1, 4, xplanet1, xplanet4, cx, cy, base);
-      DrawAspectRelation(2, 4, xplanet2, xplanet4, cx, cy, base);
-      DrawAspectRelation(3, 4, xplanet3, xplanet4, cx, cy, base);
+      DrawAspectRelation(1, 4, pxp[1], pxp[4], cx, cy, base);
+      DrawAspectRelation(2, 4, pxp[2], pxp[4], cx, cy, base);
+      DrawAspectRelation(3, 4, pxp[3], pxp[4], cx, cy, base);
       if (fQuin) {
-        DrawAspectRelation(1, 5, xplanet1, xplanet5, cx, cy, base);
-        DrawAspectRelation(2, 5, xplanet2, xplanet5, cx, cy, base);
-        DrawAspectRelation(3, 5, xplanet3, xplanet5, cx, cy, base);
-        DrawAspectRelation(4, 5, xplanet4, xplanet5, cx, cy, base);
+        DrawAspectRelation(1, 5, pxp[1], pxp[5], cx, cy, base);
+        DrawAspectRelation(2, 5, pxp[2], pxp[5], cx, cy, base);
+        DrawAspectRelation(3, 5, pxp[3], pxp[5], cx, cy, base);
+        DrawAspectRelation(4, 5, pxp[4], pxp[5], cx, cy, base);
         if (fHexa) {
-          DrawAspectRelation(1, 6, xplanet1, xplanet6, cx, cy, base);
-          DrawAspectRelation(2, 6, xplanet2, xplanet6, cx, cy, base);
-          DrawAspectRelation(3, 6, xplanet3, xplanet6, cx, cy, base);
-          DrawAspectRelation(4, 6, xplanet4, xplanet6, cx, cy, base);
-          DrawAspectRelation(5, 6, xplanet5, xplanet6, cx, cy, base);
+          DrawAspectRelation(1, 6, pxp[1], pxp[6], cx, cy, base);
+          DrawAspectRelation(2, 6, pxp[2], pxp[6], cx, cy, base);
+          DrawAspectRelation(3, 6, pxp[3], pxp[6], cx, cy, base);
+          DrawAspectRelation(4, 6, pxp[4], pxp[6], cx, cy, base);
+          DrawAspectRelation(5, 6, pxp[5], pxp[6], cx, cy, base);
         }
       }
     }
@@ -790,8 +815,9 @@ void XChartEphemeris()
     ciCore = ciMain;
     if (cYea) {
       MM = mon; DD = day; YY = yea;
-    } else
-      DD = d;
+    } else {
+      MM = mon0; DD = d; YY = yea0;
+    }
     if (us.fProgress) {
       is.JDp = MdytszToJulian(MM, DD, YY, TT, SS, ZZ);
       ciCore = ciMain;
@@ -808,8 +834,9 @@ void XChartEphemeris()
     // Draw planet glyphs along top of chart.
     if (d <= 1) {
       for (i = 0; i <= is.nObj; i++) {
-        j = !FProper(i);
+        j = !FProperEphem2(i);
         symbol[i*2] = (j || us.nRel > rcDual) ? -rLarge : cp2.obj[i];
+        j = !FProper(i);
         symbol[i*2+1] = (j ? -rLarge : planet[i]);
       }
       FillSymbolLine(symbol);
@@ -822,7 +849,7 @@ void XChartEphemeris()
       gs.fLabel = fSav;
       if (us.nRel <= rcDual) {
         for (i = is.nObj; i >= 0; i--) {
-          if (!FProper(i))
+          if (!FProperEphem2(i))
             continue;
           j = x1 + (int)((real)xs * cp2.obj[i] / rDegMax);
           DrawColor(kObjB[i]);
@@ -1171,8 +1198,7 @@ void XChartTransit(flag fTrans, flag fProg)
           continue;
         ppw = &(*rgEph)[x][y][asp];
         if (*ppw == NULL) {
-          *ppw = (word *)PAllocate(cTot * sizeof(word),
-            "transit ephemeris entry");
+          *ppw = RgAllocate(cTot, word, "transit ephemeris entry");
           if (*ppw == NULL)
             goto LDone;
           pw = *ppw;
@@ -1190,8 +1216,7 @@ void XChartTransit(flag fTrans, flag fProg)
           if (et > etNone) {
             ppw = &(*rgEph)[y][x][asp];
             if (*ppw == NULL) {
-              *ppw = (word *)PAllocate(cTot * sizeof(word),
-                "transit eclipse entry");
+              *ppw = RgAllocate(cTot, word, "transit eclipse entry");
               if (*ppw == NULL)
                 goto LDone;
               pw2 = *ppw;
@@ -1371,16 +1396,13 @@ LDone:
     for (x = 0; x < (fTrans ? is.nObj+1 : y); x++)
       for (asp = 1; asp <= cAsp; asp++) {
         pw = (*rgEph)[x][y][asp];
-        if (pw != NULL)
-          DeallocateP(pw);
+        DeallocatePIf(pw);
         if (fEclipse) {
           pw2 = (*rgEph)[y][x][asp];
-          if (pw2 != NULL)
-            DeallocateP(pw2);
+          DeallocatePIf(pw2);
         }
       }
-  if (rgEph != NULL)
-    DeallocateP(rgEph);
+  DeallocatePIf(rgEph);
   ciCore = ciMain;
   us.fProgress = fFalse;
   CastChart(1);
