@@ -1,8 +1,8 @@
 /*
-** Astrolog (Version 7.60) File: xdevice.cpp
+** Astrolog (Version 7.80) File: xdevice.cpp
 **
 ** IMPORTANT NOTICE: Astrolog and all chart display routines and anything
-** not enumerated below used in this program are Copyright (C) 1991-2023 by
+** not enumerated below used in this program are Copyright (C) 1991-2025 by
 ** Walter D. Pullen (Astara@msn.com, http://www.astrolog.org/astrolog.htm).
 ** Permission is granted to freely use, modify, and distribute these
 ** routines provided these credits and notices remain unmodified with any
@@ -10,8 +10,8 @@
 **
 ** The main ephemeris databases and calculation routines are from the
 ** library SWISS EPHEMERIS and are programmed and copyright 1997-2008 by
-** Astrodienst AG. The use of that source code is subject to the license for
-** Swiss Ephemeris Free Edition, available at http://www.astro.com/swisseph.
+** Astrodienst AG. Use of that source code is subject to license for Swiss
+** Ephemeris Free Edition at https://www.astro.com/swisseph/swephinfo_e.htm.
 ** This copyright notice must not be changed or removed by any user of this
 ** program.
 **
@@ -48,7 +48,7 @@
 ** Initial programming 8/28-30/1991.
 ** X Window graphics initially programmed 10/23-29/1991.
 ** PostScript graphics initially programmed 11/29-30/1992.
-** Last code change made 4/8/2023.
+** Last code change made 6/19/2025.
 */
 
 #include "astrolog.h"
@@ -118,8 +118,7 @@ flag FAllocateBmp(Bitmap *b, int x, int y)
     rgb = (byte *)PAllocate(cb, "color bitmap");
     if (rgb == NULL)
       return fFalse;
-    if (b->rgb != NULL)
-      DeallocateP(b->rgb);
+    DeallocatePIf(b->rgb);
     b->rgb = rgb;
   }
   b->x = x; b->y = y;
@@ -340,6 +339,40 @@ void BmpCopyBlock(CONST Bitmap *bs, int x1, int y1, int x2, int y2,
 }
 
 
+// Adjust the color of a pixel on a world map, based on whether the location
+// is at night time, or whether the location is under a solar eclipse.
+
+void BmpDarkenKv(real lon, real lat, real lonS, real latS, flag fDoEclipse,
+  KV *pkv)
+{
+  KV kv = *pkv;
+#ifdef SWISS
+  int et;
+  real rEclipse;
+#endif
+
+  // Check whether the location is in the night time half of the world.
+  if (SphDistance(lonS, latS, lon, rDegQuad - lat) > rDegQuad) {
+    *pkv = Rgb(RgbR(kv) / 3, RgbG(kv) / 3, RgbB(kv) / 3);
+    return;
+  }
+
+#ifdef SWISS
+  // Check for a partial or annular/total solar eclipse at the location.
+  if (!fDoEclipse)
+    return;
+  et = NCheckEclipseSolarLoc(rDegHalf - lon, rDegQuad - lat, &rEclipse);
+  if (et <= etNone)
+    return;
+  if (et <= etPartial) {
+    *pkv = KvBlend(kv, kBlack, 0.33 + rEclipse/100.0*0.33);
+    return;
+  }
+  *pkv = KvBlend(kv, rgbbmp[kRed], 0.5);
+#endif
+}
+
+
 // Like BmpCopyBlock() but the source rectangle coordinates are reals instead
 // of integers. Can be used to render only subsections of source pixels.
 
@@ -347,8 +380,10 @@ void BmpCopyBlock2(CONST Bitmap *bs, real x1, real y1, real x2, real y2,
   Bitmap *bd, int x3, int y3, int x4, int y4)
 {
   int xd = x4-x3+1, yd = y4-y3+1, x, y, xT, yT, nR, nG, nB;
-  real xs, ys;
+  real xs, ys, lonS, latS, rx, ry;
   byte *pbDst;
+  flag fDoEclipse = fFalse;
+  KV kv;
 
   // Sanity checks of coordinate bounds, which shouldn't ever fail.
   Assert(FBetween(x1, 0.0, (real)bs->x - rSmall));
@@ -360,16 +395,41 @@ void BmpCopyBlock2(CONST Bitmap *bs, real x1, real y1, real x2, real y2,
   Assert(FBetween(x4, 0, bd->x-1));
   Assert(FBetween(y4, 0, bd->y-1));
 
+  if (gs.fMollewide) {
+    lonS = Tropical(planet[oSun]);
+    latS = planetalt[oSun];
+    EclToEqu(&lonS, &latS);
+    lonS = Mod(lonS - cp0.lonMC + rDegHalf - Lon);
+    if (us.fEclipse &&
+      NCheckEclipseSolar(oEar, oMoo, oSun, NULL) > etNone)
+      fDoEclipse = fTrue;
+  }
+
   xs = (x2-x1) / (real)xd;
   ys = (y2-y1) / (real)yd;
   for (y = y3; y <= y4; y++) {
     pbDst = _PbXY(bd, x3, y);
-    yT = (int)(y1 + (real)(y-y3) * ys);
-    for (x = x3; x <= x4; x++) {
-      xT = (int)(x1 + (real)(x-x3) * xs);
-      _GetRGB(_PbXY(bs, xT, yT), &nR, &nG, &nB);
-      _SetRGB(pbDst, nR, nG, nB);
-      pbDst += cbPixelK;
+    ry = y1 + (real)(y-y3) * ys;
+    yT = (int)ry;
+    if (!gs.fMollewide) {
+      // Fast loop that just copies pixels.
+      for (x = x3; x <= x4; x++) {
+        xT = (int)(x1 + (real)(x-x3) * xs);
+        _GetRGB(_PbXY(bs, xT, yT), &nR, &nG, &nB);
+        _SetRGB(pbDst, nR, nG, nB);
+        pbDst += cbPixelK;
+      }
+    } else {
+      // Slower loop that may modify colors based on position on Earth.
+      ry = ry * rDegMax / (real)bs->x;
+      for (x = x3; x <= x4; x++) {
+        rx = x1 + (real)(x-x3) * xs;
+        xT = (int)rx;
+        rx = rx * rDegMax / (real)bs->x;
+        kv = _GetXY(bs, xT, yT);
+        BmpDarkenKv(rx, ry, lonS, latS, fDoEclipse, &kv);
+        BmpSetXY(bd, x, y, kv);
+      }
     }
   }
 }
@@ -520,6 +580,7 @@ flag FBmpDrawMap()
   real deg = Mod(rDegMax - gs.rRot), lonS, latS, rxc, ryc, rzc,
     lon, lat, lat0, rT, rLen, sint, cost, sina, cosa;
   KV kv;
+  flag fDoEclipse = fFalse;
 
   // Do nothing if not drawing bitmaps, or if the Earth bitmap fails to load.
   if (!gi.fBmp || (gi.fFile && gs.ft != ftBmp))
@@ -533,6 +594,9 @@ flag FBmpDrawMap()
     bmp = &wi.bmpWin;
   }
 #endif
+  if (gs.fMollewide && us.fEclipse &&
+    NCheckEclipseSolar(oEar, oMoo, oSun, NULL) > etNone)
+    fDoEclipse = fTrue;
 
   // Compute center coordinates and horizontal map dimensions.
   xc = (gs.xWin >> 1) - !FOdd(gs.xWin); yc = (gs.yWin >> 1) - !FOdd(gs.yWin);
@@ -610,9 +674,8 @@ flag FBmpDrawMap()
         x2 = (int)(lon * ((real)gi.bmpWorld.x - rSmall) / rDegMax);
         y2 = (int)(lat * ((real)gi.bmpWorld.y - rSmall) / rDegHalf);
         kv = _GetXY(&gi.bmpWorld, x2, y2);
-        if (gs.fMollewide &&
-          SphDistance(lonS, latS, lon, rDegQuad - lat) > rDegQuad)
-          kv = Rgb(RgbR(kv) / 3, RgbG(kv) / 3, RgbB(kv) / 3);
+        if (gs.fMollewide)
+          BmpDarkenKv(lon, lat, lonS, latS, fDoEclipse, &kv);
         BmpSetXY(bmp, x1, y1, kv);
       }
     }
@@ -684,9 +747,8 @@ flag FBmpDrawMap()
         x2 = (int)(lon * ((real)gi.bmpWorld.x - rSmall) / rDegMax);
         y2 = (int)(lat * ((real)gi.bmpWorld.y - rSmall) / rDegHalf);
         kv = _GetXY(&gi.bmpWorld, x2, y2);
-        if (gs.fMollewide &&
-          SphDistance(lonS, latS, lon, rDegQuad - lat) > rDegQuad)
-          kv = Rgb(RgbR(kv) / 3, RgbG(kv) / 3, RgbB(kv) / 3);
+        if (gs.fMollewide)
+          BmpDarkenKv(lon, lat, lonS, latS, fDoEclipse, &kv);
         BmpSetXY(bmp, x1, y1, kv);
       }
     }
@@ -727,7 +789,7 @@ flag FBmpDrawMap2(int x1, int y1, int x2, int y2,
   rx1 = Mod(rx1); rx2 = Mod(rx2);
   x3 = rx1 * rx; y3 = ry1 * ry;
   x4 = rx2 * rx; y4 = ry2 * ry;
-  if (x3 <= x4) {
+  if (x3 < x4 && x4-x3 > rSmall) {
     // In most cases, just copy the entire rectangle all at once.
     BmpCopyBlock2(&gi.bmpWorld, x3, y3, x4, y4, bmp, x1, y1, x2, y2);
   } else {
@@ -746,6 +808,88 @@ flag FBmpDrawMap2(int x1, int y1, int x2, int y2,
 }
 
 
+// Adjust the window or bitmap's content to be smoother, and look antialiased.
+
+flag FBmpAntialias()
+{
+#ifdef WINANY
+  BITMAPINFO bi;
+#endif
+  Bitmap *bmp = &gi.bmp;
+  int x, y, n1, n2, n3, n4;
+  KV kv1, kv2, kv3, kv4;
+  real rBlend = !gs.fInverse ? 0.55 : 0.67;
+
+  if (!gi.fBmp || (gi.fFile && gs.ft != ftBmp))
+    return fTrue;
+#ifdef WINANY
+  // Copy the contents of the window about to be displayed to a bitmap.
+  if (!gi.fFile) {
+#ifdef WIN
+    if (!wi.fBuffer || wi.hdcPrint != NULL)
+      return fTrue;
+#endif
+    if (!FAllocateBmp(bmp, gs.xWin, gs.yWin))
+      return fFalse;
+    bi.bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
+    bi.bmiHeader.biPlanes = 1;
+    bi.bmiHeader.biBitCount = cbPixelK << 3;
+    bi.bmiHeader.biCompression = BI_RGB;
+    bi.bmiHeader.biSizeImage = 0;
+    bi.bmiHeader.biXPelsPerMeter = bi.bmiHeader.biYPelsPerMeter = 1000;
+    bi.bmiHeader.biClrUsed = 0;
+    bi.bmiHeader.biClrImportant = 0;
+    bi.bmiColors[0].rgbBlue = bi.bmiColors[0].rgbGreen =
+      bi.bmiColors[0].rgbRed = bi.bmiColors[0].rgbReserved = 0;
+    bi.bmiHeader.biWidth  =  (bmp->x);
+    bi.bmiHeader.biHeight = -(bmp->y);
+    GetDIBits(wi.hdc, wi.hbmp, 0, gs.yWin, bmp->rgb, &bi, DIB_RGB_COLORS);
+  }
+#endif
+
+  // Antialias the content on the bitmap.
+  for (y = 0; y < gs.yWin - 1; y++)
+    for (x = 0; x < gs.xWin - 1; x++) {
+      // Check each 2x2 pixel section.
+      kv1 = BmpGetXY(bmp, x, y);
+      kv2 = BmpGetXY(bmp, x+1, y);
+      kv3 = BmpGetXY(bmp, x, y+1);
+      kv4 = BmpGetXY(bmp, x+1, y+1);
+      // If all four pixels the same, skip this block.
+      if (kv1 == kv2 && kv2 == kv3 && kv3 == kv4)
+        continue;
+      // If there isn't any diagonal of pixels the same, skip.
+      if (kv1 != kv4 && kv2 != kv3)
+        continue;
+      n1 = RgbR(kv1) + RgbG(kv1) + RgbB(kv1);
+      n2 = RgbR(kv2) + RgbG(kv2) + RgbB(kv2);
+      n3 = RgbR(kv3) + RgbG(kv3) + RgbB(kv3);
+      n4 = RgbR(kv4) + RgbG(kv4) + RgbB(kv4);
+      if (gs.fInverse) {
+        n1 = 768 - n1;
+        n2 = 768 - n2;
+        n3 = 768 - n3;
+        n4 = 768 - n4;
+      }
+      // If a diagonal of pixels is brigher than the other two, blend.
+      if (kv1 == kv4 && n1 >= n2 && n1 >= n3) {
+        BmpSetXY(bmp, x+1, y, KvBlend(kv1, kv2, rBlend));
+        BmpSetXY(bmp, x, y+1, KvBlend(kv1, kv3, rBlend));
+      }
+      if (kv2 == kv3 && n2 >= n1 && n2 >= n4) {
+        BmpSetXY(bmp, x, y,     KvBlend(kv2, kv1, rBlend));
+        BmpSetXY(bmp, x+1, y+1, KvBlend(kv2, kv4, rBlend));
+      }
+    }
+
+#ifdef WINANY
+  if (!gi.fFile)
+    BmpCopyWin(bmp, wi.hdc, 0, 0);
+#endif
+  return fTrue;
+}
+
+
 /*
 ******************************************************************************
 ** Bitmap File Routines.
@@ -756,14 +900,14 @@ flag FBmpDrawMap2(int x1, int y1, int x2, int y2,
 // read in by the Unix X11 commands bitmap and xsetroot. The 'mode' parameter
 // defines how much white space is put in the file.
 
-void WriteXBitmap(FILE *file, CONST char *name, char mode)
+void WriteXBitmap(FILE *file, CONST char *szName, char mode)
 {
   int x, y, i, temp = 0;
   uint value;
   char szT[cchSzDef], *pchStart, *pchEnd;
 
   // Determine variable name from filename.
-  sprintf(szT, "%s", name);
+  sprintf(szT, "%s", szName);
   for (pchEnd = szT; *pchEnd != chNull; pchEnd++)
     ;
   for (pchStart = pchEnd; pchStart > szT &&
@@ -933,7 +1077,7 @@ flag BeginFileX()
         gs.ft == ftBmp ? "bitmap" : (gs.ft == ftPS ? "PostScript" :
         (gs.ft == ftWmf ? "metafile" : "wireframe")));
       InputString(sz, sz);
-      gi.szFileOut = SzCopy(sz);
+      FCloneSz(sz, &gi.szFileOut);
    }
 #else
     // If autosaving in potentially rapid succession, ensure the file isn't
@@ -955,7 +1099,7 @@ flag BeginFileX()
 #endif
     sprintf(sz, "Couldn't create output file: %s", gi.szFileOut);
     PrintWarning(sz);
-    gi.szFileOut = NULL;
+    FCloneSz(NULL, &gi.szFileOut);
 #ifdef WIN
     break;
 #endif
@@ -1137,9 +1281,9 @@ void PsFont(int nFont)
     return;
   szFont = rgszFontName[nFont];
   z = PSMUL*gi.nScale;
-  if (nFont == fiAstrolog) {
+  if (nFont == fiArial) {
     szFont = "Times-Roman"; z = 4*PSMUL*gi.nScaleText;
-  } else if (nFont == fiCourier) {
+  } else if (nFont == fiCourier || nFont == fiAstrolog) {
     szFont = "Courier"; z = 5*PSMUL*gi.nScaleText;
   }
   fprintf(gi.file, "/%s[%d 0 0 -%d 0 0]sf\n", szFont, z, z);
@@ -1536,7 +1680,7 @@ void WireSphere(int x, int y, int z, int r)
     rgz[i] = z + (int)((real)r * rgx[i]);
     rgr[i] = (real)r * rgy[i];
   }
-  for (j = 0; j <= 6; j++) {
+  for (j = 1; j <= 6; j++) {
     x2 = x + (int)rgr[j]; y2 = y;
     for (i = 0; i < 12; i++) {
       x1 = x2; y1 = y2;
@@ -1544,8 +1688,6 @@ void WireSphere(int x, int y, int z, int r)
       y2 = y + (int)(rgr[j] * rgy[i+1]);
       if (j < 6)
         WireLine(x1, y1, rgz[j], x2, y2, rgz[j]);
-      if (j <= 0)
-        continue;
       x1 = x + (int)(rgr[j-1] * rgx[i+1]);
       y1 = y + (int)(rgr[j-1] * rgy[i+1]);
       WireLine(x2, y2, rgz[j], x1, y1, rgz[j-1]);
@@ -1554,7 +1696,6 @@ void WireSphere(int x, int y, int z, int r)
 }
 
 
-#ifdef SWISS
 // Add a fixed star to the current wireframe file.
 
 void WireStar(int x, int y, int z, ES *pes)
@@ -1585,7 +1726,6 @@ void WireStar(int x, int y, int z, ES *pes)
   gi.zDefault = z + 8*gi.nScaleT;
   DrawSz(pes->pchBest, x, y, dtCent);
 }
-#endif
 
 
 // Given longitude and latitude values on a globe, return the 3D pixel
@@ -1647,9 +1787,7 @@ void WireDrawGlobe(flag fSky, real deg)
     rz, unit = 12*gi.nScale,
     x, y, z, xold, yold, zold, m, n, o, u, v, w, i, j, k, l;
   real planet1[objMax], planet2[objMax], x1, y1, rT;
-#ifdef SWISS
   ES es;
-#endif
 
   // Set up some variables.
   rz = Min(gs.xWin/2, gs.yWin/2);
@@ -1683,11 +1821,13 @@ void WireDrawGlobe(flag fSky, real deg)
     }
     // For globes, have to do a complicated transformation.
     WireGlobeCalc((real)xold, (real)yold, &m, &n, &o, rz, rT);
+#ifdef CONSTEL
     if (fSky && i > 0) {
       gi.zDefault = o;
       DrawSz(szCnstlAbbrev[i], m, n, dtCent | dtScale2);
       continue;
     }
+#endif
     WireGlobeCalc((real)x, (real)y, &u, &v, &w, rz, rT);
     WireLine(m, n, o, u, v, w);
   }
@@ -1799,6 +1939,17 @@ void WireDrawGlobe(flag fSky, real deg)
   }
 #endif
 
+  // Draw exoplanets.
+
+  if (gs.fAllExo) {
+    EnumExoplanets(NULL);
+    while (EnumExoplanets(&es)) {
+      x1 = es.lon; y1 = es.lat;
+      WireMapCalc(x1, y1, &j, &k, &l, fSky, rT, rz, deg);
+        WireStar(j, k, l, &es);
+    }
+  }
+
   // Draw grid of triangles or squares over the planet.
 
   DrawMapTriangles(fTrue, rz, NULL, deg);
@@ -1971,11 +2122,12 @@ void WireChartOrbit()
       // Draw rings around Saturn or other planet.
       j = FBetween(i, oJuC, oNeC) ? i - oJuC + oJup :
         (FBetween(i, oJup, oNep) && ignore[i + oJuC - oJup] ? i :
-        (i == oHau ? i : -1));
+        (i == oHau || i == oQua ? i : -1));
       if (j >= 0) {
         PT3R vCross, ptCen;
         real tilt, rot;
-        vCross = rgvObjRing[IObjRing(j)];
+        j = IObjRing(j);
+        vCross = rgvObjRing[j];
         // Adjust ring vector appropriately if in sidereal zodiac.
         if (is.rSid != 0.0) {
           rT = RLength2(vCross.x, vCross.y);
@@ -1989,12 +2141,12 @@ void WireChartOrbit()
         vCross.z = 0.0;
         PtSet(ptCen, 1.0, 0.0, 0.0);
         rot = VAngleD(&vCross, &ptCen);
-        if (rgvObjRing[IObjRing(j)].y < 0.0)
+        if (rgvObjRing[j].y < 0.0)
           rot = rDegMax - rot;
-        k = (int)(rgrObjRing[IObjRing(j)][0] / rAUToKm * sx);
+        k = (int)(rgrObjRing[j][0] / rAUToKm * sx);
         WireCircle(x[i], y[i], z[i], (real)k, tilt, rot);
-        if (rgrObjRing[IObjRing(j)][1] > 0.0) {
-          k = (int)(rgrObjRing[IObjRing(j)][1] / rAUToKm * sx);
+        if (rgrObjRing[j][1] > 0.0) {
+          k = (int)(rgrObjRing[j][1] / rAUToKm * sx);
           WireCircle(x[i], y[i], z[i], (real)k, tilt, rot);
         }
       }
@@ -2139,29 +2291,36 @@ void WireSphereEarth(real azi, real alt, int zr, int *xp, int *yp, int *zp)
 void WireChartSphere()
 {
   char sz[cchSzDef];
-  int rgx[objMax], rgy[objMax], rgz[objMax], zGlyph, zGlyph2,
+  int rgx[objMax], rgy[objMax], rgz[objMax], zGlyph,
     cChart, iChart, zr, xo = 0, yo = 0, zo = 0, xp, yp, zp, i, j, k, k2, nSav;
   flag fHouse3D = !us.fHouse3D, fNoHorizon;
   real rT;
   CONST CP *pcp;
   CP cpSav;
+  byte ignoreSav[objMax];
+  ES es;
 #ifdef SWISS
-  ES es, *pes1, *pes2;
+  ES *pes1, *pes2;
 #endif
 
   // Initialize variables.
-  if (gs.fText && !us.fVelocity)
+  if (gs.fText && gs.fDoSidebar)
     gs.xWin -= xSideT;
 
   fNoHorizon = ignorez[0] && ignorez[1] && ignorez[2] && ignorez[3];
-  zGlyph = 7*gi.nScale; zGlyph2 = 14*gi.nScale;
+  zGlyph = 7*gi.nScale;
   zr = Min(gs.xWin >> 1, gs.yWin >> 1) - zGlyph;
-  cChart = 1 - (FBetween(us.nRel, rcHexaWheel, rcDual) ? us.nRel : 0);
+  if (!(us.nRel <= rcTransit))
+    cChart = 1 - (FBetween(us.nRel, rcHexaWheel, rcDual) ? us.nRel : 0);
+  else {
+    cChart = 2;
+    CopyRgb(ignore, ignoreSav, sizeof(ignore));
+  }
 
   // Draw constellations.
   if (gs.fConstel) {
     neg(gs.rTilt);
-    WireDrawGlobe(fTrue, rDegMax - gs.rRot);
+    WireDrawGlobe(fTrue, gs.rRot);
     neg(gs.rTilt);
   }
 
@@ -2347,14 +2506,14 @@ void WireChartSphere()
         DrawColor(gi.kiOn);
       WireSphereLocal((real)i, 0.0, zr + k, &xp, &yp, &zp);
       sprintf(sz, "%c", rgszDir[j][0]);
-      gi.zDefault = zp + gi.nScale;
+      gi.zDefault = zp;
       DrawSz(sz, xp, yp, dtCent);
     }
     for (j = -90; j <= 90; j += nDegHalf) {
       WireSphereLocal(0.0, (real)j, zr + k, &xp, &yp, &zp);
       DrawColor(gs.fColorHouse ? gi.kiOn : (kObjB[j <= 0 ? oMC : oNad]));
       sprintf(sz, "%c", j <= 0 ? 'Z' : 'N');
-      gi.zDefault = zp + gi.nScale;
+      gi.zDefault = zp;
       DrawSz(sz, xp, yp, dtCent);
     }
   }
@@ -2390,9 +2549,21 @@ void WireChartSphere()
   }
 #endif
 
+  // Draw exoplanets.
+  if (gs.fAllExo) {
+    EnumExoplanets(NULL);
+    while (EnumExoplanets(&es)) {
+      EquToEcl(&es.lon, &es.lat);
+      WireSphereZodiac(es.lon, es.lat, zr, &xp, &yp, &zp);
+      WireStar(xp, yp, zp, &es);
+    }
+  }
+
   // Determine set of planet data to use.
   for (iChart = 1; iChart <= cChart; iChart++) {
     FProcessCommandLine(szWheelX[iChart]);
+    if (iChart == 2 && us.nRel <= rcTransit)
+      CopyRgb(ignore2, ignore, sizeof(ignore));
     if (iChart <= 1)
       pcp = rgpcp[us.nRel <= rcDual];
     else
@@ -2400,27 +2571,34 @@ void WireChartSphere()
     cpSav = cp0;
     cp0 = *pcp;
 
-  // Draw planet glyphs, and spots for actual local location.
-  for (i = 0; i <= is.nObj; i++) if (FProper(i)) {
-    WireSphereZodiac(planet[i], planetalt[i], zr, &xp, &yp, &zp);
-    rgx[i] = xp; rgy[i] = yp; rgz[i] = zp;
-    gi.zDefault = rgz[i] - zGlyph2;
-    DrawObject(i, rgx[i], rgy[i]);
-    DrawColor(kObjB[i]);
-    WireOctahedron(rgx[i], rgy[i], rgz[i], gi.nScale);
-  }
+    // Draw planet glyphs, and spots for actual local location.
+    for (i = 0; i <= is.nObj; i++) if (FProper(i)) {
+      WireSphereZodiac(planet[i], planetalt[i], zr, &xp, &yp, &zp);
+      rgx[i] = xp; rgy[i] = yp; rgz[i] = zp;
+      gi.zDefault = rgz[i] - zGlyph;
+      DrawObject(i, rgx[i], rgy[i]);
+      DrawColor(kObjB[i]);
+      WireOctahedron(rgx[i], rgy[i], rgz[i], gi.nScale);
+    }
 
-  // Draw lines connecting planets which have aspects between them.
-  if (!FCreateGrid(fFalse))
-    return;
-  for (j = is.nObj; j >= 1; j--)
-    for (i = j-1; i >= 0; i--)
-      if (grid->n[i][j] && FProper(i) && FProper(j)) {
-        DrawColor(kAspB[grid->n[i][j]]);
-        WireLine(rgx[i], rgy[i], rgz[i], rgx[j], rgy[j], rgz[j]);
-      }
+    // Draw lines connecting planets which have aspects between them.
+    if (!FCreateGrid(fFalse))
+      return;
+    for (j = is.nObj; j >= 1; j--)
+      for (i = j-1; i >= 0; i--)
+        if (grid->n[i][j] && FProper(i) && FProper(j)) {
+          DrawColor(kAspB[grid->n[i][j]]);
+          WireLine(rgx[i], rgy[i], rgz[i], rgx[j], rgy[j], rgz[j]);
+          if (gs.fLabelAsp) {
+            gi.zDefault = (rgz[i] + rgz[j]) >> 1;
+            DrawAspect(grid->n[i][j],
+              (rgx[i] + rgx[j]) >> 1, (rgy[i] + rgy[j]) >> 1);
+          }
+        }
 
     cp0 = cpSav;
+    if (iChart == 2 && us.nRel <= rcTransit)
+      CopyRgb(ignoreSav, ignore, sizeof(ignore));
   } // iChart
   FProcessCommandLine(szWheelX[0]);
 
