@@ -1,8 +1,8 @@
 /*
-** Astrolog (Version 7.60) File: xscreen.cpp
+** Astrolog (Version 7.80) File: xscreen.cpp
 **
 ** IMPORTANT NOTICE: Astrolog and all chart display routines and anything
-** not enumerated below used in this program are Copyright (C) 1991-2023 by
+** not enumerated below used in this program are Copyright (C) 1991-2025 by
 ** Walter D. Pullen (Astara@msn.com, http://www.astrolog.org/astrolog.htm).
 ** Permission is granted to freely use, modify, and distribute these
 ** routines provided these credits and notices remain unmodified with any
@@ -10,8 +10,8 @@
 **
 ** The main ephemeris databases and calculation routines are from the
 ** library SWISS EPHEMERIS and are programmed and copyright 1997-2008 by
-** Astrodienst AG. The use of that source code is subject to the license for
-** Swiss Ephemeris Free Edition, available at http://www.astro.com/swisseph.
+** Astrodienst AG. Use of that source code is subject to license for Swiss
+** Ephemeris Free Edition at https://www.astro.com/swisseph/swephinfo_e.htm.
 ** This copyright notice must not be changed or removed by any user of this
 ** program.
 **
@@ -48,7 +48,7 @@
 ** Initial programming 8/28-30/1991.
 ** X Window graphics initially programmed 10/23-29/1991.
 ** PostScript graphics initially programmed 11/29-30/1992.
-** Last code change made 4/8/2023.
+** Last code change made 6/19/2025.
 */
 
 #include "astrolog.h"
@@ -62,9 +62,9 @@
 */
 
 #ifdef X11
-// This information used to define Astrolog's X icon (ringed planet with
-// moons) is similar to the output format used by the bitmap program. You
-// could extract this section and run "xsetroot -bitmap" on it.
+// This information used to define Astrolog's X Windows icon (ringed planet
+// with moons) is similar to the output format used by the bitmap program.
+// You could extract this section and run "xsetroot -bitmap" on it.
 
 #define icon_width 48
 #define icon_height 48
@@ -268,7 +268,7 @@ LRESULT API WndProcWCLI(HWND hwnd, UINT wMsg, WPARAM wParam, LPARAM lParam)
     }
     hdc = GetDC(hwnd);
     hpen = (HPEN)CreatePen(PS_SOLID, !gs.fThick ? 0 : 2,
-      (COLORREF)rgbbmp[wi.kiPen]);
+      (COLORREF)rgbbmp[gi.kiPen]);
     hpenOld = (HPEN)SelectObject(hdc, hpen);
 
     // Ctrl+click means draw a rectangle. Ctrl+Shift+click does ellipse.
@@ -291,7 +291,7 @@ LRESULT API WndProcWCLI(HWND hwnd, UINT wMsg, WPARAM wParam, LPARAM lParam)
 
     // A simple click means set a pixel and remember that location.
     } else {
-      SetPixel(hdc, x, y, (COLORREF)rgbbmp[wi.kiPen]);
+      SetPixel(hdc, x, y, (COLORREF)rgbbmp[gi.kiPen]);
       wi.xMouse = x; wi.yMouse = y;
     }
     SelectObject(hdc, hpenOld);
@@ -455,7 +455,7 @@ void Animate(int mode, int toadd)
   }
 
   mode = NAbs(mode);
-  if (mode >= 10) {
+  if (mode == iAnimNow) {
 #ifdef TIME
     // For the continuous chart update to present moment animation mode, go
     // get whatever time it is now.
@@ -483,7 +483,7 @@ LNotNow:
     ciTran = ciCore;
     ciCore = ciMain;
     if (us.fProgress)
-      is.JDp = MdytszToJulian(MonT, DayT, YeaT, TimT, us.dstDef, us.zonDef);
+      is.JDp = MdytszToJulian(MonT, DayT, YeaT, TimT, ciDefa.dst, ciDefa.zon);
   } else
     ciMain = ciCore;
   if (us.nRel)
@@ -497,7 +497,7 @@ LNotNow:
 // This routine exits graphics mode, prompts the user for a set of command
 // switches, processes them, and returns to the previous graphics with the
 // new settings in effect, allowing one to change most any setting without
-// having to lose their graphics state or fall way back to a -Q loop.
+// having to lose their graphics state or fall back to a -Q loop.
 
 void CommandLineX()
 {
@@ -524,11 +524,7 @@ void CommandLineX()
   is.fSzInteract = fFalse;
   us.fLoop = fT;
   ciMain = ciCore;
-#ifndef WCLI
-  BeginX();
-#else
   InitColorsX();
-#endif
 }
 #endif // WIN
 
@@ -607,7 +603,7 @@ void InteractX()
   KeySym keysym;
 #endif
 #ifdef WCLI
-  HBITMAP hbmp, hbmpOld;
+  HBITMAP hbmpOld;
   HDC hdcWin;
   PAINTSTRUCT ps;
   MSG msg;
@@ -616,7 +612,6 @@ void InteractX()
   int fAutosize = fFalse, fResize = fFalse, fRedraw = fTrue, fNoChart = fFalse,
     fBreak = fFalse, fCast = fFalse, mousex = -1, mousey = -1,
     buttonx = -1, buttony = -1, length, key, i;
-  KI coldrw = gi.kiLite;
 
   neg(gs.nAnim);
   while (!fBreak) {
@@ -751,8 +746,8 @@ void InteractX()
       ClearB((pbyte)&ps, sizeof(PAINTSTRUCT));
       hdcWin = BeginPaint(wi.hwnd, &ps);
       wi.hdc = CreateCompatibleDC(hdcWin);
-      hbmp = CreateCompatibleBitmap(hdcWin, wi.xClient, wi.yClient);
-      hbmpOld = (HBITMAP)SelectObject(wi.hdc, hbmp);
+      wi.hbmp = CreateCompatibleBitmap(hdcWin, wi.xClient, wi.yClient);
+      hbmpOld = (HBITMAP)SelectObject(wi.hdc, wi.hbmp);
       if (gs.fJetTrail)
         BitBlt(wi.hdc, 0, 0, wi.xClient, wi.yClient, hdcWin, 0, 0, SRCCOPY);
       SetWindowOrg(wi.hdc, 0, 0);
@@ -779,7 +774,7 @@ void InteractX()
       BitBlt(hdcWin, 0, 0, wi.xClient, wi.yClient,
         wi.hdc, 0, 0, SRCCOPY);
       SelectObject(wi.hdc, hbmpOld);
-      DeleteObject(hbmp);
+      DeleteObject(wi.hbmp);
       DeleteDC(wi.hdc);
       EndPaint(wi.hwnd, &ps);
 #endif
@@ -840,7 +835,7 @@ void InteractX()
 
       // Check for user dragging any of the mouse buttons across window.
       case MotionNotify:
-        DrawColor(coldrw);
+        DrawColor(gi.kiPen);
         DrawLine(mousex, mousey, xevent.xbutton.x, xevent.xbutton.y);
         XSync(gi.disp, 0);
         XCopyArea(gi.disp, gi.pmap, gi.wind, gi.gc,
@@ -870,7 +865,6 @@ void InteractX()
             fRedraw = fTrue;
             break;
           default:
-            wi.kiPen = coldrw;
             DispatchMessage(&msg);
           }
         } else {
@@ -888,6 +882,11 @@ void InteractX()
           }
 #endif
           switch (key) {
+          case -1:                    // In case ~XQ returns -1
+            break;
+          case -2:
+            fResize = fCast = fTrue;  // Special ~XQ return value
+            break;
           case ' ':
             fRedraw = fTrue;
             break;
@@ -1100,7 +1099,7 @@ void InteractX()
             break;
 #endif
           case 'N':                        // The continuous update animation.
-            gs.nAnim = gs.nAnim ? 0 : -10;
+            gs.nAnim = gs.nAnim ? 0 : -iAnimNow;
             break;
 
           // These are the nine different "add time to chart" animations.
@@ -1179,10 +1178,8 @@ void InteractX()
             us.nScrollRow = 0;
             PrintL();
             if (key == 'v') {
-              if (us.nRel < rcNone)
-                ChartListingRelation();
-              else
-                ChartListing();
+              is.fMult = fFalse;
+              PrintChart(us.fProgress);
             } else
               DisplayKeysX();
             us.nScrollRow = length;
@@ -1194,22 +1191,22 @@ void InteractX()
           case chDelete:
             fRedraw = fNoChart = fTrue;
             break;
-          case 'z'-'`': coldrw = kBlack;   break;
-          case 'e'-'`': coldrw = kMaroon;  break;
-          case 'f'-'`': coldrw = kDkGreen; break;
-          case 'o'-'`': coldrw = kOrange;  break;
-          case 'n'-'`': coldrw = kDkBlue;  break;
-          case 'u'-'`': coldrw = kPurple;  break;
-          case 'k'-'`': coldrw = kDkCyan;  break;
-          case 'l'-'`': coldrw = kLtGray;  break;
-          case 'd'-'`': coldrw = kDkGray;  break;
-          case 'r'-'`': coldrw = kRed;     break;
-          case 'g'-'`': coldrw = kGreen;   break;
-          case 'y'-'`': coldrw = kYellow;  break;
-          case 'b'-'`': coldrw = kBlue;    break;
-          case 'v'-'`': coldrw = kMagenta; break;  // Ctrl+m is Enter
-          case 'j'-'`': coldrw = kCyan;    break;
-          case 'a'-'`': coldrw = kWhite;   break;
+          case 'z'-'`': gi.kiPen = kBlack;   break;
+          case 'e'-'`': gi.kiPen = kMaroon;  break;
+          case 'f'-'`': gi.kiPen = kDkGreen; break;
+          case 'o'-'`': gi.kiPen = kOrange;  break;
+          case 'n'-'`': gi.kiPen = kDkBlue;  break;
+          case 'u'-'`': gi.kiPen = kPurple;  break;
+          case 'k'-'`': gi.kiPen = kDkCyan;  break;
+          case 'l'-'`': gi.kiPen = kLtGray;  break;
+          case 'd'-'`': gi.kiPen = kDkGray;  break;
+          case 'r'-'`': gi.kiPen = kRed;     break;
+          case 'g'-'`': gi.kiPen = kGreen;   break;
+          case 'y'-'`': gi.kiPen = kYellow;  break;
+          case 'b'-'`': gi.kiPen = kBlue;    break;
+          case 'v'-'`': gi.kiPen = kMagenta; break;  // Ctrl+m is Enter
+          case 'j'-'`': gi.kiPen = kCyan;    break;
+          case 'a'-'`': gi.kiPen = kWhite;   break;
           case chEscape: case chBreak:
             fBreak = fTrue;
             break;
@@ -1219,13 +1216,13 @@ void InteractX()
               gi.nDir = (gi.nDir > 0 ? 1 : -1)*(key-'0');
               break;
             } else if (FBetween(key, 201, 248)) {
-              is.fSzInteract = fTrue;
-              if (szMacro[key-201]) {
-                FProcessCommandLine(szMacro[key-201]);
+              if (is.rgszMacro != NULL && is.rgszMacro[key-200]) {
+                is.fSzInteract = fTrue;
+                FProcessCommandLine(is.rgszMacro[key-200]);
+                is.fSzInteract = fFalse;
                 fResize = fCast = fTrue;
+                break;
               }
-              is.fSzInteract = fFalse;
-              break;
             }
             putchar(chBell);    // Any key not bound will sound a beep.
           } // switch
@@ -1268,6 +1265,54 @@ void EndX()
 ******************************************************************************
 */
 
+#ifdef SWISS
+// Process an instance of the -YXU command switch.
+
+flag FProcessYXU(CONST char *szLin, CONST char *szLnk, flag fAdd)
+{
+  char *pch;
+
+  // Allocate or extend allocation of star name list.
+  pch = (char *)PAllocate((fAdd ? CchSz(gs.szStarsLin) + 1 : 0) +
+    CchSz(szLin) + 1, "star name list");
+  if (pch == NULL)
+    return fFalse;
+  if (fAdd)
+    sprintf(pch, "%s;%s", gs.szStarsLin, szLin);
+  else
+    sprintf(pch, "%s", szLin);
+  DeallocatePIf(gs.szStarsLin);
+  gs.szStarsLin = pch;
+
+  // Allocate or extend allocation of star link list.
+  pch = (char *)PAllocate((fAdd ? CchSz(gs.szStarsLnk) + 1 : 0) +
+    CchSz(szLnk) + 2, "star link list");
+  if (pch == NULL)
+    return fFalse;
+  if (fAdd)
+    sprintf(pch, "%s;%s", gs.szStarsLnk, szLnk);
+  else
+    sprintf(pch, "%s", szLnk);
+  DeallocatePIf(gs.szStarsLnk);
+  gs.szStarsLnk = pch;
+
+  // Count total number of star names present, and reserve that many slots.
+  gi.cStarsLin = *gs.szStarsLin != chNull;
+  for (pch = gs.szStarsLin; *pch; pch++)
+    if (*pch == chSep || *pch == chSep2)
+      gi.cStarsLin++;
+  if (gi.rges != NULL) {
+    DeallocateP(gi.rges);
+    gi.rges = NULL;
+  }
+  gi.rges = RgAllocate(gi.cStarsLin, ES, "extra stars");
+  if (gi.rges == NULL)
+    gi.cStarsLin = 0;
+  return fTrue;
+}
+#endif
+
+
 // Process one command line switch passed to the program dealing with the
 // graphics features. This is just like the processing of each switch in the
 // main program, however here each switch has been prefixed with an 'X'.
@@ -1319,7 +1364,7 @@ int NProcessSwitchesX(int argc, char **argv, int pos,
       if (FErrorArgc("XM", argc, i))
         return tcError;
       for (j = 1; j <= i; j++)
-        szWheelX[(ch2 == '0' && j >= i) ? 0 : j] = SzPersist(argv[j]);
+        FCloneSz(argv[j], &szWheelX[(ch2 == '0' && j >= i) ? 0 : j]);
       darg += i;
       break;
     }
@@ -1360,7 +1405,7 @@ int NProcessSwitchesX(int argc, char **argv, int pos,
       return tcError;
     if (gs.ft == ftNone)
       gs.ft = ftBmp;
-    gi.szFileOut = SzPersist(argv[1]);
+    FCloneSz(argv[1], &gi.szFileOut);
     darg++;
     break;
 
@@ -1472,6 +1517,10 @@ int NProcessSwitchesX(int argc, char **argv, int pos,
     break;
 
   case 'x':
+    if (ch1 == '0') {
+      SwitchF(gs.fAntialias);
+      break;
+    }
     SwitchF(gs.fThick);
     break;
 
@@ -1492,6 +1541,10 @@ int NProcessSwitchesX(int argc, char **argv, int pos,
     break;
 
   case 'U':
+    if (ch1 == 'x') {
+      SwitchF(gs.fAllExo);
+      break;
+    }
     SwitchF(gs.fAllStar);
     if (FBetween(ch1, '0', '3'))
       gs.nAllStar = (ch1 - '0');
@@ -1555,12 +1608,16 @@ int NProcessSwitchesX(int argc, char **argv, int pos,
     }
     if (FErrorArgc("Xd", argc, 1))
       return tcError;
-    gs.szDisplay = SzPersist(argv[1]);
+    FCloneSz(argv[1], &gs.szDisplay);
     darg++;
     break;
 #endif
 
   case 'v':
+    if (ch1 == '0') {
+      SwitchF(gs.fDoSidebar);
+      break;
+    }
     if (FErrorArgc("Xv", argc, 1))
       return tcError;
     i = NFromSz(argv[1]);
@@ -1667,6 +1724,13 @@ int NProcessSwitchesX(int argc, char **argv, int pos,
 #endif
 
 #ifdef ISG
+  case 'k':
+    if (FErrorArgc("Xk", argc, 1))
+      return tcError;
+    gi.kiPen = NParseSz(argv[1], pmColor);
+    darg++;
+    break;
+
   case 'n':
     if (ch1 == 'p') {
       SwitchF(gi.fPause);
@@ -1685,9 +1749,9 @@ int NProcessSwitchesX(int argc, char **argv, int pos,
       darg++;
     else
       i = 10;
-    if (FErrorValN("Xn", !FBetween(i, 1, 10), i, 0))
+    if (FErrorValN("Xn", !FBetween(i, 1, 13), i, 0))
       return tcError;
-    gs.nAnim = fAnd ? -i : i;
+    gs.nAnim = (fOr || fNot ? i : -i);
     break;
 
   case 'N':
@@ -1713,10 +1777,13 @@ int NProcessSwitchesRareX(int argc, char **argv, int pos,
 {
   int darg = 0, i, j;
   real rT;
-  char ch1, *pch = NULL;
+  char ch1;
 #ifdef SWISS
   flag fAdd;
+#ifdef CONSTEL
+  CONST char **ppch;
 #endif
+#endif // SWISS
 
   ch1 = argv[0][pos+1];
   switch (argv[0][pos]) {
@@ -1788,12 +1855,14 @@ int NProcessSwitchesRareX(int argc, char **argv, int pos,
       j = NParseSz(argv[2], pmObject);
       if (FErrorValN("YXDD", !FItem(j), j, 2))
         return tcError;
-      szDrawObject[i]  = szDrawObject[j];
-      szDrawObject2[i] = szDrawObject2[j];
+      FCloneSz(szDrawObject[j], (char **)&szDrawObject[i]);
+      FCloneSz(szDrawObject2[j], (char **)&szDrawObject2[i]);
     } else {
-      szDrawObject[i] = argv[2][0] ? SzPersist(argv[2]) : szDrawObjectDef[i];
-      szDrawObject2[i] = (ch1 == '1' ? "" : (argv[3][0] ? SzPersist(argv[3]) :
-        szDrawObjectDef2[i]));
+      FCloneSzCore(argv[2][0] ? argv[2] : szDrawObjectDef[i],
+        (char **)&szDrawObject[i], szDrawObject[i] == szDrawObjectDef[i]);
+      FCloneSzCore(
+        ch1 == '1' ? "" : (argv[3][0] ? argv[3] : szDrawObjectDef2[i]),
+        (char **)&szDrawObject2[i], szDrawObject2[i] == szDrawObjectDef2[i]);
     }
     darg += 3 - (ch1 == '1' || ch1 == 'D');
     break;
@@ -1804,9 +1873,11 @@ int NProcessSwitchesRareX(int argc, char **argv, int pos,
     i = NParseSz(argv[1], pmAspect);
     if (FErrorValN("YXA", !FAspect3(i), i, 0))
       return tcError;
-    szDrawAspect[i] = argv[2][0] ? SzPersist(argv[2]) : szDrawAspectDef[i];
-    szDrawAspect2[i] = (ch1 == '1' ? "" : (argv[3][0] ? SzPersist(argv[3]) :
-      szDrawAspectDef2[i]));
+    FCloneSzCore(argv[2][0] ? argv[2] : szDrawAspectDef[i],
+      (char **)&szDrawAspect[i], szDrawAspect[i] == szDrawAspectDef[i]);
+    FCloneSzCore(
+      ch1 == '1' ? "" : (argv[3][0] ? argv[3] : szDrawAspectDef2[i]),
+      (char **)&szDrawAspect2[i], szDrawAspect2[i] == szDrawAspectDef2[i]);
     darg += 3 - (ch1 == '1');
     break;
 
@@ -1837,7 +1908,7 @@ int NProcessSwitchesRareX(int argc, char **argv, int pos,
   case 't':
     if (FErrorArgc("YXt", argc, 1))
       return tcError;
-    gs.szSidebar = SzPersist(argv[1]);
+    FCloneSz(argv[1], &gs.szSidebar);
     darg++;
     break;
 
@@ -1926,48 +1997,23 @@ int NProcessSwitchesRareX(int argc, char **argv, int pos,
 
 #ifdef SWISS
   case 'U':
-    if (FErrorArgc("YXU", argc, 2))
-      return tcError;
-    fAdd = (ch1 == '0' && FSzSet(gs.szStarsLin) && FSzSet(gs.szStarsLnk));
-    // Allocate or extend allocation of star name list.
-    pch = (char *)PAllocate((fAdd ? CchSz(gs.szStarsLin) + 1 : 0) +
-      CchSz(argv[1]) + 1, "star name list");
-    if (pch == NULL)
-      return tcError;
-    if (fAdd)
-      sprintf(pch, "%s;%s", gs.szStarsLin, argv[1]);
-    else
-      sprintf(pch, "%s", argv[1]);
-    if (gs.szStarsLin)
-      DeallocateP(gs.szStarsLin);
-    gs.szStarsLin = pch;
-    // Allocate or extend allocation of star link list.
-    pch = (char *)PAllocate((fAdd ? CchSz(gs.szStarsLnk) + 1 : 0) +
-      CchSz(argv[2]) + 2, "star link list");
-    if (pch == NULL)
-      return tcError;
-    if (fAdd)
-      sprintf(pch, "%s;%s", gs.szStarsLnk, argv[2]);
-    else
-      sprintf(pch, "%s", argv[2]);
-    if (gs.szStarsLnk)
-      DeallocateP(gs.szStarsLnk);
-    gs.szStarsLnk = pch;
-    // Count total number of star names present, and reserve that many slots.
-    gi.cStarsLin = *gs.szStarsLin != chNull;
-    for (pch = gs.szStarsLin; *pch; pch++)
-      if (*pch == chSep || *pch == chSep2)
-        gi.cStarsLin++;
-    if (gi.rges != NULL) {
-      DeallocateP(gi.rges);
-      gi.rges = NULL;
-    }
-    gi.rges = (ES *)PAllocate(gi.cStarsLin * sizeof(ES), "extra stars");
-    if (gi.rges == NULL)
-      gi.cStarsLin = 0;
-    darg += 2;
-    break;
+    if (ch1 != '1') {
+      if (FErrorArgc("YXU", argc, 2))
+        return tcError;
+      fAdd = (ch1 == '0' && FSzSet(gs.szStarsLin) && FSzSet(gs.szStarsLnk));
+      if (!FProcessYXU(argv[1], argv[2], fAdd))
+        return tcError;
+      darg += 2;
+    } else {
+#ifdef CONSTEL
+      for (ppch = szDrawConstelLine; *ppch != NULL; ppch += 2) {
+        if (!FProcessYXU(ppch[0], ppch[1], ppch != szDrawConstelLine))
+          return tcError;
+      }
 #endif
+    }
+    break;
+#endif // SWISS
 
   case 'f':
     if (FErrorArgc("YXf", argc, 1))
